c-----------------------------------------------------------------------
c --- INFOVIEW -- Summarizes Selected Model Output Files
c-----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 110301              MAIN
c
c     Copyright (c) 2008-2011 by Exponent, Inc.         
c 
c               D. Strimaitis, TRC Environmental Corp.
c
c --- PURPOSE: Examines and reports selected header information from
c              model output files.  Models currently supported:
c              CALMET  (CALMET.DAT)
c              CALPUFF (CONC.DAT)
c              MesoMet (3D.DAT/2D.DAT)
c                       --- MM5, RUC, WRF, RAMS
c              WIND1D/WIND2D (*.W1D,*.W2D)
c                       --- Direct-Access 1-layer wind field
c
c --- UPDATES:
c
c --- V1.51 (L100331) to V1.6 (L110301) DGS
c            - Add CALPUFF Dataset V2.2 header
c            Modified:  COMP.IVU
c                       COMP_PUF
c
c --- V1.5 (L090729) to V1.51 (L100331) DGS
c            - Control file image in CALMET.DAT Dataset V2.0 header
c              was not processed
c            Modified:  COMP_MET
c
c --- V1.41 (L090413) to V1.5 (L090729) DGS
c            - Add 2D.DAT files (processed almost like 3D.DAT)
c            Modified:  CTLDAT.IVU
c                       SETUP, COMP_3D, GETVER
c
c --- V1.4 (L090410) to V1.41 (L090413) DGS
c            - Allow repetition factor in CALMET height-arrays
c            Modified:  RDMETCF
c
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Increase the grid size parameters to 1000x1000
c            - Add new variables and trap obvious cases in which the
c              input file is not the type that is indicated
c            - Modify column processing to accept fewer than expected
c              number of entries (array values), and assign zero
c              to all values that are missing -- this matches the
c              way CALMET reads these entries
c            - Read first 2 times in 3D.DAT file to discover the
c              timestep and use this to compute the end-time
c            - Initiallize all variables that go into the DLL common
c            NEW:       DLLINIT
c            Modified:  CPUFF.IVU, CMET.IVU, WDA.IVU
c                       CTLDAT.IVU, PASSDLL.IVU
c                       SETUP, COMP_MET, COMP_PUF,  GETDOC_CP
c                       COLUMN320, RDMETCF, RDPUFFCF
c                       COMP_3D, COMP_WDA
c
c --- V1.2 (L090312) to V1.3 (L090317) DGS
c            - Create CSV as well as TXT files with output
c            - Process control file image from header of
c              CALMET and CALPUFF output files
c              (taken from CALEXAM)
c
c --- V1.1 (L090310) to V1.2 (L090312) DGS
c            - Implement W1D_W2D input file option
c
c --- V1.0 (L090302) to V1.1 (L090310) DGS
c            - Implement 3D.DAT input file option
c            - Add met parameters to CALMET output
c            - Create begin and end date-times, and projection strings
c              for DLL output
c            - Fixed output file name option
c
c
c --- INFOVIEW calls:     SETUP, COMP_PUFF, COMP_MET, COMP_3D
c                         COMP_WDA
c
c-----------------------------------------------------------------------
      Program INFOVIEW
c-----------------------------------------------------------------------

      include 'params.ivu'
      include 'err.cal'
      include 'ctldat.ivu'
      include 'passdll.ivu'

      character*12 ver,level

c --- set version and level number of program
      ver='1.6'
      level='110301'

c --- Initialize return variables
      iret=0
      estrng=' '

c --- Setup phase -- initialization & program setup operations
      call SETUP
      if(iret.NE.0) goto 100

c --- Computational phase
      if(mfiletype.EQ.1) call COMP_MET
      if(mfiletype.EQ.2) call COMP_PUF
      if(mfiletype.EQ.3) call COMP_3D
      if(mfiletype.EQ.4) call COMP_WDA
      if(mfiletype.EQ.5) call COMP_3D

c --- Process return code into /PASSDLL/
100   kreturn=iret
      kmessage=' '
      kmessage(1:80)=estrng

cc --- TEST
c      write(iout,*)
c      write(iout,*)'/PASSDLL/'
c      write(iout,*)'Return Code: ',kreturn
c      write(iout,*)'Message: ',TRIM(kmessage)
      write(*,*)'Return Code: ',kreturn
      write(*,*)'Message: ',TRIM(kmessage)
c      write(iout,*)'Start: ',kbyr,kbjul,kbmon,kbday,kbhr,kbsec
c      write(iout,*)'End  : ',keyr,kejul,kemon,keday,kehr,kesec
c      write(iout,*)'Zone : ',ktzone
c      write(iout,*)'map,datum : ',kpmap,kdatum
c      write(iout,*)'utm  : ',kutmzn,kutmhem
c      write(iout,*)'feast,fnorth : ',xfeast,xfnorth
c      write(iout,*)'nlatc,elonc  : ',xnlatc,xelonc
c      write(iout,*)'nlat1,nlat2  : ',xnlat1,xnlat2

c --- Termination phase -- program termination functions
      close(in)
      close(iout)
      close(icsv)

      stop
      end

c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6      Level: 090317              BLOCKDATA
c ---           D. Strimaitis
c
c----------------------------------------------------------------------
c --- NOTE:
c --- This is for setting up controls for processing the control file
c --- image from the CALMET and CALPUFF headers
c----------------------------------------------------------------------

c --- Include parameter statements & common blocks
      include 'cfdoc.ivu'

c --- OPTION common block
      data npuffopt/29/
      data cpuffopt/
     1  'MGAUSS  ','MSLUG   ','MCHEM   ','MAQCHEM ','MWET    ',
     2  'MDRY    ','MTILT   ','MDISP   ','MDISP2  ','MPDF    ',
     3  'MTURBVW ','MTAULY  ','MTAUADV ','MCTURB  ','MROUGH  ',
     4  'MBDW    ','MTRANS  ','MTIP    ','MSHEAR  ','MPARTL  ',
     5  'MTINV   ','MCTADJ  ','MCTSG   ','MSPLIT  ','MSGTIBL ',
     6  'MBCON   ','MSOURCE ','MFOG    ','MREG    ','        ',
     7  70*'        '/

      data nmetopt/34/
      data cmetopt/
     1  'NOOBS   ','ICLOUD  ','ITPROG  ','ITWPROG ','IRHPROG ',
     2  'IPROG   ','IGFMET  ','IDIOPT1 ','ISURFT  ','IDIOPT2 ',
     3  'IUPT    ','IDIOPT3 ','IUPWND  ','IDIOPT4 ','IDIOPT5 ',
     4  'IEXTRP  ','ICALM   ','LVARY   ','NBAR    ','NFLAGP  ',
     5  'IWFCOD  ','IFRADJ  ','IKINE   ','IOBR    ','ISLOPE  ',
     6  'IMIXH   ','ICOARE  ','IWARM   ','ICOOL   ','LLBREZE ',
     7  'IAVEZI  ','IAVET   ','IRTYPE  ','LCALGRD ','        ',
     8  65*'        '/

c --- PARVAL common block

      data nmetpvl/95/

c Data statement ordered for tabular output by category:
c ---------------------------------------------------------------------
c Critical Wind Field Settings
c       'TERRAD  ','IEXTRP  ','ICALM   ','RMAX1   ','RMAX2   ',
c       'RMAX3   ','R1      ','R2      ',
c Vertical Layers
c       'ZFACE   ','BIAS    ','NSMTH   ','NINTR2  ','FEXTR2  ',
c Primary Configuration
c - Data Choices
c       'NOOBS   ','ICLOUD  ','ITPROG  ','ITWPROG ','IRHPROG ',
c       'IPROG   ','IGFMET  ','ISURFT  ','IUPT    ','IUPWND  ',
c       'NSSTA   ','NUSTA   ','NOWSTA  ','NPSTA   ','NM3D    ',
c       'NIGF    ',
c - Grid
c       'NZ      ','NX      ','NY      ','DGRIDKM ','XORIGKM ',
c       'YORIGKM ',
c Methods Choices
c - Winds
c   -- Wind Field Modules
c       'IWFCOD  ','IFRADJ  ','IKINE   ','IOBR    ','ISLOPE  ',
c       'ISTEPPGS',
c   -- Spatial Features
c       'NBAR    ','KBAR    ','LLBREZE ',
c - Mixing Height
c       'IMIXH   ','THRESHL ','THRESHW ','ILUOC3D ',
c       'IAVEZI  ','MNMDAV  ','HAFANG  ','ILEVZI  ',
c       'ZIMIN   ','ZIMAX   ','ZIMINW  ','ZIMAXW  ',
c       'DPTMIN  ','DZZI    ',
c - Temperature
c       'IRAD    ','TRADKM  ','NUMTS   ','JWAT1   ','JWAT2   ',
c       'IAVET   ',
c - Precipitation
c       'NFLAGP  ','SIGMAP  ','CUTP    ',
c - Overwater Fluxes
c       'ICOARE  ','IWARM   ','ICOOL   ','DSHELF  ',
c Default Selections Normally Retained
c       'LVARY   ','RMIN    ','RMIN2   ','RPROG   ',
c       'CRITFN  ','TGDEFB  ','TGDEFA  ','ZUPT    ','ZUPWND  ',
c       'IRTYPE  ','LCALGRD ','DIVLIM  ','NITER   ','ALPHA   ',
c       'CONSTB  ','CONSTE  ','CONSTN  ','CONSTW  ','FCORIOL ',
c       'IDIOPT1 ','IDIOPT2 ','IDIOPT3 ','IDIOPT4 ','IDIOPT5 ',
c ---------------------------------------------------------------------

      data cmetpvl/
     1  'TERRAD  ','IEXTRP  ','ICALM   ','RMAX1   ','RMAX2   ',
     2  'RMAX3   ','R1      ','R2      ','ZFACE   ','BIAS    ',
     3  'NSMTH   ','NINTR2  ','FEXTR2  ','NOOBS   ','ICLOUD  ',
     4  'ITPROG  ','ITWPROG ','IRHPROG ','IPROG   ','IGFMET  ',
     5  'ISURFT  ','IUPT    ','IUPWND  ','NSSTA   ','NUSTA   ',
     6  'NOWSTA  ','NPSTA   ','NM3D    ','NIGF    ','NZ      ',
     7  'NX      ','NY      ','DGRIDKM ','XORIGKM ','YORIGKM ',
     8  'IWFCOD  ','IFRADJ  ','IKINE   ','IOBR    ','ISLOPE  ',
     9  'ISTEPPGS','NBAR    ','KBAR    ','LLBREZE ','IMIXH   ',
     *  'THRESHL ','THRESHW ','ILUOC3D ','IAVEZI  ','MNMDAV  ',
     1  'HAFANG  ','ILEVZI  ','ZIMIN   ','ZIMAX   ','ZIMINW  ',
     2  'ZIMAXW  ','DPTMIN  ','DZZI    ','IRAD    ','TRADKM  ',
     3  'NUMTS   ','JWAT1   ','JWAT2   ','IAVET   ','NFLAGP  ',
     4  'SIGMAP  ','CUTP    ','ICOARE  ','IWARM   ','ICOOL   ',
     5  'DSHELF  ','LVARY   ','RMIN    ','RMIN2   ','RPROG   ',
     6  'CRITFN  ','TGDEFB  ','TGDEFA  ','ZUPT    ','ZUPWND  ',
     7  'IRTYPE  ','LCALGRD ','DIVLIM  ','NITER   ','ALPHA   ',
     8  'CONSTB  ','CONSTE  ','CONSTN  ','CONSTW  ','FCORIOL ',
     9  'IDIOPT1 ','IDIOPT2 ','IDIOPT3 ','IDIOPT4 ','IDIOPT5 ',
     *  5*'        '/

c --- TIME common block
      data npufftim/16/
      data cpufftim/
     1  'IBYR  ','IBMO  ','IBDY  ','IBHR  ','IBMIN ','IBSEC ',
     2  'IEYR  ','IEMO  ','IEDY  ','IEHR  ','IEMIN ','IESEC ',
     3  'XBTZ  ','ABTZ  ','IRLG  ','NSECDT',
     4  4*'      '/

      data nmettim/14/
      data cmettim/
     1  'IBYR  ','IBMO  ','IBDY  ','IBHR  ','IBSEC ',
     2  'IEYR  ','IEMO  ','IEDY  ','IEHR  ','IESEC ',
     3  'IBTZ  ','ABTZ  ','IRLG  ','NSECDT',
     4  6*'      '/

c --- MAPC common block
      data npuffmap/10/
      data cpuffmap/
     1  'PMAP  ','DATUM ','IUTMZN','UTMHEM','RLAT0 ',
     2  'RLON0 ','XLAT1 ','XLAT2 ','FEAST ','FNORTH',
     3  10*'      '/
      data nmetmap/10/
      data cmetmap/
     1  'PMAP  ','DATUM ','IUTMZN','UTMHEM','RLAT0 ',
     2  'RLON0 ','XLAT1 ','XLAT2 ','FEAST ','FNORTH',
     3  10*'      '/
c --- Set corresponding pointers
      data kpmap/1/,kdatum/2/,kutmzn/3/,kutmhem/4/,krlat0/5/
      data krlon0/6/,kxlat1/7/,kxlat2/8/,keast/9/,knorth/10/

c --- RECEPTOR common block
      data npuffrec/2/
      data cpuffrec/
     1  'LSAMP ','NREC  ',
     7  18*'      '/

      end

c----------------------------------------------------------------------
c --- Bring in utility routines set up for DLL applications (ERR)
      include 'calutils_err.for'
c----------------------------------------------------------------------

c----------------------------------------------------------------------
      subroutine setup
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090729             SETUP
c               D. Strimaitis
c
c --- PURPOSE:  Determine application and open files
c
c --- UPDATES:
c --- V1.4 (L090410) to V1.5 (L090729) DGS
c            - Add 2D.DAT input file option
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Add MRUNTYPE flag
c --- V1.2 (L090312) to V1.3 (L090317) DGS
c            - Add scratch file for control file image
c --- V1.1 (L090310) to V1.2 (L090312) DGS
c            - Implement W1D_W2D input file option
c --- V1.0 (L090302) to V1.1 (L090310) DGS
c            - Implement 3D.DAT input file option
c
c --- SETUP called by:  (main)
c --- SETUP calls:      COMLINE, SRCINIT, RUNINIT
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'ctldat.ivu'
      include 'wda.ivu'
      include 'err.cal'

      character*132 a132


c --- Control file name may be provided on command line
      filecf=' '
      filecf='infoview.inp'
      call COMLINE(filecf)

c --- Process instructions from the control file
      open(ictl,file=filecf,status='old')

c --- First record is file type (CALMET, CALPUFF, M3D, M2D, or W1D_W2D)
      mfiletype=0
      a132=' '
      read(ictl,*) a132
c --- Test for CALMET
      i=INDEX(a132,'CALMET')
      if(i.GT.0) mfiletype=1
c --- Test for CALPUFF
      i=INDEX(a132,'CALPUFF')
      if(i.GT.0) mfiletype=2
c --- Test for M3D
      i=INDEX(a132,'M3D')
      if(i.GT.0) mfiletype=3
c --- Test for W1D_W2D
      i=INDEX(a132,'W1D_W2D')
      if(i.GT.0) mfiletype=4
c --- Test for M2D
      i=INDEX(a132,'M2D')
      if(i.GT.0) mfiletype=5

      if(mfiletype.EQ.0) then
         kl=LEN_TRIM(a132)
         write(*,*)'Bad file type: ',a132(1:kl)
         write(*,*)'Expected: CALMET, CALPUFF, M3D, M2D, W1D_W2D'
         stop
      endif

c --- Second record is file name
      a132=' '
      read(ictl,*) a132
      kl=LEN_TRIM(a132)
      fileinp(1:kl)=a132(1:kl)
      do n=kl+1,132
         fileinp(n:n)=' '
      enddo

c --- Third record is flag for default (from input name) or fixed
c --- name for FILEOUT
c ---   0 = Use default name = fileinp_HDR.txt (.csv)
c ---   1 = Use fixed name = infoview.txt (.csv)
      read(ictl,*) ioutflag

c --- Fourth record is flag for type of run
c ---   0 = Continue processing
c ---   1 = Halt with error message
      mruntype=0
      read(ictl,*,end=99) mruntype

99    close(ictl)

c --- Create output file name
      if(ioutflag.EQ.0) then
         fileout=fileinp(1:kl)//'_HDR.txt'
         filecsv=fileinp(1:kl)//'_HDR.csv'
      else
         fileout='infoview.txt'
         filecsv='infoview.csv'
      endif

c --- Open files
      open(iout,file=fileout,status='unknown')
      open(icsv,file=filecsv,status='unknown')
      if(mfiletype.EQ.1 .OR. mfiletype.EQ.2) then
c ---    Binary file
         open(in,file=fileinp,form='unformatted',status='old')
      elseif(mfiletype.EQ.3 .OR. mfiletype.EQ.5) then
c ---    ASCII file
         open(in,file=fileinp,form='formatted',status='old')
      elseif(mfiletype.EQ.4) then
c ---    Binary Direct Access file
c ---    Obtain record length
         open(in,file=fileinp,access='TRANSPARENT')
         read(in, err=101) krecl
c ---    Expect krecl of order 4*nx*ny bytes
         krecx=10*4*mxnx_da*mxny_da
         if(krecl.LE.0 .OR. krecl.GT.krecx) goto 101
         close(in)
c ---    Open file as Direct Access
         open(in,file=fileinp,access='DIRECT',
     &        form='UNFORMATTED',recl=krecl,
     &        iostat=istat,err=101)
         if(istat.NE.0) goto 101
      endif

c --- Actions to set up control file image processing
c ----------------------------------------------------
c --- Open scratch file for CALMET or CALPUFF control file image
      open(icfx,status='scratch')
c --- Initialize source data
      call SRCINIT
c --- Initialize model options
      call RUNINIT

      return

101   iret=1
      estrng='Invalid WIND1D or WIND2D file'
      return

      end

c----------------------------------------------------------------------
      subroutine comp_met
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 100331          COMP_MET
c               D. Strimaitis
c
c --- PURPOSE:  Read and report selected information from a CALMET
c               file header
c
c --- UPDATES:
c --- V1.4 (L090410) to V1.41 (L100331) DGS
c            - Save embedded control file image from CALMET.DAT Dataset
c              V2.0
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Add MRUNTYPE logic branches
c            NEW:     DLLINIT
c --- V1.1 (L090310) to V1.3 (L090317) DGS
c            - Add CALMET control file processing
c            - Add CSV output
c            NEW:     COMP_METCF
c --- V1.0 (L090302) to V1.1 (L090310) DGS
c            - Pass begin and end date-times, and projection data
c              for DLL output
c            NEW:     DLLARG
c
c --- INPUTS:
c
c --- OUTPUT:
c     Common block /HEAD_CM/ variables:
c
c --- COMP_MET called by:  (main)
c --- COMP_MET calls:      RDR1D, RDR2D, RDI2D, YR4
c                          INCR, GRDAY, JULDAY
c                          BASRUTC, UTCBASR, DLLARG, DLLINIT,
c                          COMP_METCF
c----------------------------------------------------------------------
c
c --- Include parameters

      include 'params.ivu'
      include 'cmet.ivu'
      include 'ctldat.ivu'
      include 'passdll.ivu'
      include 'err.cal'

      real xbuf(mxnx_cm,mxny_cm)
      integer ibuf(mxnx_cm,mxny_cm)

      character*70 messag
      character*8 clab1,clab2,vermet
      character*8 clabexp,clabex1,clabex2
      logical ldate,lecho

c --- Local Variables
      character*8 axbtzm
      character*16 dataset,dataver
      character*33 blank33
      character*64 datamod
      character*80 doc1
      character*132 comment1,blank
      character*3 cmonth(12)
      real zmid(mxnzp1_cm)
      logical lcfx

      character*1 cc,cm,ds,xp,dqt

c --- Set months
      data cmonth/'Jan','Feb','Mar','Apr','May','Jun',
     &            'Jul','Aug','Sep','Oct','Nov','Dec'/

      data cc/':'/
      data cm/','/
      data ds/'-'/
      data xp/'!'/
      data dqt/'"'/

      data ldate/.FALSE./, lecho/.FALSE./
      data blank33/'                                 '/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Initialize data used in DLL output argument array
      call DLLINIT(ibymet_cm,ibjdmet_cm,ibmmet_cm,ibdmet_cm,
     &            ibhmet_cm,ibsmet_cm,
     &            ieymet_cm,iejdmet_cm,iemmet_cm,iedmet_cm,
     &            iehmet_cm,iesmet_cm,
     &            rnlat0_cm,relon0_cm,xlat1_cm,xlat2_cm,feast_cm,
     &            fnorth_cm,iutmzn_cm,axbtz_cm,utmhem_cm,datum_cm,
     &            pmap_cm)

c --- Find version of dataset
c ---------------------------
      read(in, err=99, end=99) dataset,dataver,datamod
      if(dataset.NE.'CALMET.DAT') then
         if(mruntype.EQ.0) then
c ---       Process as Older CALMET dataset 
            REWIND(in)
c ---       record #1 - run title -- 60 words
            read(in)title_cm
c ---       record #2 - run control parameters
            read(in)vermet,levmet_cm
            read(levmet_cm(1:6),'(i6)') ilevmet
            read(vermet(1:6),'(f6.0)') rvermet
            if(ilevmet.GE.980304 .OR. rvermet.GT.5.1) then
               dataver='1.5'
            else
               dataver='1.0'
            endif
         elseif(mruntype.EQ.1) then
c ---       Dataset name does not match so return ERROR
            iret=1
            estrng='Input file Dataset Name is not CALMET.DAT: '//
     &              dataset
            return
         elseif(mruntype.EQ.2) then
c ---       Dataset name does not match so return first record
            REWIND(in)
            iret=-1
            read(in)title_cm(1)
            estrng=title_cm(1)
            return
         endif
      endif

c --- Set dataset version as a real  (not currently used)
      read(dataver,'(f16.0)') dversm_cm


c -------------------------------------------------------
c --- Start Output
c -------------------------------------------------------

c --- Type
      write(iout,'(//1x,a26)')'CALMET Model Configuration'
      write(iout,'(1x,a26/)') '--------------------------'
      write(icsv,'(//1x,a26)')'CALMET Model Configuration'
 
c --- File
      k=LEN_TRIM(fileinp)
      write(iout,'(a)')' File Examined: '//fileinp(1:k)
      write(iout,*)
      write(icsv,'(a)')dqt//' File Examined: '//fileinp(1:k)//dqt
      write(icsv,*)

c --- Titles
      write(iout,'(a)')' Titles:'
      write(icsv,'(a)')dqt//' Titles:'//dqt
      iohead=1

c --- Read records
c ----------------
      REWIND(in)

      if(dataver.EQ.'2.1') then
c ---   Dataset 2.1 with comment records and begin/end times
c ----------------------------------------------------------
        itimes_cm=1
        i2dmet_cm=1
c
c ---   Record #1 - File Declaration -- 24 words
        read(in) dataset,dataver,datamod
c
c ---   Record #2 - Number of comment lines -- 1 word
        read(in) ncom
c ---   Loop over comment records and harvest control file image
c ---   to scratch file
        lcfx=.TRUE.
        do i=1,ncom
          comment1=blank
          read(in) comment1
          if(LCFX) then
             if(comment1(1:12).EQ.'............') then
c ---           End of control file image
                lcfx=.FALSE.
             else
                write(icfx,'(a)')TRIM(comment1)
             endif
          endif
          if(i.EQ.1) then
c ---       Save model version line
            doc1=comment1(1:80)
          elseif(i.LE.4) then
c ---       Save 3 title lines
            title_cm(i-1)=comment1(1:80)
          endif

c ---     Output
          if(iohead.EQ.1) then
             ktxt=INDEX(comment1,'---------------')
             if(ktxt.EQ.0) then
                kl=LEN_TRIM(comment1)
                write(iout,'(a)')'  '//comment1(1:kl)
                write(icsv,'(a)')dqt//'  '//comment1(1:kl)//dqt
             else
                iohead=0
             endif
           endif

        enddo
c
c ---   record #NCOM+3 - run control parameters -- 39 words
        read(in)
     1   ibymet_cm,ibmmet_cm,ibdmet_cm,ibhmet_cm,ibsmet_cm,
     2   ieymet_cm,iemmet_cm,iedmet_cm,iehmet_cm,iesmet_cm,
     3   axbtz_cm,irlgmet_cm,irtype_cm,nxm_cm,nym_cm,nzm_cm,
     4   xgridm_cm,xorigm_cm,yorigm_cm,iwfcod_cm,nssta_cm,nusta_cm,
     5   npsta_cm,nowsta_cm,nlu_cm,iwat1_cm,iwat2_cm,lcalgrd_cm,
     6   pmap_cm,datum_cm,daten_cm,feast_cm,fnorth_cm,
     7   utmhem_cm,iutmzn_cm,
     8   rnlat0_cm,relon0_cm,xlat1_cm,xlat2_cm

        call YR4(iout,ibymet_cm,ierrb)
        call YR4(iout,ieymet_cm,ierre)
        if(ierrb.NE.0 .OR. ierre.NE.0) then
           iret=1
           return
        endif

        call JULDAY(iout,ibymet_cm,ibmmet_cm,ibdmet_cm,ibjdmet_cm)
        call JULDAY(iout,ieymet_cm,iemmet_cm,iedmet_cm,iejdmet_cm)
        if(iret.NE.0) return

c ---   Process character AXBTZ into real XBTZ
        call UTCBASR(axbtz_cm,xbtz_cm)

        rlat0_cm=rnlat0_cm
        rlon0_cm=-relon0_cm

      elseif(dataver.EQ.'2.0') then
c ---   Dataset 2.0 with comment records
c --------------------------------------
        itimes_cm=0
        i2dmet_cm=1
c
c ---   Record #1 - File Declaration -- 24 words
        read(in) dataset,dataver,datamod
c
c ---   Record #2 - Number of comment lines -- 1 word
        read(in) ncom
c ---   Loop over comment records and harvest control file image
c ---   to scratch file
        lcfx=.TRUE.
        do i=1,ncom
          comment1=blank
          read(in) comment1
          if(LCFX) then
             if(comment1(1:12).EQ.'............') then
c ---           End of control file image
                lcfx=.FALSE.
             else
                write(icfx,'(a)')TRIM(comment1)
             endif
          endif
          if(i.EQ.1) then
c ---       Save model version line
            doc1=comment1(1:80)
          elseif(i.LE.4) then
c ---       Save 3 title lines
            title_cm(i-1)=comment1(1:80)
          endif

c ---     Output
          if(iohead.EQ.1) then
             ktxt=INDEX(comment1,'---------------')
             if(ktxt.EQ.0) then
                kl=LEN_TRIM(comment1)
                write(iout,'(a)')'  '//comment1(1:kl)
                write(icsv,'(a)')dqt//'  '//comment1(1:kl)//dqt
             else
                iohead=0
             endif
           endif
        enddo
c
c ---   record #NCOM+3 - run control parameters -- 37 words
        read(in)
     1   ibymet_cm,ibmmet_cm,ibdmet_cm,ibhmet_cm,ibtz_cm,irlgmet_cm,
     2   irtype_cm,nxm_cm,nym_cm,nzm_cm,xgridm_cm,xorigm_cm,yorigm_cm,
     3   iwfcod_cm,nssta_cm,nusta_cm,npsta_cm,nowsta_cm,nlu_cm,
     4   iwat1_cm,iwat2_cm,lcalgrd_cm,
     5   pmap_cm,datum_cm,daten_cm,feast_cm,fnorth_cm,
     6   utmhem_cm,iutmzn_cm,
     7   rnlat0_cm,relon0_cm,xlat1_cm,xlat2_cm

        call YR4(iout,ibymet_cm,ierrb)
        if(ierrb.NE.0) then
           iret=1
           return
        endif

c ---   Place integer IBTZ into real XBTZ
        xbtz_cm=FLOAT(ibtz_cm)
c ---   Process into character AXBTZ
        call BASRUTC(xbtz_cm,axbtz_cm)

        rlat0_cm=rnlat0_cm
        rlon0_cm=-relon0_cm

c ---   Convert start time from end to beginning of first hour
        call JULDAY(iout,ibymet_cm,ibmmet_cm,ibdmet_cm,ibjdmet_cm)
        if(iret.NE.0) return
        call INCR(iout,ibymet_cm,ibjdmet_cm,ibhmet_cm,-1)
        call GRDAY(iout,ibymet_cm,ibjdmet_cm,ibmmet_cm,ibdmet_cm)
        ibsmet_cm=0
c ---   Find end of run (irlgmet is integer hours)
        ieymet_cm=ibymet_cm
        iemmet_cm=ibmmet_cm
        iedmet_cm=ibdmet_cm
        iejdmet_cm=ibjdmet_cm
        iehmet_cm=ibhmet_cm
        call INCR(iout,ieymet_cm,iejdmet_cm,iehmet_cm,irlgmet_cm)
        call GRDAY(iout,ieymet_cm,iejdmet_cm,iemmet_cm,iedmet_cm)
        iesmet_cm=0

      else
c ---   Read older CALMET header records
c --------------------------------------
        itimes_cm=0
        i2dmet_cm=0
c
c ---   record #1 - run title -- 60 words
        read(in)title_cm

c ---   Output
        do k=1,3
           write(iout,'(a)')'  '//title_cm(k)
        enddo
c
c ---   record #2 - run control parameters -- 26 words
c ---   (vermet, levmet are both 8 bytes)
        read(in)
     1   vermet,levmet_cm,ibymet_cm,ibmmet_cm,ibdmet_cm,ibhmet_cm,
     2   ibtz_cm,irlgmet_cm,irtype_cm,nxm_cm,nym_cm,nzm_cm,
     3   xgridm_cm,xorigm_cm,yorigm_cm,iutmznm_cm,iwfcod_cm,
     4   nssta_cm,nusta_cm,npsta_cm,nowsta_cm,nlu_cm,iwat1_cm,
     5   iwat2_cm,lcalgrd_cm


c ---   Place integer IBTZM into real XBTZM
        xbtz_cm=FLOAT(ibtz_cm)
c ---   Process into character AXBTZ
        call BASRUTC(xbtz_cm,axbtz_cm)

c ---   Enforce YYYY format
        call YR4(iout,ibymet_cm,ierrb)
        if(ierrb.NE.0) then
           iret=1
           return
        endif
c
c ---   New record -- #3 - additional run control data -- 8 words
c ---   Null values are assigned in block data
c ---   This record was introduced in CALMET Version 5.0 (980304)
        if(dataver.EQ.'1.5') then
           read(in) xlat0_cm,xlon0_cm,llconf_cm,conec_cm,
     &              xlat1_cm,xlat2_cm,rlat0_cm,rlon0_cm
        endif
c
c ---   Recast map projection information
        if(LLCONF_cm) then
           pmap_cm='LCC     '
        else
           pmap_cm='UTM     '
           UTMHEM_cm='N   '
           if(xlat0_cm.LT.0.) UTMHEM_cm='S   '
        endif
c ---   Set datum to unknown
        datum_cm='unknown'

c ---   Convert start time from end to beginning of first hour
        call JULDAY(iout,ibymet_cm,ibmmet_cm,ibdmet_cm,ibjdmet_cm)
        if(iret.NE.0) return
        call INCR(iout,ibymet_cm,ibjdmet_cm,ibhmet_cm,-1)
        call GRDAY(iout,ibymet_cm,ibjdmet_cm,ibmmet_cm,ibdmet_cm)
        ibsmet_cm=0
c ---   Find end of run (irlgmet is integer hours)
        ieymet_cm=ibymet_cm
        iemmet_cm=ibmmet_cm
        iedmet_cm=ibdmet_cm
        iejdmet_cm=ibjdmet_cm
        iehmet_cm=ibhmet_cm
        call INCR(iout,ieymet_cm,iejdmet_cm,iehmet_cm,irlgmet_cm)
        call GRDAY(iout,ieymet_cm,iejdmet_cm,iemmet_cm,iedmet_cm)
        iesmet_cm=0

      endif

c --- Check array space
      if(nxm_cm.GT.mxnx_cm .OR. nym_cm.GT.mxny_cm)then
         write(iout,*)'ERROR -- X,Y grid dimensions exceed',
     1   'array allocation'
         write(iout,*)'MXNX,MXNY = ',mxnx_cm,mxny_cm
         write(iout,*)'  NX,  NY = ',nxm_cm,nym_cm
         iret=1
         return
      endif

      nzmp1_cm=nzm_cm+1
      if(mxnzp1_cm.lt.nzmp1_cm)then
         write(iout,*)'ERROR -- dimension of ZFACEM ',
     1   'array is too small for no. layers in MET file'
         write(iout,*)'MXNZP1 = ',mxnzp1_cm,' NZMP1 = ',nzmp1_cm
         iret=1
         return
      endif

c -------------------------------------
c
c --- record #NCOM+4 - cell face heights (NZ + 1 words)
      call rdr1d(in,itimes_cm,zfacem_cm,nzmp1_cm,clab1,
     &           idum,idum,idum,idum)
c
      if(LECHO)then
         write(iout,203)(zfacem_cm(n),n=1,nzmp1_cm)
203      format(1x,'ZFACEM  = ',10(f9.3,', ')/)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ZFACE')then
         clabexp='ZFACE'
         go to 999
      endif


      nums=nssta_cm+nusta_cm+npsta_cm
      if(lecho .AND. nums.GT.0) then
         write(iout,*)'Met Station Locations (Met Grid meters)'
      endif
c
c --- records #NCOM+5 & 6 - x, y coordinates of surface stations
c --- (NSSTA words each record)
      if(nssta_cm.gt.0)then
         call rdr1d(in,itimes_cm,xssta_cm,nssta_cm,clab1,
     &              idum,idum,idum,idum)
         call rdr1d(in,itimes_cm,yssta_cm,nssta_cm,clab2,
     &              idum,idum,idum,idum)
         if(LECHO)then
            write(iout,*)clab1,' = ',(xssta_cm(n),n=1,nssta_cm)
            write(iout,*)clab2,' = ',(yssta_cm(n),n=1,nssta_cm)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XSSTA'.or.clab2.ne.'YSSTA')then
            clabex1='XSSTA'
            clabex2='YSSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- records #NCOM+7 & 8 - x, y coordinates of upper air stations
c --- (NUSTA words each record)
      if(nusta_cm.gt.0)then
         call rdr1d(in,itimes_cm,xusta_cm,nusta_cm,clab1,
     &              idum,idum,idum,idum)
         call rdr1d(in,itimes_cm,yusta_cm,nusta_cm,clab2,
     &              idum,idum,idum,idum)
         if(LECHO)then
            write(iout,*)clab1,' = ',(xusta_cm(n),n=1,nusta_cm)
            write(iout,*)clab2,' = ',(yusta_cm(n),n=1,nusta_cm)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XUSTA'.or.clab2.ne.'YUSTA')then
            clabex1='XUSTA'
            clabex2='YUSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- records #NCOM+9 & 10 - x, y coordinates of precipitation stations
c --- (NPSTA words each record)
      if(npsta_cm.gt.0)then
         call rdr1d(in,itimes_cm,xpsta_cm,npsta_cm,clab1,
     &              idum,idum,idum,idum)
         call rdr1d(in,itimes_cm,ypsta_cm,npsta_cm,clab2,
     &              idum,idum,idum,idum)
         if(LECHO)then
            write(iout,*)clab1,' = ',(xpsta_cm(n),n=1,npsta_cm)
            write(iout,*)clab2,' = ',(ypsta_cm(n),n=1,npsta_cm)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XPSTA'.or.clab2.ne.'YPSTA')then
            clabex1='XPSTA'
            clabex2='YPSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- record #NCOM+11 - surface roughness lengths (NX * NY words)
      call rdr2d(in,itimes_cm,z0_cm,xbuf,
     1           mxnx_cm,mxny_cm,nxm_cm,nym_cm,clab1,
     2           idum,idum,idum,idum,ieof)
      if(ieof.EQ.1) then
         write(iout,*)'Unexpected EOF in CALMET header'
         iret=1
         return
      endif
c      if(LECHO)then
c         messag='Surface roughness lengths (m)'
c         messag(63:70)=clab1
c         call OUT(z0_cm,idum,1,5,ldate,messag,nxm_cm,nym_cm)
c      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'Z0')then
         clabexp='Z0'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+12 - land use categories (NX * NY words)
      call rdi2d(in,itimes_cm,ilandu_cm,ibuf,
     1           mxnx_cm,mxny_cm,nx_cm,ny_cm,clab1,
     2           idum,idum,idum,idum)
c      if(LECHO)then
c         messag='Land use categories'
c         messag(63:70)=clab1
c         call OUT(xdum,ilandu_cm,2,5,ldate,messag,nx_cm,ny_cm)
c      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ILANDU')then
         clabexp='ILANDU'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+13 - elevations (NX * NY words)
      call rdr2d(in,itimes_cm,elev_cm,xbuf,
     1          mxnx_cm,mxny_cm,nx_cm,ny_cm,clab1,
     2          idum,idum,idum,idum,ieof)
      if(ieof.EQ.1) then
         write(iout,*)'Unexpected EOF in CALMET header'
         iret=1
         return
      endif
c      if(LECHO)then
c         messag='Terrain heights (m)'
c         messag(63:70)=clab1
c         call OUT(elev_cm,idum,1,5,ldate,messag,nx_cm,ny_cm)
c      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ELEV')then
         clabexp='ELEV'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+14 - leaf area index (NX * NY words)
      call rdr2d(in,itimes_cm,xlai_cm,xbuf,
     1           mxnx_cm,mxny_cm,nx_cm,ny_cm,clab1,
     2           idum,idum,idum,idum,ieof)
      if(ieof.EQ.1) then
         write(iout,*)'Unexpected EOF in CALMET header'
         iret=1
         return
      endif
c      if(LECHO)then
c         messag='Leaf area index'
c         messag(63:70)=clab1
c         call OUT(xlai_cm,idum,1,5,ldate,messag,nx_cm,ny_cm)
c      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'XLAI')then
         clabexp='XLAI'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+15 - nearest surface station to each grid point
c                  (NX * NY words)
c frr (09/01)
      if (nssta_cm.ge.1) then
         call rdi2d(in,itimes_cm,nears_cm,ibuf,
     1              mxnx_cm,mxny_cm,nx_cm,ny_cm,clab1,
     2              idum,idum,idum,idum)
c         if(LECHO)then
c           messag='Nearest surface station no. to each grid point'
c           messag(63:70)=clab1
c           call OUT(xdum,nears_cm,2,5,ldate,messag,nx_cm,ny_cm)
c         endif
c
c ---    Check that record label matches expected label
         if(clab1.ne.'NEARS')then
            clabexp='NEARS'
            go to 999
         endif

      endif


c -------------------------------------
c --- Output
c -------------------------------------

c --- Create minutes and seconds from 3600 seconds
      iminb=ibsmet_cm/60
      isecb=ibsmet_cm-iminb*60
      imine=iesmet_cm/60
      isece=iesmet_cm-imine*60

c --- Convert selected meters to kilometers
      xgridkm=xgridm_cm*0.001
      xorigkm=xorigm_cm*0.001
      yorigkm=yorigm_cm*0.001
c --- Compute mid-pt heights of each layer
      do j=1,nzm_cm
         zmid(j)=0.5*(zfacem_cm(j)+zfacem_cm(j+1))
      enddo

c --- Time Period
c ---TXT---
      write(iout,*)
      write(iout,'(a)')' Time Period:'
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data BEGIN on ',ibdmet_cm,cmonth(ibmmet_cm),ibymet_cm,
     &   ' at ',ibhmet_cm,cc,iminb,cc,isecb,' (hh:mm:ss) ',axbtz_cm
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data   END on ',iedmet_cm,cmonth(iemmet_cm),ieymet_cm,
     &   ' at ',iehmet_cm,cc,imine,cc,isece,' (hh:mm:ss) ',axbtz_cm
      write(iout,*)
      write(iout,'(a25,i10)') ' Number of model steps = ',irlgmet_cm
      write(iout,*)
c ---CSV---
      write(icsv,*)
      write(icsv,'(a)')dqt//' Time Period:'//dqt
      write(icsv,'(a1,a6,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'BEGIN:',dqt,cm,ibdmet_cm,ds,cmonth(ibmmet_cm),ds,
     &   ibymet_cm,cm,ibhmet_cm,cc,iminb,cc,isecb,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,axbtz_cm,dqt
      write(icsv,'(a1,a4,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'END:',dqt,cm,iedmet_cm,ds,cmonth(iemmet_cm),ds,
     &   ieymet_cm,cm,iehmet_cm,cc,imine,cc,isece,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,axbtz_cm,dqt
      write(icsv,*)
      write(icsv,'(a13,a1,i10)') '# Model Steps',cm,irlgmet_cm
      write(icsv,*)

c --- Meteorological Variables
c ---TXT---
      write(iout,'(a)')' 3-D Meteorological Variables:'
      write(iout,*) '  Horizontal Wind [U,V]              (m/s)'
      if(LCALGRD_cm) then
         write(iout,*) '  Vertical   Wind [W]                (m/s)'
         write(iout,*) '  Air Temperature                    (K)'
      endif
      write(iout,*)
      write(iout,'(a)')' 2-D Meteorological Variables:'
      write(iout,*) '  Pasquill-Gifford Stability Class'
      write(iout,*) '  Friction Velocity [U-star]         (m/s)'
      write(iout,*) '  Mixing Height [Zi]                 (m)'
      write(iout,*) '  Monin-Obukhov Length [L]           (m)'
      write(iout,*) '  Convective Velocity Scale [W-star] (m/s)'
      write(iout,*) '  Precipitation rate                 (mm/hr)'
      if(i2dmet_cm.EQ.1) then
         write(iout,*) '  Precipitation code'
         write(iout,*) '  Air Density                        (kg/m3)'
         write(iout,*) '  Relative Humidity                  (%)'
         write(iout,*) '  Short-wave Solar Radiation         (W/m2)'
      else
         write(iout,*)
         write(iout,'(a)')' Meteorological Variables at Met Stations:'
         write(iout,*) '  Precipitation code'
         write(iout,*) '  Air Density                        (kg/m3)'
         write(iout,*) '  Relative Humidity                  (%)'
         write(iout,*) '  Air Temperature                    (K)'
         write(iout,*) '  Short-wave Solar Radiation         (W/m2)'
      endif
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' 3-D Meteorological Variables:'
      write(icsv,*) 'Horizontal Wind [U-V]              (m/s)'
      if(LCALGRD_cm) then
         write(icsv,*) 'Vertical   Wind [W]                (m/s)'
         write(icsv,*) 'Air Temperature                    (K)'
      endif
      write(icsv,*)
      write(icsv,'(a)')' 2-D Meteorological Variables:'
      write(icsv,*) 'Pasquill-Gifford Stability Class'
      write(icsv,*) 'Friction Velocity [U-star]         (m/s)'
      write(icsv,*) 'Mixing Height [Zi]                 (m)'
      write(icsv,*) 'Monin-Obukhov Length [L]           (m)'
      write(icsv,*) 'Convective Velocity Scale [W-star] (m/s)'
      write(icsv,*) 'Precipitation rate                 (mm/hr)'
      if(i2dmet_cm.EQ.1) then
         write(icsv,*) 'Precipitation code'
         write(icsv,*) 'Air Density                        (kg/m3)'
         write(icsv,*) 'Relative Humidity                  (%)'
         write(icsv,*) 'Short-wave Solar Radiation         (W/m2)'
      else
         write(icsv,*)
         write(icsv,'(a)')' Meteorological Variables at Met Stations:'
         write(icsv,*) 'Precipitation code'
         write(icsv,*) 'Air Density                        (kg/m3)'
         write(icsv,*) 'Relative Humidity                  (%)'
         write(icsv,*) 'Air Temperature                    (K)'
         write(icsv,*) 'Short-wave Solar Radiation         (W/m2)'
      endif
      write(icsv,*)

c --- Grid
c ---TXT---
      write(iout,'(a)')' Grid Configuration:'
      write(iout,*)'  NX Cells        = ',nxm_cm
      write(iout,*)'  NY Cells        = ',nym_cm
      write(iout,*)'  Cell Size (km)  = ',xgridkm
      write(iout,*)'  SW Corner (km)  = ',xorigkm,yorigkm
      write(iout,*)'  Vertical Layers = ',nzm_cm
      write(iout,*)
      write(iout,68) '   ZFACE  (m) ',(zfacem_cm(i),i=1,nzm_cm+1)
      write(iout,65) '   LAYER      ',(i,i=1,nzm_cm)
      write(iout,66) '   MID-PT (m) ',(zmid(i),i=1,nzm_cm)
65    format(1x,a14,2x,20i6)
66    format(1x,a14,4x,20f6.0)
68    format(1x,a14,20f6.0)
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Grid Configuration:'
      write(icsv,*)nxm_cm,cm,cm,'NX Cells'
      write(icsv,*)nym_cm,cm,cm,'NY Cells'
      write(icsv,*)xgridkm,cm,cm,'Cell Size (km)'
      write(icsv,*)xorigkm,cm,yorigkm,cm,'SW Corner (km)'
      write(icsv,*)nzm_cm,cm,cm,'Vertical Layers'
      write(icsv,*)
      write(icsv,168) 'ZFACE  (m)',(cm,zfacem_cm(i),i=1,nzm_cm+1)
      write(icsv,165) 'LAYER     ',(cm,i,i=1,nzm_cm)
      write(icsv,166) 'MID-PT (m)',(cm,zmid(i),i=1,nzm_cm)
165   format(1x,a10,2x,20(a1,i6))
166   format(1x,a10,20(a1,f6.0))
168   format(1x,a10,20(a1,f6.0))
      write(icsv,*)


c --- Coordinate System
c ---TXT---
      write(iout,'(a)')' Coordinate System:'
      write(iout,*)'  DATUM Code              = ',datum_cm
      write(iout,*)'  Map Projection          = ',pmap_cm
      if(PMAP_cm.EQ.'UTM     ') then
         write(iout,*)'  UTM Zone (1-60)         = ',iutmzn_cm
         write(iout,*)'  Hemisphere              = ',utmhem_cm
      else
         write(iout,*)'  Center Latitude   (deg) = ',rnlat0_cm
         write(iout,*)'  Center Longitude  (deg) = ',relon0_cm
         write(iout,*)'  Matching Latitude (deg) = ',xlat1_cm
         write(iout,*)'  Matching Latitude (deg) = ',xlat2_cm
         write(iout,*)'  False Easting      (km) = ',feast_cm
         write(iout,*)'  False Northing     (km) = ',fnorth_cm
      endif
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Coordinate System:'
      write(icsv,*)datum_cm,cm,cm,'DATUM Code'
      write(icsv,*)pmap_cm,cm,cm,'Map Projection'
      if(PMAP_cm.EQ.'UTM     ') then
         write(icsv,*)iutmzn_cm,cm,cm,'UTM Zone (1-60)'
         write(icsv,*)utmhem_cm,cm,cm,'Hemisphere'
      else
         write(icsv,*)rnlat0_cm,cm,'(deg)',cm,'Center Latitude'
         write(icsv,*)relon0_cm,cm,'(deg)',cm,'Center Longitude'
         write(icsv,*)xlat1_cm,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)xlat2_cm,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)feast_cm,cm,'(km)',cm,'False Easting'
         write(icsv,*)fnorth_cm,cm,'(km)',cm,'False Northing'
      endif
      write(icsv,*)

c --- Process CALMET control file image
      REWIND(icfx)
      call COMP_METCF
      CLOSE(icfx)


c --- Assign data to DLL output argument array
      call DLLARG(ibymet_cm,ibjdmet_cm,ibmmet_cm,ibdmet_cm,
     &            ibhmet_cm,ibsmet_cm,
     &            ieymet_cm,iejdmet_cm,iemmet_cm,iedmet_cm,
     &            iehmet_cm,iesmet_cm,
     &            rnlat0_cm,relon0_cm,xlat1_cm,xlat2_cm,feast_cm,
     &            fnorth_cm,iutmzn_cm,axbtz_cm,utmhem_cm,datum_cm,
     &            pmap_cm)

      return

c --- Write error messages

c --- Dataset name does not match so return ERROR
99    iret=1
      estrng='Input file Dataset Name is not CALMET.DAT: '//
     &        dataset
      return

c --- Incorrect record label read
999   continue
      write(iout,1001)clab1,clabexp
1001  format(/1x,'ERROR in subr. MET1 -- incorrect record label ',
     1 'read from MET data file'//1x,'Label read     = ',a8/
     2                               1x,'Label expected = ',a8)
      iret=1
      return
1999  continue
      write(iout,2001)clab1,clab2,clabex1,clabex2
2001  format(/1x,'ERROR in subr. MET1 -- incorrect record label ',
     1 'read from MET data file'//
     2 1x,'Labels read     = ',a8,2x,a8/
     3 1x,'Labels expected = ',a8,2x,a8)
      iret=1
      return

      end
c----------------------------------------------------------------------
      subroutine rdi1d(iomet,mtver,idat,nwords,clabel,ndathrb,
     &                 nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- PRTMET   Version: 4.44      Level: 051012                   RDI1D
c ---          J. Scire, Earth Tech
c
c --- PURPOSE:  Read "NWORDS" of a one-dimensional integer array
c
c --- UPDATE
c --- V4.4-V4.41    051012  (DGS): resolve times to the second, and
c                                  include begin/end times
c                                  (remains compatible with
c                                  older end-time version)
c
c --- INPUTS:
c         IOMET - integer       - Fortran unit number of input file
c         MTVER - integer       - Time-mark flag
c                                 0: end-time (no seconds)
c                                 1: begin-time / end-time
c
c --- OUTPUT:
c  IDAT(nwords) - integer array - Array read from file
c        NWORDS - integer       - Number of words to read
c        CLABEL - character*8   - Variable name
c       NDATHRB - integer       - Beginning date and time (YYYYJJJHH)
c         NSECB - integer       - Beginning seconds (SSSS)
c       NDATHRE - integer       - Ending date and time (YYYYJJJHH)
c         NSECE - integer       - Ending seconds (SSSS)
c
c --- RDI1D called by:  METHDX, RDMET
c --- RDI1D calls:      none
c----------------------------------------------------------------------
c
      integer idat(nwords)
      character*8 clabel
c
      if(mtver.EQ.1) then
         read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,idat
      elseif(mtver.EQ.0) then
         read(iomet)clabel,ndathre,idat
         nsece=0
         ndathrb=0
         nsecb=0
      endif
      return
      end
c----------------------------------------------------------------------
      subroutine rdi2d(iomet,mtver,idat,ibuf,mxnx,mxny,nx,ny,clabel,
     &                 ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- PRTMET   Version: 4.44      Level: 051012                   RDI2D
c              J. Scire, Earth Tech
c
c --- PURPOSE:  Read NX * NY words of a 2-D integer array
c
c --- UPDATE
c --- V4.4-V4.41    051012  (DGS): resolve times to the second, and
c                                  include begin/end times for CALMET
c                                  Version 6 (remains compatible with
c                                  older version of CALMET)
c
c --- INPUTS:
c               IOMET - integer       - Fortran unit number of input
c                                       file
c               MTVER - integer       - Time-mark flag
c                                       0: end-time (no seconds)
c                                       1: begin-time / end-time
c         IBUF(nx,ny) - integer array - Buffer to hold input data
c           MXNX,MXNY - integers      - Dimensions of data array
c               NX,NY - integers      - Actual size of grid to read
c
c --- OUTPUT:
c     IDAT(mxnx,mxny) - integer array - Input data array (padded if
c                                       necessary)
c              CLABEL - character*8   - Variable name
c             NDATHRB - integer       - Beginning date and time (YYYYJJJHH)
c               NSECB - integer       - Beginning seconds (SSSS)
c             NDATHRE - integer       - Ending date and time (YYYYJJJHH)
c               NSECE - integer       - Ending seconds (SSSS)
c
c --- RDI2D called by:  METHDX, RDMET
c --- RDI2D calls:      none
c----------------------------------------------------------------------
      integer idat(mxnx,mxny),ibuf(nx,ny)
      character*8 clabel
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- read full grid
         if(mtver.EQ.1) then
            read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,idat
         elseif(mtver.EQ.0) then
            read(iomet)clabel,ndathre,idat
            nsece=0
            ndathrb=0
            nsecb=0
         endif
      else
c
c ---    only a portion of grid being used -- read and
c ---    transfer from buffer
c
         if(mtver.EQ.1) then
            read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,ibuf
         elseif(mtver.EQ.0) then
            read(iomet)clabel,ndathre,ibuf
            nsece=0
            ndathrb=0
            nsecb=0
         endif
c
         do 10 i=1,nx
         do 10 j=1,ny
         idat(i,j)=ibuf(i,j)
10       continue
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine rdr1d(iomet,mtver,x,nwords,clabel,
     &                 ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- PRTMET   Version: 4.44      Level: 051012                   RDR1D
c ---          J. Scire, Earth Tech
c
c --- PURPOSE:  Read "NWORDS" of a one-dimensional real array
c
c --- UPDATE
c --- V4.4-V4.41    051012  (DGS): resolve times to the second, and
c                                  include begin/end times for CALMET
c                                  Version 6 (remains compatible with
c                                  older version of CALMET)
c
c --- INPUTS:
c         IOMET - integer     - Fortran unit number of input file
c         MTVER - integer     - Time-mark flag
c                               0: end-time (no seconds)
c                               1: begin-time / end-time
c
c --- OUTPUT:
c     X(nwords) - real array  - Array read from file
c        NWORDS - integer     - Number of words to read
c        CLABEL - character*8 - Variable name
c       NDATHRB - integer     - Beginning date and time (YYYYJJJHH)
c         NSECB - integer     - Beginning seconds (SSSS)
c       NDATHRE - integer     - Ending date and time (YYYYJJJHH)
c         NSECE - integer     - Ending seconds (SSSS)
c
c --- RDR1D called by:  METHDX, RDMET
c --- RDR1D calls:      none
c----------------------------------------------------------------------
      real x(nwords)
      character*8 clabel
c
      if(mtver.EQ.1) then
         read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,x
      elseif(mtver.EQ.0) then
         read(iomet)clabel,ndathre,x
         nsece=0
         ndathrb=0
         nsecb=0
      endif
      return
      end
c----------------------------------------------------------------------
      subroutine rdr2d(iomet,mtver,x,xbuf,mxnx,mxny,nx,ny,clabel,
     &                 ndathrb,nsecb,ndathre,nsece,ieof)
c----------------------------------------------------------------------
c
c --- PRTMET   Version: 4.44      Level: 051012                   RDR2D
c              J. Scire, Earth Tech
c
c --- PURPOSE:  Read NX * NY words of a 2-D real array
c
c --- UPDATE 
c --- V4.4-V4.41    051012 (DGS): resolve times to the second, and
c                                 include begin/end times for CALMET
c                                 Version 6 (remains compatible with
c                                 older version of CALMET)
c --- 940830  to    040923 (DGS): add IEOF to recover from end-of-file
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of input file
c            MTVER - integer     - Time-mark flag
c                                  0: end-time (no seconds)
c                                  1: begin-time / end-time
c      XBUF(nx,ny) - real array  - Buffer to hold input data
c      MXNX,MXNY   - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to read
c
c --- OUTPUT:
c     X(mxnx,mxny) - real array  - Input data array (padded if nec.)
c           CLABEL - character*8 - Variable name
c          NDATHRB - integer     - Beginning date and time (YYYYJJJHH)
c            NSECB - integer     - Beginning seconds (SSSS)
c          NDATHRE - integer     - Ending date and time (YYYYJJJHH)
c            NSECE - integer     - Ending seconds (SSSS)
c             IEOF - integer     - End-of-File status
c                                  0 = pointer within file
c                                  1 = EOF reached on read
c
c --- RDR2D called by:  METHDX, RDMET
c --- RDR2D calls:      none
c----------------------------------------------------------------------
      real x(mxnx,mxny),xbuf(nx,ny)
      character*8 clabel

c --- Set EOF
      ieof=0
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- read full grid
         if(mtver.EQ.1) then
            read(iomet,end=999)clabel,ndathrb,nsecb,ndathre,nsece,x
         elseif(mtver.EQ.0) then
            read(iomet,end=999)clabel,ndathre,x
            nsece=0
            ndathrb=0
            nsecb=0
         endif
      else
c
c ---    only a portion of grid being used -- read and
c ---    transfer from buffer
         if(mtver.EQ.1) then
            read(iomet,end=999)clabel,ndathrb,nsecb,ndathre,nsece,xbuf
         elseif(mtver.EQ.0) then
            read(iomet,end=999)clabel,ndathre,xbuf
            nsece=0
            ndathrb=0
            nsecb=0
         endif
c
         do 10 i=1,nx
         do 10 j=1,ny
         x(i,j)=xbuf(i,j)
10       continue
      endif
c
      return

999   ieof=1
      return

      end

c----------------------------------------------------------------------
      subroutine comp_puf
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 110301          COMP_PUF
c               D. Strimaitis
c
c --- PURPOSE:  Read and report selected information from a CALPUFF
c               file header
c
c --- UPDATES:
c --- V1.4 (L090410) to V1.6 (L110301) DGS
c            - Add Dataset v2.2
c
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Add IRET check after GETDOC_CP call
c            NEW:     DLLINIT
c
c --- V1.1 (L090310) to V1.3 (L090317) DGS
c            - Add CALPUFF control file processing
c            - Add CSV output
c            NEW:     COMP_PUFCF
c
c --- V1.0 (L090302) to V1.1 (L090310) DGS
c            - Pass begin and end date-times, and projection data
c              for DLL output
c            NEW:     DLLARG
c
c
c --- INPUTS:
c
c --- OUTPUT:
c     Common block /HEAD_CP/ variables:
c
c --- COMP_PUF called by:  (main)
c --- COMP_PUF calls:      INCR, INCRS, YR4, GETDOC_CP
c                          GRDAY, JULDAY
c                          BASRUTC, UTCBASR, DLLARG, DLLINIT,
c                          COMP_PUFCF
c----------------------------------------------------------------------
c
c --- Include parameters

      include 'params.ivu'
      include 'cpuff.ivu'
      include 'ctldat.ivu'
      include 'passdll.ivu'
      include 'err.cal'

c --- Variables added for DATASET VERSION 2.2 (Stored in /HEAD_CP/)
c     ABTZ                   base time zone as UTC+hhmm
c     ASPUNIT(mxsplv,mxfile) units name for each species-level

c --- Variables added for DATASET VERSION 2.1
c     NSTYPE(mxfile,8) holds the number of each type of source
c                     in each file
c               1 - point sources (Control File)
c               2 - point sources (Var. Emissions File)
c               3 - area sources (Control File)
c               4 - area sources (Var. Emissions File)
c               5 - line sources (Control File)
c               6 - line sources (Var. Emissions File)
c               7 - volume sources (Control File)
c               8 - volume sources (Var. Emissions File)
      integer nstype(mxfile_cp,8)
      character*4 cstype(8)
      character*1 cc,cm,ds,dqt
      character*3 cmonth(12)
      character*8 axbtz

      character*48 msg(8),msg0
      logical lfatal,ldbg, lvisib
      logical lspunit

      data msg0  /'  **********************************************'/
      data msg(1)/'  *      FATAL Problem Found in COMP_PUF       *'/
      data msg(2)/'  * ------------------------------------------ *'/
      data msg(3)/'  *  Size of Array(s) in CALPUFF.DAT too Large *'/
      data msg(4)/'  *  Re-size Array Parameter(s) in CPUFF.IUV   *'/
      data msg(5)/'  *         and Recompile METSERIES            *'/
      data msg(6)/'  * ------------------------------------------ *'/
      data msg(7)/'  * Parameter       CPUFF.IVU      CALPUFF.DAT *'/
      data msg(8)/'  * ---------       ----------     ----------- *'/


c --- Set names of each source type for error report section
      data cstype/'pt1 ','pt2 ','ar1 ','ar2 ',
     &            'ln1 ','ln2 ','vl1 ','vl2 '/
      data cc/':'/
      data cm/','/
      data ds/'-'/
      data dqt/'"'/

c --- Set months
      data cmonth/'Jan','Feb','Mar','Apr','May','Jun',
     &            'Jul','Aug','Sep','Oct','Nov','Dec'/

      ldbg=.FALSE.
      lfatal=.FALSE.
      lvisib=.FALSE.
      lspunit=.FALSE.
      nf=1

c --- Initialize data used in DLL output argument array
      call DLLINIT(msyr_cp,mjsday_cp,ibmon,ibday,mshr_cp,mssec_cp,
     &            ieyr,iejul,iemon,ieday,iehr,iesec,
     &            rnlat0_cp,relon0_cp,xlat1_cp,xlat2_cp,feast_cp,
     &            fnorth_cp,iutmzn_cp,axbtz,utmhem_cp,datum_cp,
     &            pmap_cp)

c -------------------------------------------------------
c --- Start Output
c -------------------------------------------------------

c --- Type
      write(iout,'(//1x,a27)')'CALPUFF Model Configuration'
      write(iout,'(1x,a27/)') '---------------------------'
      write(icsv,'(//1x,a27)')'CALPUFF Model Configuration'
 
c --- File
      k=LEN_TRIM(fileinp)
      write(iout,'(a)')' File Examined: '//fileinp(1:k)
      write(iout,*)
      write(icsv,'(a)')dqt//' File Examined: '//fileinp(1:k)//dqt
      write(icsv,*)

c --- Titles
      write(iout,'(a)')' Titles:'
      write(icsv,'(a)')dqt//' Titles:'//dqt

c --- Process first section of CALPUFF output file (including comment
c --- records), and determine dataset version
      call GETDOC_CP(in,iout,icsv,icfx,nf,idtype,cpverx_cp(nf))
      if(iret.NE.0) return

c  Read control variables (record NCOM+3)

      if(idtype.EQ.0) then
c ---   Dataset EARLIER than Version 2.0
        i2drhu_cp=0
        read(in,iostat=ierr)
     *     amodel_cp,aver_cp,alev_cp,msyr_cp,mjsday_cp,mshr_cp,
     *     mssec_cp,mnrun_cp,mavgpd_cp,nsecdt_cp,
     *     ielmet_cp,jelmet_cp,delx_cp,dely_cp,nz_cp,
     *     xorigkm_cp,yorigkm_cp,nstas_cp,
     *     iastar_cp,iastop_cp,jastar_cp,jastop_cp,
     *     isastr_cp,jsastr_cp,isastp_cp,jsastp_cp,
     *     meshdn_cp,kpts,kareas,klines,kvols,
     *     ndrec_cp,nctrec_cp,LSGRID_cp,nszout_cp(nf),lcomprs_cp(nf)
        if(ierr.EQ.0) then
           iptimex_cp(nf)=2
           isrcinfox_cp(nf)=0
           isrcindv_cp=0
        else
c ---      May have hit end of header record; try again with old format
           rewind(in)
           read(in)
     *        amodel_cp,aver_cp,alev_cp,msyr_cp,mjsday_cp,mshr_cp,
     *        mnhrs_cp,mavg_cp,
     *        ielmet_cp,jelmet_cp,delx_cp,dely_cp,nz_cp,
     *        xorigkm_cp,yorigkm_cp,nstas_cp,
     *        iastar_cp,iastop_cp,jastar_cp,jastop_cp,
     *        isastr_cp,jsastr_cp,isastp_cp,jsastp_cp,
     *        meshdn_cp,kpts,kareas,klines,kvols,
     *        ndrec_cp,nctrec_cp,LSGRID_cp,nszout_cp(nf),
     *        lcomprs_cp(nf),i2drhu_cp
           iptimex_cp(nf)=1
           isrcinfox_cp(nf)=0
           isrcindv_cp=0
        endif

      else

c ---   Dataset version 2.0 or later format
        if(cpverx_cp(nf).EQ.'2.0             ') then
          read(in)
     *       amodel_cp,aver_cp,alev_cp,msyr_cp,mjsday_cp,mshr_cp,
     *       mnhrs_cp,mavg_cp,
     *       ielmet_cp,jelmet_cp,delx_cp,dely_cp,nz_cp,
     *       xorigkm_cp,yorigkm_cp,nstas_cp,
     *       iastar_cp,iastop_cp,jastar_cp,jastop_cp,
     *       isastr_cp,jsastr_cp,isastp_cp,jsastp_cp,
     *       meshdn_cp,kpts,kareas,klines,kvols,
     *       ndrec_cp,nctrec_cp,LSGRID_cp,nszout_cp(nf),
     *       lcomprs_cp(nf),i2drhu_cp,
     *       iutmzn_cp,feast_cp,fnorth_cp,rnlat0_cp,relon0_cp,
     *       xlat1_cp,xlat2_cp,pmap_cp,utmhem_cp,datum_cp,daten_cp,
     *       clat0_cp,clon0_cp,clat1_cp,clat2_cp
          iptimex_cp(nf)=1
          isrcinfox_cp(nf)=0
          isrcindv_cp=0

        elseif(cpverx_cp(nf).EQ.'2.1             ') then
          read(in)
     *       amodel_cp,aver_cp,alev_cp,msyr_cp,mjsday_cp,mshr_cp,
     *       mssec_cp,xbtz_cp,mnrun_cp,mavgpd_cp,nsecdt_cp,
     *       ielmet_cp,jelmet_cp,delx_cp,dely_cp,nz_cp,
     *       xorigkm_cp,yorigkm_cp,nstas_cp,
     *       iastar_cp,iastop_cp,jastar_cp,jastop_cp,
     *       isastr_cp,jsastr_cp,isastp_cp,jsastp_cp,
     *       meshdn_cp,nstype(nf,1),nstype(nf,2),nstype(nf,3),
     *       nstype(nf,4),nstype(nf,5),nstype(nf,6),
     *       nstype(nf,7),nstype(nf,8),msource_cp,
     *       ndrec_cp,nctrec_cp,LSGRID_cp,nszout_cp(nf),
     *       lcomprs_cp(nf),i2drhu_cp,
     *       iutmzn_cp,feast_cp,fnorth_cp,rnlat0_cp,relon0_cp,
     *       xlat1_cp,xlat2_cp,pmap_cp,utmhem_cp,datum_cp,daten_cp,
     *       clat0_cp,clon0_cp,clat1_cp,clat2_cp
          iptimex_cp(nf)=2
          isrcinfox_cp(nf)=1
          isrcindv_cp=msource_cp
          kpts=nstype(nf,1)+nstype(nf,2)
          kareas=nstype(nf,3)+nstype(nf,4)
          klines=nstype(nf,5)+nstype(nf,6)
          kvols=nstype(nf,7)+nstype(nf,8)

        elseif(cpverx_cp(nf).EQ.'2.2             ') then
          read(in)
     *       amodel_cp,aver_cp,alev_cp,msyr_cp,mjsday_cp,mshr_cp,
     *       mssec_cp,abtz_cp,mnrun_cp,mavgpd_cp,nsecdt_cp,
     *       ielmet_cp,jelmet_cp,delx_cp,dely_cp,nz_cp,
     *       xorigkm_cp,yorigkm_cp,nstas_cp,
     *       iastar_cp,iastop_cp,jastar_cp,jastop_cp,
     *       isastr_cp,jsastr_cp,isastp_cp,jsastp_cp,
     *       meshdn_cp,nstype(nf,1),nstype(nf,2),nstype(nf,3),
     *       nstype(nf,4),nstype(nf,5),nstype(nf,6),
     *       nstype(nf,7),nstype(nf,8),msource_cp,
     *       ndrec_cp,nctrec_cp,LSGRID_cp,nszout_cp(nf),
     *       lcomprs_cp(nf),i2drhu_cp,
     *       iutmzn_cp,feast_cp,fnorth_cp,rnlat0_cp,relon0_cp,
     *       xlat1_cp,xlat2_cp,pmap_cp,utmhem_cp,datum_cp,daten_cp,
     *       clat0_cp,clon0_cp,clat1_cp,clat2_cp
          iptimex_cp(nf)=2
          isrcinfox_cp(nf)=1
          isrcindv_cp=msource_cp
          kpts=nstype(nf,1)+nstype(nf,2)
          kareas=nstype(nf,3)+nstype(nf,4)
          klines=nstype(nf,5)+nstype(nf,6)
          kvols=nstype(nf,7)+nstype(nf,8)
c ---     Convert from UTC string to real base time zone
          call UTCBASR(abtz_cp,xbtz_cp)
c ---     Specie-level units are in header
          lspunit=.TRUE.

        else
          write(iout,*)'ERROR:  Unknown DATASET Version: ',cpverx_cp(nf)
          iret=1
          return
        endif

      endif

c --- Update of number of sources for output header
      if(isrcinfox_cp(nf).EQ.0) then
c ---    DO NOT UPDATE SOURCE NUMBERS
cc ---    No individual source information is available, so just keep
cc ---    the maximum number of each type (cannot test for duplicates)
c         npts=MAX(npts,kpts)
c         nareas=MAX(nareas,kareas)
c         nlines=MAX(nlines,klines)
c         nvols=MAX(nvols,kvols)
      elseif(isrcinfox_cp(nf).EQ.1) then
c ---    Retain all source names even though some may be duplicates
c ---    (2 files may hold different species for the same sources)
         npts_cp=npts_cp+kpts
         nareas_cp=nareas_cp+kareas
         nlines_cp=nlines_cp+klines
         nvols_cp=nvols_cp+kvols
      endif

c --- Check year format
      call YR4(iout,msyr_cp,ierry)
      if(ierry.NE.0) then
         iret=1
         return
      endif

      if(iptimex_cp(nf).EQ.1) then
c ---    Move starting time back 1 hour to identify the start of the
c ---    first hour
         call INCR(iout,msyr_cp,mjsday_cp,mshr_cp,-1)
c ---    Set period step to 1 hour
         nsecdt_cp=3600
         mssec_cp=0
c ---    Rename averaging period variables
         mnrun_cp=mnhrs_cp
         mavgpd_cp=mavg_cp
      endif

c  Check size of array dimensions (compute gridded rec. array dims.)
c  with limits contained in CPUFF.SER file
      ngx_cp=(isastp_cp-isastr_cp)*meshdn_cp+1
      ngy_cp=(jsastp_cp-jsastr_cp)*meshdn_cp+1
      ngrec_cp=ngx_cp*ngy_cp
      if(ngx_cp .GT. mxgx_cp) lfatal=.TRUE.
      if(ngy_cp .GT. mxgy_cp) lfatal=.TRUE.
      if(ndrec_cp .GT. mxdrec_cp) lfatal=.TRUE.
      if(nctrec_cp .GT. mxctrec_cp) lfatal=.TRUE.
      if(nszout_cp(nf) .GT. mxsplv_cp) lfatal=.TRUE.
      nsrc_cp=0
      if(isrcinfox_cp(nf).EQ.1) then
         do k=1,8
            nsrc_cp=MAX(nsrc_cp,nstot_cp(k)+nstype(nf,k))
         enddo
      endif
      if(nsrc_cp .GT. mxsrc_cp) lfatal=.TRUE.

c  Set indicator for full sampling grid
      ifull_cp=0
      if(ngrec_cp .EQ. mxgrec_cp) ifull_cp=1

      if(LFATAL) then
c ---    Report problem information and quit
         write(iout,*)
         write(iout,100) msg0
         do i=1,8
            write(iout,100) msg(i)
         enddo
         write(iout,101)'  *     MXGX',mxgx_cp,ngx_cp
         write(iout,101)'  *     MXGY',mxgy_cp,ngy_cp
         write(iout,101)'  *   MXDREC',mxdrec_cp,ndrec_cp
         write(iout,101)'  *  MXCTREC',mxctrec_cp,nctrec_cp
         write(iout,101)'  *   MXSPLV',mxsplv_cp,nszout_cp(nf)
         write(iout,101)'  *    MXSRC',mxsrc_cp,nsrc_cp
         write(iout,100) msg0

         iret=1
         return
      endif
c
c  Read title of run (record NCOM+4)
      read(in) atitle_cp
c
c  Read specie/level list (record NCOM+5)
      read(in) (asplst_cp(i,nf),i=1,nszout_cp(nf))

c --- Read units for each specie/level in list (record NCOM+5a)
c --- (introduced in Dataset 2.2)
      if(LSPUNIT) then
         read(in) (aspunit_cp(i,nf),i=1,nszout_cp(nf))
      else
         do i=1,nszout_cp(nf)
            aspunit_cp(i,nf)='NO_UNITS_IN_FILE'
         enddo
      endif
c
c  Read coordinates of non-gridded receptors if any are used
c  (record NCOM+6)
      if(ndrec_cp .NE. 0) read(in) (xrec_cp(i),i=1,ndrec_cp),
     *                          (yrec_cp(i),i=1,ndrec_cp),
     *                          (zrec_cp(i),i=1,ndrec_cp)
c
c  Read coordinates of complex terrain receptors if any are used
c  (record NCOM+7)
      if(nctrec_cp .NE. 0) read(in) (xctr_cp(i),i=1,nctrec_cp),
     *                           (yctr_cp(i),i=1,nctrec_cp),
     *                           (zctr_cp(i),i=1,nctrec_cp),
     *                           (ihill_cp(i),i=1,nctrec_cp)
c

c  Set logicals for receptor types in file
      ldisc_cp=.FALSE.
      lctsg_cp=.FALSE.
      if(ndrec_cp .GT. 0) ldisc_cp=.TRUE.
      if(nctrec_cp .GT. 0) lctsg_cp=.TRUE.

c --- Compute coordinates of gridded receptors when present
      if(LSGRID_cp .AND. ngrec_cp .NE. 0) then
c --- Coord. for lower-left point in sampling grid, and spacing
         gx0=xorigkm_cp+delx_cp*(isastr_cp-0.5)
         gy0=yorigkm_cp+dely_cp*(jsastr_cp-0.5)
         delgx=delx_cp/meshdn_cp
         delgy=dely_cp/meshdn_cp
c ---    Fill out arrays
         do i=1,ngx_cp
            do j=1,ngy_cp
               xgrd_cp(i,j)=gx0+(i-1)*delgx
               ygrd_cp(i,j)=gy0+(j-1)*delgy
            enddo
         enddo
      endif

c --- Dataset version 2.1 introduces source names
c --- HEADER RECORDs #NCOM+8 to 15 -- Source names
      if(isrcinfox_cp(nf).EQ.1) then
         lfatal=.FALSE.
c ---    Loop over source types (points,areas,lines,volumes)
         do k=1,8
            if(nstype(nf,k).GT.0 .AND. .NOT.LFATAL) then
               n1=nstot_cp(k)+1
               nstot_cp(k)=nstot_cp(k)+nstype(nf,k)
               read(in) ktype_cp,(csource_cp(n,k),n=n1,nstot_cp(k))
               if(ktype_cp.NE.k) lfatal=.TRUE.
            endif
         enddo
         if(LFATAL) then
c ---       Report problem information and quit
            write(iout,*)
            write(iout,*)
            write(iout,100)'  *  Unexpected source type found, or   *'
            write(iout,100)'  *  Unexpected source number found.    *'
            write(iout,100)'  *  Number of sources expected:        *'
            write(iout,101)'  *     NPTS',nstot_cp(1),nstot_cp(2)
            write(iout,101)'  *   NAREAS',nstot_cp(3),nstot_cp(4)
            write(iout,101)'  *   NLINES',nstot_cp(5),nstot_cp(6)
            write(iout,101)'  *    NVOLS',nstot_cp(7),nstot_cp(8)
            write(iout,100)'  *  Source Names Read:                 *'
            do k=1,8
               do n=1,nstot_cp(k)
                  write(iout,*)cstype(k),csource_cp(n,k)
               enddo
            enddo
            iret=1
            return
         endif
      endif
c
c  Compute number of averaging periods in file
      mnper_cp=mnrun_cp/mavgpd_cp
c
c  Determine length of averaging period found in model file
      if(MOD(nsecdt_cp,3600).EQ.0) then
c ---    Hours
         mavg_cp=mavgpd_cp*(nsecdt_cp/3600)
         avtime_cp='  HOUR'
      elseif(MOD(nsecdt_cp,60).EQ.0) then
c ---    Minutes
         mavg_cp=mavgpd_cp*(nsecdt_cp/60)
         avtime_cp='MINUTE'
      else
c ---    Seconds
         mavg_cp=mavgpd_cp*nsecdt_cp
         avtime_cp='SECOND'
      endif
c



c -------------------------------------
c --- Output
c -------------------------------------
c --- Process time zone into character AXBTZ
      call BASRUTC(xbtz_cp,axbtz)

c --- Month and day at start
      call GRDAY(iout,msyr_cp,mjsday_cp,ibmon,ibday)

c --- Averaging time for output periods
      iavgsec=mavgpd_cp*nsecdt_cp

c --- Find end of run
c --- Number of hours per averaging period (real)
      hour_avgpd=FLOAT(nsecdt_cp*mavgpd_cp)/3600.
c --- Number of hours per run (real)
      hour_run=hour_avgpd*mnper_cp
c --- Whole hours per run (integer)
      nhour_run=INT(hour_run)
c --- Residual seconds
      nsec=3600.*(hour_run-FLOAT(nhour_run))
c --- End date
      ieyr=msyr_cp
      iejul=mjsday_cp
      iehr=mshr_cp
      iesec=mssec_cp
      call INCR(iout,ieyr,iejul,iehr,nhour_run)
      call INCRS(iout,ieyr,iejul,iehr,iesec,nsec)
      call GRDAY(iout,ieyr,iejul,iemon,ieday)

c --- Create minutes and seconds from 3600 seconds
      iminb=mssec_cp/60
      isecb=mssec_cp-iminb*60
      imine=iesec/60
      isece=iesec-imine*60

c --- Time Period
c ---TXT---
      write(iout,*)
      write(iout,'(a)')' Time Period:'
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data BEGIN on ',ibday,cmonth(ibmon),msyr_cp,
     &   ' at ',mshr_cp,cc,iminb,cc,isecb,' (hh:mm:ss) ',axbtz
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data   END on ',ieday,cmonth(iemon),ieyr,
     &   ' at ',iehr,cc,imine,cc,isece,' (hh:mm:ss) ',axbtz
      write(iout,*)
      write(iout,'(a24,i10)') ' Number of periods    = ',mnper_cp
      write(iout,'(a24,i10)') ' Averaging time (sec) = ',iavgsec
      write(iout,*)
c ---CSV---
      write(icsv,*)
      write(icsv,'(a)')dqt//' Time Period:'//dqt
      write(icsv,'(a1,a6,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'BEGIN:',dqt,cm,ibday,ds,cmonth(ibmon),ds,
     &   msyr_cp,cm,mshr_cp,cc,iminb,cc,isecb,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,axbtz,dqt
      write(icsv,'(a1,a4,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'END:',dqt,cm,ieday,ds,cmonth(iemon),ds,
     &   ieyr,cm,iehr,cc,imine,cc,isece,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,axbtz,dqt
      write(icsv,*)
      write(icsv,'(a9,a1,i10)') '# Periods',cm,mnper_cp
      write(icsv,'(a9,a1,i10)') 'Avg (sec)',cm,iavgsec
      write(icsv,*)

c --- Species
c ---TXT---
      write(iout,'(a)')' Species:'
      do i=1,nszout_cp(nf)
         write(iout,*) '   '//asplst_cp(i,nf)(1:12)//
     &                 '    '//TRIM(aspunit_cp(i,nf))
      enddo
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Species:'
      do i=1,nszout_cp(nf)
         write(icsv,*) asplst_cp(i,nf)(1:12),cm,TRIM(aspunit_cp(i,nf))
      enddo
      write(icsv,*)

c --- Grid
c ---TXT---
      write(iout,'(a)')' Grid Configuration:'
      write(iout,*)'  NX Cells        = ',ielmet_cp
      write(iout,*)'  NY Cells        = ',jelmet_cp
      write(iout,*)'  Cell Size (km)  = ',delx_cp
      write(iout,*)'  SW Corner (km)  = ',xorigkm_cp,yorigkm_cp
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Grid Configuration:'
      write(icsv,*)ielmet_cp,cm,cm,'NX Cells'
      write(icsv,*)jelmet_cp,cm,cm,'NY Cells'
      write(icsv,*)delx_cp,cm,cm,'Cell Size (km)'
      write(icsv,*)xorigkm_cp,cm,yorigkm_cp,cm,'SW Corner (km)'
      write(icsv,*)

c --- Coordinate System
c ---TXT---
      write(iout,'(a)')' Coordinate System:'
      write(iout,*)'  DATUM Code              = ',datum_cp
      write(iout,*)'  Map Projection          = ',pmap_cp
      if(PMAP_cp.EQ.'UTM     ') then
         write(iout,*)'  UTM Zone         (1-60) = ',iutmzn_cp
         write(iout,*)'  Hemisphere              = ',utmhem_cp
      else
         write(iout,*)'  Center Latitude   (deg) = ',clat0_cp
         write(iout,*)'  Center Longitude  (deg) = ',clon0_cp
         write(iout,*)'  Matching Latitude (deg) = ',clat1_cp
         write(iout,*)'  Matching Latitude (deg) = ',clat2_cp
         write(iout,*)'  False Easting      (km) = ',feast_cp
         write(iout,*)'  False Northing     (km) = ',fnorth_cp
      endif
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Coordinate System:'
      write(icsv,*)datum_cp,cm,cm,'DATUM Code'
      write(icsv,*)pmap_cp,cm,cm,'Map Projection'
      if(PMAP_cp.EQ.'UTM     ') then
         write(icsv,*)iutmzn_cp,cm,cm,'UTM Zone (1-60)'
         write(icsv,*)utmhem_cp,cm,cm,'Hemisphere'
      else
         write(icsv,*)clat0_cp,cm,'(deg)',cm,'Center Latitude'
         write(icsv,*)clon0_cp,cm,'(deg)',cm,'Center Longitude'
         write(icsv,*)clat1_cp,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)clat2_cp,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)feast_cp,cm,'(km)',cm,'False Easting'
         write(icsv,*)fnorth_cp,cm,'(km)',cm,'False Northing'
      endif
      write(icsv,*)

c --- Sources
c ---TXT---
      write(iout,'(a)')' Sources:'
      write(iout,*)'  Number of Points    = ',npts_cp
      write(iout,*)'  Number of Volumes   = ',nvols_cp
      write(iout,*)'  Number of Areas     = ',nareas_cp
      write(iout,*)'  Number of Lines     = ',nlines_cp
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Sources:'
      write(icsv,*)npts_cp,cm,'Number of Points'
      write(icsv,*)nvols_cp,cm,'Number of Volumes'
      write(icsv,*)nareas_cp,cm,'Number of Areas'
      write(icsv,*)nlines_cp,cm,'Number of Lines'
      write(icsv,*)

c --- Receptors
c ---TXT---
      write(iout,'(a)')' Receptors:'
      write(iout,*)'  Grid (ngx,ngy)      = ',ngx_cp,ngy_cp
      write(iout,*)'  Discrete            = ',ndrec_cp
      write(iout,*)'  CTSG Terrain        = ',nctrec_cp
c ---CSV---
      write(icsv,'(a)')' Receptors:'
      write(icsv,*)ngx_cp,cm,ngy_cp,cm,'Grid (ngx,ngy)'
      write(icsv,*)ndrec_cp,cm,cm,'Discrete'
      write(icsv,*)nctrec_cp,cm,cm,'CTSG Terrain'


c --- Process CALPUFF control file image
      REWIND(icfx)
      call COMP_PUFCF
      CLOSE(icfx)

100   format(1x,a48)
101   format(1x,a12,2(8x,i8),'   *')

c --- Assign data to DLL output argument array
      call DLLARG(msyr_cp,mjsday_cp,ibmon,ibday,mshr_cp,mssec_cp,
     &            ieyr,iejul,iemon,ieday,iehr,iesec,
     &            rnlat0_cp,relon0_cp,xlat1_cp,xlat2_cp,feast_cp,
     &            fnorth_cp,iutmzn_cp,axbtz,utmhem_cp,datum_cp,
     &            pmap_cp)

      return
      end

c----------------------------------------------------------------------
      subroutine getdoc_cp(io,iout,icsv,iox,ifil,ifilver,dataver)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090410         GETDOC_CP
c               D. Strimaitis
c
c --- Adapted From:
c --- POSTUTIL Version: 1.57      Level: 040818                  GETDOC
c ---          D. Strimaitis,    Earth Tech, Inc.
c
c --- PURPOSE:  Read header comment records of the CALPUFF output
c               data file, and set file type.
c
c --- UPDATES:
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Add MRUNTYPE logic branches
c
c --- V1.0 (L090302) to V1.3 (L090317) DGS
c            - Pass CALPUFF control file image to scratch file
c            - Add CSV output
c
c --- INPUTS:
c            io - integer    - Unit number for CALPUFF file
c          iout - integer    - Unit number for output TXT file
c          icsv - integer    - Unit number for output CSV file
c           iox - integer    - Unit number for scratch file
c          ifil - integer    - File number (not used)
c
c
c --- OUTPUT:
c       ifilver - integer    - Dataset version flag
c                              0: Before 2.0
c                              1: 2.0 or later
c       dataver - C*16       - Dataset version string
c
c     Common /CTRL/:
c           ncommout
c
c --- GETDOC_CP called by:  COMP_PUFF
c --- GETDOC_CP calls:      none
c----------------------------------------------------------------------
c --- Include parameters and commons
      include 'cpuff.ivu'
      include 'ctldat.ivu'
      include 'err.cal'

c --- Local Variables
      character*16 dataset,dataset0,dataver,temp16
      character*33 blank33,break33
      character*64 datamod
      character*132 comment1,blank,break
      character*1 dqt
      logical lcfx

      data blank33/'                                 '/
      data break33/'.................................'/

      data dqt/'"'/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Set break (132 characters)
      break(1:33)=break33
      break(34:66)=break33
      break(67:99)=break33
      break(100:132)=break33

c --- Read and test first record to check header format
c --- Record #1 - File Declaration -- 24 words
      read(io, err=99, end=99) dataset,dataver,datamod
      ifilver=0
      if(dataset.EQ.'CONC.DAT') then
         ifilver=1
      elseif(dataset.EQ.'DFLX.DAT') then
         ifilver=1
      elseif(dataset.EQ.'WFLX.DAT') then
         ifilver=1
      elseif(dataset.EQ.'VISB.DAT') then
         ifilver=1
      ELSEIF(dataset.EQ.'TK2D.DAT') then
         ifilver=1
      ELSEIF(dataset.EQ.'RHO2D.DAT') then
         ifilver=1
      endif

      if(ifilver.EQ.0) then
         if(mruntype.EQ.0) then
c ---       Process as Older CALPUFF dataset with no comment records
            dataver='0.0             '
            REWIND(io)
         elseif(mruntype.EQ.1) then
c ---       Dataset name does not match so return ERROR
            iret=1
            estrng='Input file Dataset Name is not CONC.DAT: '//
     &              dataset
            return
         elseif(mruntype.EQ.2) then
c ---       Dataset name does not match so return first record
            REWIND(io)
            iret=-1
            read(io) atitle_cp(1)
            estrng=atitle_cp(1)
            return
         endif
      elseif(ifilver.EQ.1) then
c ---    Number of comment records
         iohead=1
         read(io) ncom
c ---    Loop over comment records and harvest control file image
c ---    to scratch file
         lcfx=.TRUE.
         do k=1,ncom
            comment1=blank
            read(io) comment1

            if(LCFX) then
               if(comment1(1:12).EQ.'............') then
c ---             End of control file image
                  lcfx=.FALSE.
               else
                  write(iox,'(a)')TRIM(comment1)
               endif
            endif

c ---       Output
            if(iohead.EQ.1) then
              ktxt=INDEX(comment1,'-------------')
              if(ktxt.EQ.0) then
                 kl=LEN_TRIM(comment1)
                 write(iout,'(a)')'  '//comment1(1:kl)
                write(icsv,'(a)')dqt//'  '//comment1(1:kl)//dqt
              else
                 iohead=0
              endif
            endif
         enddo
      endif

c --- Prepare dataver string for testing in calling program
      temp16='                '
      k=0
      do i=1,16
         if(dataver(i:i).NE.' ') then
            k=k+1
            temp16(k:k)=dataver(i:i)
         endif
      enddo
      dataver=temp16

      return

c --- Error
c --- Dataset record format is not correct
99    iret=1
      estrng='Input file Dataset Name is not CONC.DAT: '//
     &        dataset
      return

      end

c----------------------------------------------------------------------
      subroutine comp_3d
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090729           COMP_3D
c               D. Strimaitis
c
c --- PURPOSE:  Read and report selected information from a 3D.DAT
c               or 2D.DAT file header
c
c --- UPDATES:
c --- V1.4 (L090410) to V1.5 (L090729) DGS
c            - Add branch to process 2D.DAT
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Read first and second data entries to obtain timestep
c              which may be different from 1 hour
c            NEW:     DLLINIT
c
c --- V1.1 (L090310) to V1.3 (L090317) DGS
c            - Add CSV output
c
c --- INPUTS:
c
c --- OUTPUT:
c     Common block /HEAD_3D/ variables:
c
c --- COMP_3D called by:  (main)
c --- COMP_3D calls:      GETVER, YR4
c                         INCR, INCRS, GRDAY, JULDAY, DLLARG, DLLINIT
c----------------------------------------------------------------------
c
c --- Include parameters

      include 'params.ivu'
      include '3d.ivu'
      include 'ctldat.ivu'
      include 'passdll.ivu'
      include 'err.cal'

c --- Dummy array to hold one row of 2D variables
      real dummy(1000)

      character*132 buff1,buff2,buff3
      character*12 cset,cver,clev
      character*4 cmap
      character*3 cmonth(12)
      character*1 cc,cm,ds,dqt

      data cc/':'/
      data cm/','/
      data ds/'-'/
      data dqt/'"'/

c --- Set months
      data cmonth/'Jan','Feb','Mar','Apr','May','Jun',
     &            'Jul','Aug','Sep','Oct','Nov','Dec'/

c --- Initialize data used in DLL output argument array
      call DLLINIT(ibyr_3d,ibjul_3d,ibmon_3d,ibday_3d,ibhr_3d,ibsec_3d,
     &            ieyr_3d,iejul_3d,iemon_3d,ieday_3d,iehr_3d,iesec_3d,
     &            rlatc_3d,rlonc_3d,truelat1_3d,truelat2_3d,feast_3d,
     &            fnorth_3d,iutmzn_3d,abtz_3d,utmhem_3d,datum_3d,
     &            pmap_3d)

c --- Initialize selected data
      abtz_3d='UTC+0000'
      datum_3d='NWS-84  '
      daten_3d='02-21-2003  '
      pmap_3d='LCC     '
      feast_3d=0.0
      fnorth_3d=0.0
      iutmzn_3d=0
      utmhem_3d='N'
      rlatc_3d=0.0
      rlonc_3d=0.0
      truelat1_3d=0.0
      truelat2_3d=0.0

c -------------------------------------------------------
c --- Start Output
c -------------------------------------------------------

c --- Type
      if(mfiletype.EQ.3) then
         write(iout,'(//1x,a26)')'3D.DAT Model Configuration'
         write(iout,'(1x,a26/)') '--------------------------'
         write(icsv,'(//1x,a26)')'3D.DAT Model Configuration'
      elseif(mfiletype.EQ.5) then
         write(iout,'(//1x,a26)')'2D.DAT Model Configuration'
         write(iout,'(1x,a26/)') '--------------------------'
         write(icsv,'(//1x,a26)')'2D.DAT Model Configuration'
      endif
 
c --- File
      k=LEN_TRIM(fileinp)
      write(iout,'(a)')' File Examined: '//fileinp(1:k)
      write(iout,*)
      write(icsv,'(a)')dqt//' File Examined: '//fileinp(1:k)//dqt
      write(icsv,*)

c --- Titles
      write(iout,'(a)')' Titles:'
      write(icsv,'(a)')dqt//' Titles:'//dqt


c -------------------------------------------------------
c ---    Read header information from 3D
c -------------------------------------------------------
c --- Read first two records to determine  file format 
c ---   ivs3 = 0 for MM5.DAT file structure
c ---   ivs3 = 1 for 3D.DAT file structure prior to Version 2.0
c ---   ivs3 = 2 for 3D.DAT file structure, Version 2.x
c ---   ivs3 = 3 for 3D.DAT file structure, version 3.0 or later
      read(in,'(a)')buff1
      read(in,'(a)')buff2

      call GETVER(buff1,buff2,ivs3)
      if(iret.NE.0) return

      if(ivs3.eq.0) then   ! V2 format
c ---    Record #1     
         read(buff1,'(a80,2a12)')title_3d,cver,clev
         dataver_3d=cver//'    '

c ---    Record #2
         read(buff2,'(5i3)')ioutw_3d,ioutq_3d,ioutc_3d,
     &                      iouti_3d,ioutg_3d

c ---    Record #3. Projection parameters
         read(in,1060)rlatc_3d,rlonc_3d,truelat1_3d,truelat2_3d
 1060    format(9x,f6.2,7x,f8.2,8x,f6.1,8x,f6.1)

c ---    Record #4. MPHYSICS
         read(in,'(8i3)')inhyd_3d,imphys_3d,icupa_3d,ibltyp_3d,
     &                   ifrad_3d,isoil_3d,ifddaan_3d,ifddaob_3d

c ---    Record #5
         read(in,1062)idatebeg,nhours_3d,nxsub_3d,nysub_3d,nzsub_3d
 1062    format(i8,i5,3i4)
         nz_3d=nzsub_3d   ! only nzsub layers in data
	    
c ---    Record #6
         read(in,1063)nx1_3d,ny1_3d,nx2_3d,ny2_3d,
     &                rxmin_3d,rxmax_3d,rymin_3d,rymax_3d
 1063    format(4i4,4(f8.2))

c ---    Output
c ---    ------
         kl=LEN_TRIM(title_3d)
         write(iout,'(a)')'  '//title_3d(1:kl)

      elseif(ivs3.gt.0) then  ! V3 formats
         if(ivs3.eq.1) then ! V3 format (no comment lines)
c ---       Record #1,Record #2
            read(buff1,'(a80)')title_3d
            read(buff2,'(3a12)')cset,cver,clev
            dataset_3d=cset//'    '
            dataver_3d=cver//'    '

c ---       Output
c ---       ------
            kl=LEN_TRIM(title_3d)
            write(iout,'(a)')'  '//title_3d(1:kl)

         elseif(ivs3.eq.2) then
            read(buff1,'(2a16,a64)')dataset_3d,dataver_3d,
     &                              datamod_3d
            read(buff2,*)ncomm
            do i=1,ncomm
               buff3=' '
               read(in,'(a132)') buff3
c ---          Output
c ---          ------
               kl=LEN_TRIM(buff3)
               write(iout,'(a)')'  '//buff3(1:kl)
            enddo
         else
            write(iout,*)'Dataset Format not set:',ivs3
            iret=1
            return
         endif

c ---    Record #3
         read(in,'(6i3)')ioutw_3d,ioutq_3d,ioutc_3d,
     &                   iouti_3d,ioutg_3d,iosrf_3d
c ---    Record #4
         read(in,1113)cmap,rlatc_3d,rlonc_3d,
     &                truelat1_3d,truelat2_3d,
     &                xsw_3d,ysw_3d,dxm_3d,nx_3d,ny_3d,nz_3d
 1113    format(a4,f9.4,f10.4,2f7.2,2f10.3,f8.3,2i4,i3)

         dym_3d=dxm_3d

c ---    Record #5 - physical options
         read(in,'(30i3)')inhyd_3d,imphys_3d,icupa_3d,ibltyp_3d,
     &                    ifrad_3d,isoil_3d,ifddaan_3d,ifddaob_3d,
     &                    igrdt_3d,ipbl_3d,ishf_3d,ilhf_3d,iustr_3d,
     &                    iswdn_3d,ilwdn_3d,ist1_3d,ist2_3d,ist3_3d,
     &                    ist4_3d,ist5_3d,ist6_3d
	
c ---    Record #6 - Extracted domain Stamp 1
         read(in,1115)idatebeg,nhours_3d,nxsub_3d,nysub_3d,nzsub_3d
 1115    format(i10,i5,3i4)

c ---    header record #7: - Extracted domain Stamp 2
         read(in,1116)nx1_3d,ny1_3d,nx2_3d,ny2_3d,nz1_3d,nz2_3d,
     &                rxmin_3d,rxmax_3d,rymin_3d,rymax_3d
 1116    format(6i4,2f10.4,2f9.4)

         nz_3d=nzsub_3d   ! only nzsub layers in data
         if(nz_3d.ne.(nz2_3d-nz1_3d+1)) then
            write(iout,*)'Error in vertical layers:'
            write(iout,*) nzsub_3d,nz1_3d,nz2_3d
            iret=1
            return
         endif
      endif

c --- Extracted domain range
      x1dom=xsw_3d+(nx1_3d-1)*dxm_3d
      x2dom=xsw_3d+(nx2_3d-1)*dxm_3d
      y1dom=ysw_3d+(ny1_3d-1)*dym_3d
      y2dom=ysw_3d+(ny2_3d-1)*dym_3d

c --- Read past remaining headers to the data records
c ---------------------------------------------------
c --- Sigma levels
      do i=1,nz_3d
         read(in,1064)sigma
 1064    format(f6.3)	       
      enddo
c --- Lat/long locations
      do j=ny1_3d,ny2_3d
         do i=nx1_3d,nx2_3d
            if(ivs3.eq.0) then
               read(in,1065)ii,jj,flat,flong,ihh,iland,
     &              flatcrs,flongcrs
 1065          format(2i3,f7.3,f8.3,i5,i3,1x,f7.3,f8.3)
            else
               read(in,1165)ii,jj,flat,flong,ihh,iland,
     &              flatcrs,flongcrs
 1165          format(2i4,f9.4,f10.4,i5,i3,1x,f9.4,f10.4)
            endif
         enddo
      enddo

      if(mfiletype.EQ.3) then
c ---    Read past data for first period to get timestamp of next period
c ---    This is needed to compute the ending date-time in file
c ---    Loop over cells
         do 10 j=ny1_3d,ny2_3d
         do 10 i=nx1_3d,nx2_3d
c ---       Data record with timestamp (just read up to cell index)
            isec=0
            if(ivs3.eq.0) then
               read(in,1068,end=6500)iyr,imon,iday,ihour,ii,jj
 1068          format(4i2,2i3)
            elseif(ivs3.eq.2.or.ivs3.eq.1) then
               read(in,1168,end=6500)iyr,imon,iday,ihour,ii,jj
 1168          format(i4,3i2,2i3)
            elseif(ivs3.eq.3) then
               read(in,1268,end=6500) iyrb,imonb,idayb,ihourb,isecb,
     &                                iyr,imon,iday,ihour,isec,ii,jj
 1268          format(i4,3i2,i4,i5,3i2,i4,i4,i3)
            endif
c ---       Data record for each level (free-read to skip)   
            do iz=1,nz_3d
               read(in,*)
            enddo
 10      continue
      elseif(mfiletype.EQ.5) then
c ---    Number of 2D variables
         n2d=igrdt_3d+ipbl_3d+ishf_3d+ilhf_3d+iustr_3d+iswdn_3d+
     &       ilwdn_3d+ist1_3d+ist2_3d+ist3_3d+ist4_3d+ist5_3d
c ---    Loop over 2D variables present in file
         do k=1,n2d
c ---       Data record with timestamp
            isec=0
            if(ivs3.eq.0) then
               read(in,1068,end=6500)iyr,imon,iday,ihour
            elseif(ivs3.eq.2.or.ivs3.eq.1) then
               read(in,1168,end=6500)iyr,imon,iday,ihour
            elseif(ivs3.eq.3) then
               read(in,1268,end=6500) iyrb,imonb,idayb,ihourb,isecb,
     &                                iyr,imon,iday,ihour,isec
            endif
c ---       Loop over rows in grid
            do j=ny1_3d,ny2_3d
               read(in,*)(dummy(kk),kk=1,(nx2_3d-nx1_3d+1))
            enddo
         enddo
      endif

c --- All cells have been read for first period
      call JULDAY(ilog,iyr,imon,iday,jday1)
      idate1=iyr*100000+jday1*100+ihour
      isec1=isec

c --- Read just first date record of second period
      if(mfiletype.EQ.3) then
         isec=0
         if(ivs3.eq.0) then
            read(in,1068,end=6500)iyr,imon,iday,ihour,ii,jj
         elseif(ivs3.eq.2.or.ivs3.eq.1) then
            read(in,1168,end=6500)iyr,imon,iday,ihour,ii,jj
         elseif(ivs3.eq.3) then
            read(in,1268,end=6500) iyrb,imonb,idayb,ihourb,isecb,
     &                             iyr,imon,iday,ihour,isec,ii,jj
         endif
      elseif(mfiletype.EQ.5) then
         isec=0
         if(ivs3.eq.0) then
            read(in,1068,end=6500)iyr,imon,iday,ihour
         elseif(ivs3.eq.2.or.ivs3.eq.1) then
            read(in,1168,end=6500)iyr,imon,iday,ihour
         elseif(ivs3.eq.3) then
            read(in,1268,end=6500) iyrb,imonb,idayb,ihourb,isecb,
     &                             iyr,imon,iday,ihour,isec
         endif
      endif

      call JULDAY(ilog,iyr,imon,iday,jday2)
      idate2=iyr*100000+jday2*100+ihour
      isec2=isec

c --- Time difference in seconds
      call DELTSEC(idate1,isec1,idate2,isec2,ndelsec)
c --- Must be positive difference
      if(ndelsec.LE.0) then
         write(iout,*)'Invalid Time Difference: ',ndelsec
         iret=1
         return
      endif
      nhrstep=ndelsec/3600
      nsecstep=ndelsec-nhrstep*3600

c --- Process start and end times
c --- Decode idatebeg
      ibyr_3d=idatebeg/1000000
      ibmon_3d=idatebeg/10000-ibyr_3d*100
      ibday_3d=idatebeg/100-ibyr_3d*10000-ibmon_3d*100
      ibhr_3d=idatebeg-ibyr_3d*1000000-ibmon_3d*10000-ibday_3d*100
      call JULDAY(iout,ibyr_3d,ibmon_3d,ibday_3d,ibjul_3d)
      if(iret.NE.0) return
c --- Set seconds to zero (needs to be updated for full v3.0)
      ibsec_3d=0
c --- Calculate last date-time
      ieyr_3d=ibyr_3d
      iejul_3d=ibjul_3d
      iehr_3d=ibhr_3d
      iesec_3d=ibsec_3d
      nhrinc=(nhours_3d-1)*nhrstep
      nsecinc=(nhours_3d-1)*nsecstep
      call INCR(iout,ieyr_3d,iejul_3d,iehr_3d,nhrinc)
      if(iret.NE.0) return
      call INCRS(iout,ieyr_3d,iejul_3d,iehr_3d,iesec_3d,nsecinc)
      if(iret.NE.0) return
      call GRDAY(iout,ieyr_3d,iejul_3d,iemon_3d,ieday_3d)
      if(iret.NE.0) return

c --- Create minutes and seconds
      iminb=ibsec_3d/60
      isecb=ibsec_3d-iminb*60
      imine=iesec_3d/60
      isece=iesec_3d-imine*60

c --- Set secondary flags for 3D output
      kq=ioutq_3d
      kc=kq*ioutc_3d
      ki=kc*iouti_3d
      kg=ki*ioutg_3d

c -------------------------------------
c --- Output
c -------------------------------------

c --- Time Period
c---TXT---
      write(iout,*)
      write(iout,'(a)')' Time Period:'
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data BEGIN on ',ibday_3d,cmonth(ibmon_3d),ibyr_3d,
     &   ' at ',ibhr_3d,cc,iminb,cc,isecb,' (hh:mm:ss) ',abtz_3d
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data   END on ',ieday_3d,cmonth(iemon_3d),ieyr_3d,
     &   ' at ',iehr_3d,cc,imine,cc,isece,' (hh:mm:ss) ',abtz_3d
      write(iout,*)
      write(iout,'(a21,i10)') ' Number of Steps   = ',nhours_3d
      write(iout,'(a21,2i5)') ' Timestep (hr,sec) = ',nhrstep,nsecstep
      write(iout,*)
c ---CSV---
      write(icsv,*)
      write(icsv,'(a)')dqt//' Time Period:'//dqt
      write(icsv,'(a1,a6,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'BEGIN:',dqt,cm,ibday_3d,ds,cmonth(ibmon_3d),ds,
     &   ibyr_3d,cm,ibhr_3d,cc,iminb,cc,isecb,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,abtz_3d,dqt
      write(icsv,'(a1,a4,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'END:',dqt,cm,ieday_3d,ds,cmonth(iemon_3d),ds,
     &   ieyr_3d,cm,iehr_3d,cc,imine,cc,isece,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,abtz_3d,dqt
      write(icsv,*)
      write(icsv,'(a15,a1,i10)') 'Number of Steps',cm,nhours_3d
      write(icsv,'(a16,2(a1,i5))') 'Timestep(hr-sec)',
     &                              cm,nhrstep,cm,nsecstep
      write(icsv,*)


      if(mfiletype.EQ.3) then
c --- Meteorological Variables (3D.DAT)
c---TXT---
      write(iout,'(a)')' 3-D Meteorological Variables:'
      write(iout,*) '  Horizontal Wind Speed              (m/s)'
      write(iout,*) '  Horizontal Wind Direction (true N) (deg)'
      if(ioutw_3d.NE.0)
     &   write(iout,*) '  Vertical   Wind [W]                (m/s)'
      write(iout,*) '  Air Temperature                    (K)'
      write(iout,*) '  Pressure                           (millibar)'
      if(kq.NE.0) then
         write(iout,*) '  Relative Humidity                  (%)'
         write(iout,*) '  Vapor Mixing Ratio                 (g/kg)'
      endif
      if(kc.NE.0) then
         write(iout,*) '  Cloud Mixing Ratio                 (g/kg)'
         write(iout,*) '  Rain Mixing Ratio                  (g/kg)'
      endif
      if(ki.NE.0) then
         write(iout,*) '  Ice Mixing Ratio                   (g/kg)'
         write(iout,*) '  Snow Mixing Ratio                  (g/kg)'
      endif
      if(kg.NE.0) then
         write(iout,*) '  Graupel Mixing Ratio               (g/kg)'
      endif
      write(iout,*)
      write(iout,'(a)')' 2-D Meteorological Variables:'
      write(iout,*) '  Sea-level Pressure                 (hPa)'
      write(iout,*) '  Rainfall in last hour              (cm)'
      write(iout,*) '  Snow Cover flag                    (0,1)'
      write(iout,*) '  Surface Short-wave Solar Radiation (W/m2)'
      write(iout,*) '  Long-wave Radiation at Top         (W/m2)'
      write(iout,*) '  Air Temperature at 2m              (K)'
      write(iout,*) '  Specific Humidity at 2m            (g/kg)'
      write(iout,*) '  Wind Speed at 2m                   (m/s)'
      write(iout,*) '  Wind Direction at 2m               (deg)'
      write(iout,*) '  Sea-Surface/Ground Temperature     (K)'
      write(iout,*)
c---CSV---
      write(icsv,'(a)')' 3-D Meteorological Variables:'
      write(icsv,*) '  Horizontal Wind Speed              (m/s)'
      write(icsv,*) '  Horizontal Wind Direction (true N) (deg)'
      if(ioutw_3d.NE.0)
     &   write(icsv,*) '  Vertical   Wind [W]                (m/s)'
      write(icsv,*) '  Air Temperature                    (K)'
      write(icsv,*) '  Pressure                           (millibar)'
      if(kq.NE.0) then
         write(icsv,*) '  Relative Humidity                  (%)'
         write(icsv,*) '  Vapor Mixing Ratio                 (g/kg)'
      endif
      if(kc.NE.0) then
         write(icsv,*) '  Cloud Mixing Ratio                 (g/kg)'
         write(icsv,*) '  Rain Mixing Ratio                  (g/kg)'
      endif
      if(ki.NE.0) then
         write(icsv,*) '  Ice Mixing Ratio                   (g/kg)'
         write(icsv,*) '  Snow Mixing Ratio                  (g/kg)'
      endif
      if(kg.NE.0) then
         write(icsv,*) '  Graupel Mixing Ratio               (g/kg)'
      endif
      write(icsv,*)
      write(icsv,'(a)')' 2-D Meteorological Variables:'
      write(icsv,*) '  Sea-level Pressure                 (hPa)'
      write(icsv,*) '  Rainfall in last hour              (cm)'
      write(icsv,*) '  Snow Cover flag                    (0,1)'
      write(icsv,*) '  Surface Short-wave Solar Radiation (W/m2)'
      write(icsv,*) '  Long-wave Radiation at Top         (W/m2)'
      write(icsv,*) '  Air Temperature at 2m              (K)'
      write(icsv,*) '  Specific Humidity at 2m            (g/kg)'
      write(icsv,*) '  Wind Speed at 2m                   (m/s)'
      write(icsv,*) '  Wind Direction at 2m               (deg)'
      write(icsv,*) '  Sea-Surface/Ground Temperature     (K)'
      write(icsv,*)
      endif



      if(mfiletype.EQ.5) then
c --- Meteorological Variables (2D.DAT)
c---TXT---
      write(iout,'(a)')' 2-D Meteorological Variables:'
      if(igrdt_3d.EQ.1)
     &   write(iout,*) '  Ground Temperature                 (K)'
      if(ipbl_3d.EQ.1)
     &   write(iout,*) '  PBL Height                         (m)'
      if(ishf_3d.EQ.1)
     &   write(iout,*) '  Sensible Heat Flux                 (W/m2)'
      if(ilhf_3d.EQ.1)
     &   write(iout,*) '  Latent Heat Flux                   (W/m2)'
      if(iustr_3d.EQ.1)
     &   write(iout,*) '  Friction Velocity                  (m/s)'
      if(iswdn_3d.EQ.1)
     &   write(iout,*) '  Surface Short-wave Solar Radiation (W/m2)'
      if(ilwdn_3d.EQ.1)
     &   write(iout,*) '  Long-wave Radiation Flux           (W/m2)'
      if(ist1_3d.EQ.1)
     &   write(iout,*) '  Air Temperature at 2m              (K)'
      if(ist2_3d.EQ.1)
     &   write(iout,*) '  Specific Humidity at 2m            (g/kg)'
      if(ist3_3d.EQ.1)
     &   write(iout,*) '  U-Wind Speed at 2m                 (m/s)'
      if(ist4_3d.EQ.1)
     &   write(iout,*) '  V-Wind Speed at 2m                 (m/s)'
      if(ist5_3d.EQ.1)
     &   write(iout,*) '  Sea-Surface Temperature            (K)'
      write(iout,*)
c---CSV---
      write(icsv,'(a)')' 2-D Meteorological Variables:'
      if(igrdt_3d.EQ.1)
     &   write(icsv,*) '  Ground Temperature                 (K)'
      if(ipbl_3d.EQ.1)
     &   write(icsv,*) '  PBL Height                         (m)'
      if(ishf_3d.EQ.1)
     &   write(icsv,*) '  Sensible Heat Flux                 (W/m2)'
      if(ilhf_3d.EQ.1)
     &   write(icsv,*) '  Latent Heat Flux                   (W/m2)'
      if(iustr_3d.EQ.1)
     &   write(icsv,*) '  Friction Velocity                  (m/s)'
      if(iswdn_3d.EQ.1)
     &   write(icsv,*) '  Surface Short-wave Solar Radiation (W/m2)'
      if(ilwdn_3d.EQ.1)
     &   write(icsv,*) '  Long-wave Radiation Flux           (W/m2)'
      if(ist1_3d.EQ.1)
     &   write(icsv,*) '  Air Temperature at 2m              (K)'
      if(ist2_3d.EQ.1)
     &   write(icsv,*) '  Specific Humidity at 2m            (g/kg)'
      if(ist3_3d.EQ.1)
     &   write(icsv,*) '  U-Wind Speed at 2m                 (m/s)'
      if(ist4_3d.EQ.1)
     &   write(icsv,*) '  V-Wind Speed at 2m                 (m/s)'
      if(ist5_3d.EQ.1)
     &   write(icsv,*) '  Sea-Surface Temperature            (K)'
      write(icsv,*)
      endif



c --- Grid
c---TXT---
      write(iout,'(a)')' Grid Configuration (original model grid):'
      write(iout,*)'  NX Cells        = ',nx_3d
      write(iout,*)'  NY Cells        = ',ny_3d
      write(iout,*)'  Cell Size  (km) = ',dxm_3d
      write(iout,*)'  SW Corner  (km) = ',xsw_3d,ysw_3d
      write(iout,*)'  Vertical Layers = ',nz_3d
      write(iout,*)
      write(iout,'(a)')' Grid Configuration (available subgrid):'
      write(iout,*)'  NX Cells        = ',nxsub_3d
      write(iout,*)'  NY Cells        = ',nysub_3d
      write(iout,*)'  Cell Size  (km) = ',dxm_3d
      write(iout,*)'  I index range   = ',nx1_3d,nx2_3d
      write(iout,*)'  J index range   = ',ny1_3d,ny2_3d
      write(iout,*)'  SW Corner  (km) = ',x1dom,y1dom
      write(iout,*)'  NE Corner  (km) = ',x2dom,y2dom
      write(iout,*)'  Vertical Layers = ',nzsub_3d
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Grid Configuration:'
      write(icsv,*)nx_3d,cm,cm,'NX Cells'
      write(icsv,*)ny_3d,cm,cm,'NY Cells'
      write(icsv,*)dxm_3d,cm,cm,'Cell Size (km)'
      write(icsv,*)xsw_3d,cm,ysw_3d,cm,'SW Corner (km)'
      write(icsv,*)nz_3d,cm,cm,'Vertical Layers'
      write(icsv,*)
      write(icsv,'(a)')' Grid Configuration (available subgrid):'
      write(icsv,*)nxsub_3d,cm,cm,'NX Cells'
      write(icsv,*)nysub_3d,cm,cm,'NY Cells'
      write(icsv,*)dxm_3d,cm,cm,'Cell Size (km)'
      write(icsv,*)nx1_3d,cm,nx2_3d,cm,'I index range'
      write(icsv,*)ny1_3d,cm,ny2_3d,cm,'J index range'
      write(icsv,*)x1dom,cm,y1dom,cm,'SW Corner (km)'
      write(icsv,*)x2dom,cm,y2dom,cm,'NE Corner (km)'
      write(icsv,*)nzsub_3d,cm,cm,'Vertical Layers'
      write(icsv,*)

c --- Coordinate System
c---TXT---
      write(iout,'(a)')' Coordinate System:'
      write(iout,*)'  DATUM Code              = ',datum_3d
      write(iout,*)'  Map Projection          = ',pmap_3d
      if(PMAP_3D.EQ.'UTM     ') then
         write(iout,*)'  UTM Zone         (1-60) = ',iutmzn_3d
         write(iout,*)'  Hemisphere              = ',utmhem_3d
      else
         write(iout,*)'  Center Latitude   (deg) = ',rlatc_3d
         write(iout,*)'  Center Longitude  (deg) = ',rlonc_3d
         write(iout,*)'  Matching Latitude (deg) = ',truelat1_3d
         write(iout,*)'  Matching Latitude (deg) = ',truelat2_3d
         write(iout,*)'  False Easting      (km) = ',feast_3d
         write(iout,*)'  False Northing     (km) = ',fnorth_3d
      endif
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Coordinate System:'
      write(icsv,*)datum_3d,cm,cm,'DATUM Code'
      write(icsv,*)pmap_3d,cm,cm,'Map Projection'
      if(PMAP_3d.EQ.'UTM     ') then
         write(icsv,*)iutmzn_3d,cm,cm,'UTM Zone (1-60)'
         write(icsv,*)utmhem_3d,cm,cm,'Hemisphere'
      else
         write(icsv,*)rlatc_3d,cm,'(deg)',cm,'Center Latitude'
         write(icsv,*)rlonc_3d,cm,'(deg)',cm,'Center Longitude'
         write(icsv,*)truelat1_3d,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)truelat2_3d,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)feast_3d,cm,'(km)',cm,'False Easting'
         write(icsv,*)fnorth_3d,cm,'(km)',cm,'False Northing'
      endif
      write(icsv,*)

c --- Assign data to DLL output argument array
      call DLLARG(ibyr_3d,ibjul_3d,ibmon_3d,ibday_3d,ibhr_3d,ibsec_3d,
     &            ieyr_3d,iejul_3d,iemon_3d,ieday_3d,iehr_3d,iesec_3d,
     &            rlatc_3d,rlonc_3d,truelat1_3d,truelat2_3d,feast_3d,
     &            fnorth_3d,iutmzn_3d,abtz_3d,utmhem_3d,datum_3d,
     &            pmap_3d)

      return

6500  write(iout,*)'Premature End-of-File Encountered'
      iret=1
      estrng='Premature End-of-File Encountered'
      return

      end

c----------------------------------------------------------------------
	Subroutine getver(buff1,buff2,ivs3)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090729           GETVER
c
c --- Adapted From:
c
c --- METSERIES   Version: 1.6       Level: 060615              GETVER
C     Zhong-Xiang Wu
C     1/16/2004
C
C --- Purpose: Get 3D.DAT format/version number
c
c --- UPDATES:
c --- V1.4 (L090410) to V1.5 (L090729) DGS
c            - Add 2D.DAT
c --- V1.1 (L090310) to V1.4 (L090410) DGS
c            - Add MRUNTYPE logic branches
C 
C --- Input: BUFF1, BUFF2 - First two records
C --- Output: ivs3 - 3D.DAT (2D.DAT) format key number
C --- Called by: Main
C --- Calling: None
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'ctldat.ivu'
      include 'err.cal'

      character*132 buff1,buff2
      character*12 cset3d

      ivs3=-999
      ncomm=-999

c --- Step (1): Check for "3D.DAT" or "MM53D.DAT" on second 
c --- record of file (indication of old 3D.DAT file structure
c --- prior to Version 2.0)
      read(buff2,'(a12)')cset3d

      if(cset3d.eq.'3D.DAT'.or.cset3d.eq.'MM53D.DAT')then
         ivs3=1
      elseif(cset3d.eq.'2D.DAT'.or.cset3d.eq.'MM53D.DAT')then
         ivs3=1
      else

c ---    Step (2) check for standard 3D.DAT file structure 
c ---    with dataset name, version and comments on first record
         if(buff1(1:6).eq.'3D.DAT' .OR. buff1(1:6).eq.'2D.DAT')then
            read(buff2,*,err=1000)ncomm
            if(ncomm.ge.1) then
               ivs3=2
c ---          Version 3.x (explicit beg/end times with seconds)
               if(buff1(17:17).eq.'3')ivs3=3
               goto 2000
            endif
 1000       continue		! no comment lines, assume old MM5 format
            write(iout,*)
     &        '3D(or 2D).DAT comment line format but no comment lines'
            write(iout,*)'Check/correct the 3D.DAT'
            iret=1
            return
         else

c ---       Structure does not fit 3D.DAT conventions -- assume
c ---       file is in MM5.DAT format
            if(mruntype.EQ.0) then
               ivs3=0
            elseif(mruntype.EQ.1) then
c ---          Dataset name does not match so return ERROR
               iret=1
               estrng='Input file does not fit 3D.DAT conventions'
               return
            elseif(mruntype.EQ.2) then
c ---          File may be MM5.DAT structure
               iret=-1
               estrng=buff1(1:80)
               return
            endif
         endif
      endif

c 2000	print *,'File format - ivs3/ncom = ',ivs3, ncomm
2000  continue

      if(ivs3.lt.0 .or. ivs3.gt.3) then
         write(iout,*)'Illegal 3D(or 2D).DAT format. Check data files'
         iret=1
         return
      endif
 
      return
      end

c----------------------------------------------------------------------
      subroutine DLLARG(ibyr,ibjul,ibmon,ibday,ibhr,ibsec,
     &                  ieyr,iejul,iemon,ieday,iehr,iesec,
     &                  rnlatc,relonc,rnlat1,rnlat2,rfeast,rfnorth,
     &                  iutmzn,ctzone,cutmhem,cdatum,cpmap)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090310           DLLARG
c               D. Strimaitis
c
c --- PURPOSE:  Pass selected data into common used to create arguments
c               for output in the DLL argument list.
c
c --- INPUTS:
c
c --- OUTPUT:
c     Common block /PASSDLL/ variables:
c        kutmzn,
c        kbyr,kbjul,kbmon,kbday,kbhr,kbsec,
c        keyr,kejul,kemon,keday,kehr,kesec,
c        xnlatc,xelonc,xnlat1,xnlat2,xfeast,xfnorth,
c        ktzone,kpmap,kutmhem,kdatum
c
c --- DLLARG called by:  COMP_MET, COMP_PUF, COMP_3D, COMP_WDA
c --- DLLARG calls:      none
c----------------------------------------------------------------------
c
c --- Include parameters

      include 'params.ivu'
      include 'passdll.ivu'

      character*8 ctzone
      character*4 cutmhem
      character*8 cdatum,cpmap

      integer ibyr,ibjul,ibmon,ibday,ibhr,ibsec
      integer ieyr,iejul,iemon,ieday,iehr,iesec
      integer iutmzn

      real rnlatc,relonc,rnlat1,rnlat2
      real rfeast,rfnorth

c --- Assign inputs to outputs
      kutmzn=iutmzn
      kbyr=ibyr
      kbjul=ibjul
      kbmon=ibmon
      kbday=ibday
      kbhr=ibhr
      kbsec=ibsec
      keyr=ieyr
      kejul=iejul
      kemon=iemon
      keday=ieday
      kehr=iehr
      kesec=iesec
      xnlatc=rnlatc
      xelonc=relonc
      xnlat1=rnlat1
      xnlat2=rnlat2
      xfeast=rfeast
      xfnorth=rfnorth
      ktzone=ctzone
      kpmap=cpmap
      kutmhem=cutmhem
      kdatum=cdatum

      return
      end

c----------------------------------------------------------------------
      subroutine DLLINIT(ibyr,ibjul,ibmon,ibday,ibhr,ibsec,
     &                  ieyr,iejul,iemon,ieday,iehr,iesec,
     &                  rnlatc,relonc,rnlat1,rnlat2,rfeast,rfnorth,
     &                  iutmzn,ctzone,cutmhem,cdatum,cpmap)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090410          DLLINIT
c               D. Strimaitis
c
c --- PURPOSE:  Initialize variables used to create arguments
c               for output in the DLL argument list.
c
c --- INPUTS:
c
c --- OUTPUT:
c      ibyr,ibjul,ibmon,ibday,ibhr,ibsec,
c      ieyr,iejul,iemon,ieday,iehr,iesec,
c      rnlatc,relonc,rnlat1,rnlat2,rfeast,rfnorth,
c      utmzn,ctzone,cutmhem,cdatum,cpmap
c
c --- DLLINIT called by:  COMP_MET, COMP_PUF, COMP_3D, COMP_WDA
c --- DLLINIT calls:      none
c----------------------------------------------------------------------

      character*8 ctzone
      character*4 cutmhem
      character*8 cdatum,cpmap

      integer ibyr,ibjul,ibmon,ibday,ibhr,ibsec
      integer ieyr,iejul,iemon,ieday,iehr,iesec
      integer iutmzn

      real rnlatc,relonc,rnlat1,rnlat2
      real rfeast,rfnorth

c --- Assign initial values
      iutmzn=0
      ibyr=0
      ibjul=0
      ibmon=0
      ibday=0
      ibhr=0
      ibsec=0
      ieyr=0
      iejul=0
      iemonv=0
      ieday=0
      iehr=0
      iesec=0
      rnlatc=0.0
      relonc=0.0
      rnlat1=0.0
      rnlat2=0.0
      rfeast=0.0
      rfnorth=0.0
      ctzone='UTM+hhmm'
      cpmap='NONE    '
      cutmhem='X   '
      cdatum='NONE    '

      return
      end

c----------------------------------------------------------------------
      subroutine comp_wda
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090410          COMP_WDA
c               D. Strimaitis
c
c --- PURPOSE:  Read and report selected information from DA wind
c               file header
c
c --- UPDATES:
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Initialize DLL variables
c            NEW:     DLLINIT
c --- V1.2 (L090312) to V1.3 (L090317) DGS
c            - Create CSV output
c
c --- INPUTS:
c     Common block /HEAD_DA/ variables:
c      krecl_da
c
c --- OUTPUT:
c     Common block /HEAD_DA/ variables:
c      mode_da,nheader_da,iconfig_da,ncoml_da,
c      nx_da,ny_da,nz_da,dgrid_da,dgridi_da,xorig_da,yorig_da,
c      nsstn_da,nustn_da,kwat1_da,kwat2_da,zface_da(mxnzp1_da),
c      xsstn_da(mxss_da),ysstn_da(mxss_da),xustn_da(mxus_da),
c      yustn_da(mxus_da),z0a_da(mxnx_da,mxny_da),
c      ilandua_da(mxnx_da,mxny_da),eleva_da(mxnx_da,mxny_da),
c      xlaia_da(mxnx_da,mxny_da),irlg_da,nsecdt_da,hrdt_da,
c      ihrmet2tr_da,
c      ibyr_da,ibmo_da,ibdy_da,ibjd_da,ibhr_da,ibsec_da,ibdathr_da,
c      ieyr_da,iemo_da,iedy_da,iejd_da,iehr_da,iesec_da,iedathr_da,
c      rnlat0_da,relon0_da,rnlat1_da,rnlat2_da,iutmzn_da,
c      feast_da,fnorth_da,pmap_da,datum_da,daten_da,utmhem_da,
c      clat0_da,clon0_da,clat1_da,clat2_da,axbtz_da,
c      dataver_da,dataset_da,datamod_da
c
c --- COMP_WDA called by:  (main)
c --- COMP_WDA calls:      JULDAY, XTRACTLL, DLLARG, DLLINIT
c----------------------------------------------------------------------
c
c --- Include parameters

      include 'params.ivu'
      include 'wda.ivu'
      include 'ctldat.ivu'
      include 'passdll.ivu'
      include 'err.cal'


c --- Local Variables
      character*3 cmonth(12)
      character*1 cc,cm,ds,dqt

c --- Set months
      data cmonth/'Jan','Feb','Mar','Apr','May','Jun',
     &            'Jul','Aug','Sep','Oct','Nov','Dec'/

      data cc/':'/
      data cm/','/
      data ds/'-'/
      data dqt/'"'/

c --- Initialize data used in DLL output argument array
      call DLLINIT(ibyr_da,ibjd_da,ibmo_da,ibdy_da,ibhr_da,ibsec_da,
     &            ieyr_da,iejd_da,iemo_da,iedy_da,iehr_da,iesec_da,
     &            rnlat0,relon0,rnlat1,rnlat2,feast_da,
     &            fnorth_da,iutmzn_da,axbtz_da,utmhem_da,datum_da,
     &            pmap_da)

c --- Get enough information from the first record to tell
c --- if this is a 1D or 2D file, requiring different reads
      read(in,rec=1,err=99) krecl_da,dataset_da
      if(dataset_da.EQ.'WIND1D.DAT      ') then
         mode_da=1
      elseif(dataset_da.EQ.'WIND2D.DAT      ') then
         mode_da=2
      else
         goto 99
      endif

c -------------------------------------------------------
c --- Start Output
c -------------------------------------------------------

c --- Type
      if(mode_da.EQ.1) then
         write(iout,'(//1x,a26)')'WIND1D Model Configuration'
         write(icsv,'(//1x,a26)')'WIND1D Model Configuration'
      elseif(mode_da.EQ.2) then
         write(iout,'(//1x,a26)')'WIND2D Model Configuration'
         write(icsv,'(//1x,a26)')'WIND2D Model Configuration'
      endif
      write(iout,'(1x,a26/)') '--------------------------'
 
c --- File
      k=LEN_TRIM(fileinp)
      write(iout,'(a)')' File Examined: '//fileinp(1:k)
      write(iout,*)
      write(icsv,'(a)')dqt//' File Examined: '//fileinp(1:k)//dqt
      write(icsv,*)

c --- Read records
c ----------------

      if(mode_da.EQ.1) then
c ---    1D Record Structure
c ---    -------------------
c ---    First record
         read(in,rec=1) krecl_da,dataset_da,ncoml_da,
     &                  nheader_da,iconfig_da
c ---    Header Documentation records
c ---    Doc Rec 1
         read(in,rec=2) dataver_da
c ---    Doc Rec 2
         read(in,rec=3) datamod_da(1:32)
c ---    Doc Rec 3
         read(in,rec=4) datamod_da(33:64)
c ---    Model configuration records
         irec=iconfig_da
         read(in,rec=irec) nsecdt_da,irlg_da,ibyr_da,ibmo_da,ibdy_da,
     &                     ibhr_da,ibsec_da
         irec=irec+1
         read(in,rec=irec) ieyr_da,iemo_da,iedy_da,
     &                     iehr_da,iesec_da,axbtz_da
         irec=irec+1
         read(in,rec=irec) nx_da,ny_da,dgrid_da,xorig_da,yorig_da,
     &                     feast_da,fnorth_da
         irec=irec+1
         read(in,rec=irec) pmap_da,datum_da,daten_da
         irec=irec+1
         read(in,rec=irec) utmhem_da,iutmzn_da
         irec=irec+1
         read(in,rec=irec) clat0_da,clon0_da
         irec=irec+1
         read(in,rec=irec) clat1_da,clat2_da
         nz_da=1
c ---    One data record per time
         nrecord=nheader_da+irlg_da
      else
c ---    2D Record Structure
c ---    -------------------
c ---    First record
         read(in,rec=1) krecl_da,dataset_da,dataver_da,datamod_da,
     &                  ncoml_da,nheader_da,iconfig_da,
     &                  ncalmet_da,method_da,layerlo_da,layerup_da,
     &                  heightlo_da,heightup_da,mavg_da
c ---    Model configuration record
         read(in,rec=iconfig_da) 
     1      ibyr_da,ibmo_da,ibdy_da,ibhr_da,ibsec_da,
     2      ieyr_da,iemo_da,iedy_da,iehr_da,iesec_da,
     3      axbtz_da,irlg_da,nsecdt_da,
     4      nx_da,ny_da,nz_da,dgrid_da,xorig_da,yorig_da,
     5      nsstn_da,nustn_da,kwat1_da,kwat2_da,
     6      pmap_da,datum_da,daten_da,feast_da,fnorth_da,
     7      utmhem_da,iutmzn_da,
     8      clat0_da,clon0_da,clat1_da,clat2_da
c ---    Two data records per time
         nrecord=nheader_da+2*irlg_da
      endif

      call JULDAY(iout,ibyr_da,ibmo_da,ibdy_da,ibjd_da)
      call JULDAY(iout,ieyr_da,iemo_da,iedy_da,iejd_da)
      ibdathr_da=ibyr_da*100000+ibjd_da*100+ibhr_da
      iedathr_da=ieyr_da*100000+iejd_da*100+iehr_da

c --- Reciprocal grid cell size
      dgridi_da=1./dgrid_da

c --- Convert selected meters to kilometers
      xgridkm=dgrid_da*0.001
      xorigkm=xorig_da*0.001
      yorigkm=yorig_da*0.001
c --- Set Upper Right met grid corner (km)
      xUR=xorigkm+FLOAT(nx_da)*xgridkm
      yUR=yorigkm+FLOAT(ny_da)*xgridkm

c --- Process lat/lon character variables to extract content
      if(clat1_da.EQ.'                ') clat1_da='0.0N'
      if(clat2_da.EQ.'                ') clat2_da='0.0N'
      if(clat0_da.EQ.'                ') clat0_da='0.0N'
      if(clon0_da.EQ.'                ') clon0_da='0.0E'
      call XTRACTLL(iout,'LAT ',clat1_da,rnlat1)
      call XTRACTLL(iout,'LAT ',clat2_da,rnlat2)
      call XTRACTLL(iout,'LAT ',clat0_da,rnlat0)
      call XTRACTLL(iout,'LON ',clon0_da,relon0)


c -------------------------------------
c --- Output
c -------------------------------------

c --- Create minutes and seconds from 3600 seconds
      iminb=ibsec_da/60
      isecb=ibsec_da-iminb*60
      imine=iesec_da/60
      isece=iesec_da-imine*60

c --- Time Period
c ---TXT---
      write(iout,*)
      write(iout,'(a)')' Time Period:'
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data BEGIN on ',ibdy_da,cmonth(ibmo_da),ibyr_da,
     &   ' at ',ibhr_da,cc,iminb,cc,isecb,' (hh:mm:ss) ',axbtz_da
      write(iout,'(a15,i3.2,1x,a3,i5.4,a4,2(i2.2,a1),i2.2,2a12)')
     &   '  Data   END on ',iedy_da,cmonth(iemo_da),ieyr_da,
     &   ' at ',iehr_da,cc,imine,cc,isece,' (hh:mm:ss) ',axbtz_da
      write(iout,*)
      write(iout,'(a25,i10)') ' Number of model steps  = ',irlg_da
      write(iout,'(a25,i10)') ' Number of seconds/step = ',nsecdt_da
      write(iout,*)
c ---CSV---
      write(icsv,*)
      write(icsv,'(a)')dqt//' Time Period:'//dqt
      write(icsv,'(a1,a6,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'BEGIN:',dqt,cm,ibdy_da,ds,cmonth(ibmo_da),ds,
     &   ibyr_da,cm,ibhr_da,cc,iminb,cc,isecb,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,axbtz_da,dqt
      write(icsv,'(a1,a4,2a1,i3.2,a1,a3,a1,i4,a1,
     &             2(i2.2,a1),i2.2,2a1,a12,3a1,a12,a1)')
     &   dqt,'END:',dqt,cm,iedy_da,ds,cmonth(iemo_da),ds,
     &   ieyr_da,cm,iehr_da,cc,imine,cc,isece,cm,dqt,
     &   ' (hh:mm:ss) ',dqt,cm,dqt,axbtz_da,dqt
      write(icsv,*)
      write(icsv,'(a13,a1,i10)') '# Model Steps',cm,irlg_da
      write(icsv,'(a10,a1,i10)') '# Sec/Step',cm,nsecdt_da
      write(icsv,*)

c --- Meteorological Variables
c ---TXT---
      write(iout,'(a)')' Meteorological Variables:'
      write(iout,*) '  Horizontal Wind [U,V]              (m/s)'
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Meteorological Variables:'
      write(icsv,*) '  Horizontal Wind [U-V]              (m/s)'
      write(icsv,*)

c --- Grid
c ---TXT---
      write(iout,'(a)')' Grid Configuration:'
      write(iout,*)'  NX Cells        = ',nx_da
      write(iout,*)'  NY Cells        = ',ny_da
      write(iout,*)'  Cell Size (km)  = ',xgridkm
      write(iout,*)'  SW Corner (km)  = ',xorigkm,yorigkm
      write(iout,*)'  NE Corner (km)  = ',xUR,yUR
      write(iout,*)'  Vertical Layers = ',nz_da
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Grid Configuration:'
      write(icsv,*)nx_da,cm,cm,'NX Cells'
      write(icsv,*)ny_da,cm,cm,'NY Cells'
      write(icsv,*)xgridkm,cm,cm,'Cell Size (km)'
      write(icsv,*)xorigkm,cm,yorigkm,cm,'SW Corner (km)'
      write(icsv,*)xUR,cm,yUR,cm,'SE Corner (km)'
      write(icsv,*)nz_da,cm,cm,'Vertical Layers'
      write(icsv,*)


c --- Coordinate System
c ---TXT---
      write(iout,'(a)')' Coordinate System:'
      write(iout,*)'  DATUM Code              = ',datum_da
      write(iout,*)'  Map Projection          = ',pmap_da
      if(PMAP_da.EQ.'UTM     ') then
         write(iout,*)'  UTM Zone (1-60)         = ',iutmzn_da
         write(iout,*)'  Hemisphere              = ',utmhem_da
      else
         write(iout,*)'  Center Latitude   (deg) = ',clat0_da
         write(iout,*)'  Center Longitude  (deg) = ',clon0_da
         write(iout,*)'  Matching Latitude (deg) = ',clat1_da
         write(iout,*)'  Matching Latitude (deg) = ',clat2_da
         write(iout,*)'  False Easting      (km) = ',feast_da
         write(iout,*)'  False Northing     (km) = ',fnorth_da
      endif
      write(iout,*)
c ---CSV---
      write(icsv,'(a)')' Coordinate System:'
      write(icsv,*)datum_da,cm,cm,'DATUM Code'
      write(icsv,*)pmap_da,cm,cm,'Map Projection'
      if(PMAP_da.EQ.'UTM     ') then
         write(icsv,*)iutmzn_da,cm,cm,'UTM Zone (1-60)'
         write(icsv,*)utmhem_da,cm,cm,'Hemisphere'
      else
         write(icsv,*)clat0_da,cm,'(deg)',cm,'Center Latitude'
         write(icsv,*)clon0_da,cm,'(deg)',cm,'Center Longitude'
         write(icsv,*)clat1_da,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)clat2_da,cm,'(deg)',cm,'Matching Latitude'
         write(icsv,*)feast_da,cm,'(km)',cm,'False Easting'
         write(icsv,*)fnorth_da,cm,'(km)',cm,'False Northing'
      endif
      write(icsv,*)


c --- Assign data to DLL output argument array
      call DLLARG(ibyr_da,ibjd_da,ibmo_da,ibdy_da,ibhr_da,ibsec_da,
     &            ieyr_da,iejd_da,iemo_da,iedy_da,iehr_da,iesec_da,
     &            rnlat0,relon0,rnlat1,rnlat2,feast_da,
     &            fnorth_da,iutmzn_da,axbtz_da,utmhem_da,datum_da,
     &            pmap_da)

      return

99    write(iout,*)'COMP_WDA: Met dataset name invalid: ',dataset_da
      write(iout,*)'   Expected: WIND1D.DAT or WIND2D.DAT'
      iret=1
      estrng='WIND1D/WIND2D dataset name invalid: '//dataset_da
      return

      end

c----------------------------------------------------------------------
      subroutine srcinit
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          SRCINIT
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 060928                SRCINIT
c ---           D. Strimaitis
c
c --- PURPOSE:  Initializes source arrays
c
c --- UPDATES:
c --- Version 1.1 Level 060922 to Version 1.2 Level 060928   (DGS)
c            - Add text version of input variables
c
c --- INPUTS:
c
c
c --- OUTPUT:
c
c     Common block /SOURCE/
c        x,y,zht,zbase,diam,vel,tempk,sigy,sigz,
c        qrate,bdw,fmf,zplt,iutmzn,
c        ax,ay,azht,azbase,adiam,avel,atempk,asigy,asigz,
c        aqrate,abdw,afmf,azplt,
c        name,state,class,order,type,
c
c --- SRCINIT called by:  COMP_METCF
c --- SRCINIT calls:      none
c----------------------------------------------------------------------
      include 'cfdoc.ivu'

      do i=1,mxsrc_cf
         do k=1,2
            name(k,i)=' '
         enddo
         type(i)='NONE    '
         x(i)=-999.
         y(i)=-999.
         zht(i)=-999.
         zbase(i)=-999.
         diam(i)=-999.
         vel(i)=-999.
         tempk(i)=-999.
         sigy(i)=0.
         sigz(i)=0.
         bdw(i)=0.
         fmf(i)=1.
         zplt(i)=0.
         do k=1,mxspec_cf
            qrate(k,i)=0.
         enddo

         ax(i)    ='      -    '
         ay(i)    ='      -    '
         azht(i)  ='      -   '
         azbase(i)='      -   '
         adiam(i) ='      -   '
         avel(i)  ='      -   '
         atempk(i)='      -   '
         asigy(i) ='       -  '
         asigz(i) ='       -  '
         abdw(i)  ='    -  '
         afmf(i)  ='    -  '
         azplt(i) ='      -   '
         do k=1,mxspec_cf
            aqrate(k,i)='         -    '
         enddo

      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine runinit
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          RUNINIT
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113                RUNINIT
c ---           D. Strimaitis
c
c --- PURPOSE:  Initializes run information arrays
c
c --- UPDATES:
c --- Version 1.2 Level 060928 to Version 2.0 Level 081113   (DGS)
c            - Add CALMET processing variables
c
c --- INPUTS:
c
c
c --- OUTPUT:
c
c     Common block /OPTION/
c        mpuffopt, mmetopt
c     Common block /TIME/
c        pufftim, mettim
c     Common block /RECEPTOR/
c        puffrec
c
c --- RUNINIT called by:  COMP_METCF
c --- RUNINIT calls:      none
c----------------------------------------------------------------------
      include 'cfdoc.ivu'

      do i=1,mxopt_cf        
         do j=1,mxfil_cf
            mpuffopt(i,j)='-     '
            mmetopt(i,j)='-     '
            mmetpvl(i,j)='-                                           '
         enddo
      enddo

      do i=1,mxval_cf
         do j=1,mxfil_cf
            pufftim(i,j)='-       '
            puffrec(i,j)='-       '
            puffmap(i,j)='NONE'
            mettim(i,j)='-       '
            metmap(i,j)='NONE'
         enddo
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine comp_metcf
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317       COMP_METCF
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113               COMP_MET
c ---           D. Strimaitis
c
c --- PURPOSE:  Process CALMET control file image
c
c --- INPUTS:
c
c --- OUTPUT:
c
c --- COMP_METCF called by: COMP_MET
c --- COMP_METCF calls:     RDMETCF, DOCMAP, DOCFILL
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'cfdoc.ivu'
      include 'err.cal'

c --- Local real variables
      real zfacex(11),biasx(10),fextrp2x(10)

c --- Local integer variables
      integer nsmthx(10),nintr2x(10)
      integer lenname(4)

c --- Local logical variables
      logical lerror
      logical lkeep

c --- Local string variables
      character*1 cm,xp,dqt
      character*1 copt1(mxopt_cf,7)
      character*24 sname
      character*40 a40,b40

      data cm/','/
      data xp/'!'/
      data b40/'                                        '/
      data dqt/'"'/

c --- Set up for 1 CALMET file
      mcalmet=1
      mfilemode=1

c -------------------------------------------------------------------
c --- INPUT phase
c -------------------------------------------------------------------

c --- Create character array for titles of CALMET options table
      do j=1,7
         do i=1,nmetopt
            copt1(i,j)=cmetopt(i)(j:j)
         enddo
         do i=nmetopt+1,mxopt_cf
            copt1(i,j)=' '
         enddo
      enddo

c --- Read scratch file image
      call RDMETCF
      if(iret.NE.0) return

c --- Fill documentation information
      call DOCFILL

c --- Create map of pointers from options and parameters read
c --- to corresponding documentation for output
      call DOCMAP

c -------------------------------------------------------------------
c --- OUTPUT phase
c -------------------------------------------------------------------

c --- NOTE:  Order of options and parameters in data arrays are set
c ---        in BLOCKDATA;  the grouping done here for output must
c ---        be based on that ordering


c --- Single File
c ---------------------------------------------------
      if(mfilemode.EQ.1) then
         ifile=1

64       format(1x,a8,1x,a8,1x,a4,2x,a8,a52)

65       format(1x,a8,a4,2x,20i6)
66       format(1x,a8,a4,4x,20f6.0)
67       format(1x,a8,a4,4x,20f6.2)
68       format(1x,a8,a4,20f6.0)

164      format(a1,a8,3a1,a8,3a1,a4,3a1,a8,3a1,a52,a1)

165      format(a1,a8,3a1,a4,a1,20(2a1,i6,a1))
166      format(a1,a8,3a1,a4,a1,20(2a1,f6.0,a1))
167      format(a1,a8,3a1,a4,a1,20(2a1,f6.2,a1))


c ---LST---
         write(iout,'(//1x,2a32)')'[<NA> indicates values that are ',
     &                            'Not Used given options selected]'
         write(iout,'(//1x,a28//1x,a40)')'Critical Wind Field Settings',
     &   '   Value Found          Typical Values  '
         do i=1,8
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(//1x,2a32)')'[<NA> indicates values that are ',
     &                            'Not Used given options selected]'
         write(icsv,'(//a28//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Critical Wind Field Settings',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=1,8
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a18/)')'Vertical Variables'
         write(iout,68) 'ZFACE   ',aface,(zface(i),i=1,nz+1)
         write(iout,65) 'LAYER   ','    ',(i,i=1,nz)
         write(iout,66) 'MID-PT  ',amid,(zmid(i),i=1,nz)
         write(iout,67) 'BIAS    ',abias,(bias(i),i=1,nz)
         write(iout,65) 'NSMTH   ',asmth,(nsmth(i),i=1,nz)
         write(iout,65) 'NINTR2  ',aintr2,(nintr2(i),i=1,nz)
         write(iout,67) 'FEXTRP2 ',aextrp2,(fextrp2(i),i=1,nz)

         write(iout,'(/1x,a16)')'Typical Values  '
         do i=1,10
            biasx(i)=0.0
            fextrp2x(i)=0.0
            nsmthx(i)=4
            nintr2x(i)=99
         enddo
         nsmthx(1)=2
         zfacex(1)=0.
         zfacex(2)=20.
         zfacex(3)=40.
         zfacex(4)=80.
         zfacex(5)=160.
         zfacex(6)=320.
         zfacex(7)=640.
         zfacex(8)=1000.
         zfacex(9)=1500.
         zfacex(10)=2000.
         zfacex(11)=3000.
         write(iout,68) 'ZFACE   ',aface,(zfacex(i),i=1,11)
         write(iout,65) 'LAYER   ','    ',(i,i=1,10)
         write(iout,67) 'BIAS    ',abias,(biasx(i),i=1,10)
         write(iout,65) 'NSMTH   ',asmth,(nsmthx(i),i=1,10)
         write(iout,65) 'NINTR2  ',aintr2,(nintr2x(i),i=1,10)
         write(iout,67) 'FEXTRP2 ',aextrp2,(fextrp2x(i),i=1,10)
c ---CSV---
         write(icsv,'(/1x,a18/)')'Vertical Variables'
         write(icsv,166) dqt,'ZFACE   ',dqt,cm,dqt,aface,dqt,
     &                   (cm,dqt,zface(i),dqt,i=1,nz+1)
         write(icsv,165) dqt,'LAYER   ',dqt,cm,dqt,'    ',dqt,
     &                   (cm,dqt,i,dqt,i=1,nz)
         write(icsv,166) dqt,'MID-PT  ',dqt,cm,dqt,amid,dqt,
     &                   (cm,dqt,zmid(i),dqt,i=1,nz)
         write(icsv,167) dqt,'BIAS    ',dqt,cm,dqt,abias,dqt,
     &                   (cm,dqt,bias(i),dqt,i=1,nz)
         write(icsv,165) dqt,'NSMTH   ',dqt,cm,dqt,asmth,dqt,
     &                   (cm,dqt,nsmth(i),dqt,i=1,nz)
         write(icsv,165) dqt,'NINTR2  ',dqt,cm,dqt,aintr2,dqt,
     &                   (cm,dqt,nintr2(i),dqt,i=1,nz)
         write(icsv,167) dqt,'FEXTRP2 ',dqt,cm,dqt,aextrp2,dqt,
     &                   (cm,dqt,fextrp2(i),dqt,i=1,nz)

         write(icsv,'(/a16)')'Typical Values  '
         write(icsv,166) dqt,'ZFACE   ',dqt,cm,dqt,aface,dqt,
     &                   (cm,dqt,zfacex(i),dqt,i=1,11)
         write(icsv,165) dqt,'LAYER   ',dqt,cm,dqt,'    ',dqt,
     &                   (cm,dqt,i,dqt,i=1,10)
         write(icsv,167) dqt,'BIAS    ',dqt,cm,dqt,abias,dqt,
     &                   (cm,dqt,biasx(i),dqt,i=1,10)
         write(icsv,165) dqt,'NSMTH   ',dqt,cm,dqt,asmth,dqt,
     &                   (cm,dqt,nsmthx(i),dqt,i=1,10)
         write(icsv,165) dqt,'NINTR2  ',dqt,cm,dqt,aintr2,dqt,
     &                   (cm,dqt,nintr2x(i),dqt,i=1,10)
         write(icsv,167) dqt,'FEXTRP2 ',dqt,cm,dqt,aextrp2,dqt,
     &                   (cm,dqt,fextrp2x(i),dqt,i=1,10)

c ---LST---
         write(iout,'(/1x,a12//1x,a40)')'Data Choices',
     &   '   Value Found          Typical Values  '
         do i=14,29
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/a12//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Data Choices',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=14,29
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a22,a20//1x,a40)')'Method Choices - Winds',
     &   ': Wind Field Modules',
     &   '   Value Found          Typical Values  '
         do i=36,41
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/a22,a20//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Method Choices - Winds',': Wind Field Modules',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=36,41
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a23,a18//1x,a40)')'Method Choices - Winds',
     &   ': Spatial Features',
     &   '   Value Found          Typical Values  '
         do i=42,44
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/a23,a18//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Method Choices - Winds',': Spatial Features',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=42,44
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a14,a16//1x,a40)')'Method Choices',
     &   ' - Mixing Height',
     &   '   Value Found          Typical Values  '
         do i=45,58
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/a14,a16//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Method Choices',' - Mixing Height',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=45,58
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a14,a14//1x,a40)')'Method Choices',
     &   ' - Temperature',
     &   '   Value Found          Typical Values  '
         do i=59,64
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
        enddo
c ---CSV---
         write(icsv,'(/a14,a14//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Method Choices',' - Temperature',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=59,64
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
        enddo

c ---LST---
         write(iout,'(/1x,a14,a16//1x,a40)')'Method Choices',
     &   ' - Precipitation',
     &   '   Value Found          Typical Values  '
         do i=65,67
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/,a14,a16//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Method Choices',' - Precipitation',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=65,67
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a14,a19//1x,a40)')'Method Choices',
     &   ' - Overwater Fluxes',
     &   '   Value Found          Typical Values  '
         do i=68,71
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/a14,a19//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Method Choices',' - Overwater Fluxes',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=68,71
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

c ---LST---
         write(iout,'(/1x,a23,a35//1x,a40)')'Miscellaneous Variables',
     &   '  Default values normally retained',
     &   '   Value Found          Typical Values  '
         do i=72,95
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(iout,64) cmetpvl(i),
     &         mmetpvl(i,ifile),fmetdoc(j),ametdoc(j),dmetdoc(j)
         enddo
c ---CSV---
         write(icsv,'(/a23,a35//a5,a1,a5,a1,a1,a7,a1,a6)')
     &   'Miscellaneous Variables',
     &   '  Default values normally retained',
     &   'Value',cm,'Found',cm,cm,'Typical',cm,'Values'
         do i=72,95
            j=mpvldoc(i)
            if(kmetdoc(j).EQ.0) write(icsv,164) dqt,cmetpvl(i),dqt,cm,
     &         dqt,mmetpvl(i,ifile),dqt,cm,dqt,fmetdoc(j),dqt,cm,
     &         dqt,ametdoc(j),dqt,cm,dqt,dmetdoc(j),dqt
         enddo

         return

      endif

      return
      end

c----------------------------------------------------------------------
      subroutine comp_pufcf
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317       COMP_PUFCF
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113               COMP_PUF
c ---           D. Strimaitis
c
c --- PURPOSE:  Process CALPUFF control file image
c
c --- INPUTS:
c
c --- OUTPUT:
c
c --- COMP_PUFCF called by:  COMP_PUF
c --- COMP_PUFCF calls:      RDPUFFCF
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'cfdoc.ivu'
      include 'err.cal'

c --- Local real variables
      real q(mxspec_cf)

c --- Local integer variables
      integer lenname(4)

c --- Local logical variables
      logical lerror
      logical lkeep

c --- Local string variables
      character*1 cm,xp
      character*1 copt1(mxopt_cf,7)
      character*12 specdoc(mxspec_cf,3)
      character*20 null2
      character*37 null4
      character*44 vols5
      character*54 area6
      character*30 dot30
      character*24 sname
      character*80 fmt1,fmt2,fmt3,fmta1,fmta2,fmta3
      character*180 char180,blnk180, fmt,fmta
      character*320 char320,blnk320

      data cm/','/
      data xp/'!'/

      data null2/'       -         -  '/
      data null4/'      -         -         -        - '/
      data vols5/'      -         -         -        -      - '/
      data area6/
     &   '      -         -         -        -      -        -  '/

      data dot30/'..............................'/

c --- Set up for 1 CALPUFF file, report all sources (Buoyant lines NA)
      mcalpuff=1
      mpoint=1
      marea=1
      mvolume=1

c --- Base formats (up to start of emissions) for list file table
c --- Points with intital sigmas
      fmt1='(a22,1x,a8,2f11.4,2f10.1,2f10.2,f10.1,2f7.0,2f10.2,'
      nfmt1=LEN_TRIM(fmt1)
c --- Areas
      fmt2='(a22,1x,a8,2f11.4,2f10.1,a54,f10.2,'
      nfmt2=LEN_TRIM(fmt2)
c --- Volumes
      fmt3='(a22,1x,a8,2f11.4,2f10.1,a44,2f10.2,'
      nfmt3=LEN_TRIM(fmt3)
c --- Buoyant Lines (not implemented)

c --- Base formats (up to start of emissions) for list file table
c --- showing actual input images as text
c --- Points with intital sigmas
      fmta1='(a22,1x,a8,2a11,2a10,2a10,a10,2a7,2a10,'
      nfmta1=LEN_TRIM(fmta1)
c --- Areas (mean x,y computed so write as real)
      fmta2='(a22,1x,a8,2f11.4,2a10,a54,a10,'
      nfmta2=LEN_TRIM(fmta2)
c --- Volumes
      fmta3='(a22,1x,a8,2a11,2a10,a44,2a10,'
      nfmta3=LEN_TRIM(fmta3)
c --- Buoyant Lines (not implemented)

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

c --- Create character array for titles of CALPUFF options table
      do j=1,7
         do i=1,npuffopt
            copt1(i,j)=cpuffopt(i)(j:j)
         enddo
         do i=npuffopt+1,mxopt_cf
            copt1(i,j)=' '
         enddo
      enddo

c --- Read scratch file image
      ifile=1
      call RDPUFFCF
      if(iret.NE.0) return


c --- CALPUFF model option documentation for each file
c ---TXT---
      write(iout,'(///a21/)')'CALPUFF Model Options'
      do j=1,7
         write(iout,62) (copt1(i,j),i=1,npuffopt)
      enddo
      write(iout,*)
62    format(11(a1,2x),2(a1,5x),16(a1,2x))
      write(iout,63)(mpuffopt(i,ifile),i=1,npuffopt)
63    format(11a3,2a6,16a3)
c ---CSV---
      write(icsv,'(///a21/)')'CALPUFF Model Options'
      write(icsv,*) cpuffopt(1),(cm,cpuffopt(i),i=2,npuffopt)
      write(icsv,*)
      write(icsv,*)mpuffopt(1,ifile),
     &            (cm,mpuffopt(i,ifile),i=2,npuffopt)


c --- Source Details

c --- Use actual input images
c --- Set species documentation for list file
      do n=1,nspec
         specdoc(n,1)=cspec(n)
         specdoc(n,2)='Rate        '
         specdoc(n,3)='user units  '
      enddo

c ---TXT---
      write(iout,'(///a22/)')'CALPUFF Source Details'
      write(iout,51)' ',' ',' ','Mean  ','Mean  ',
     &              'Release','Base ','Stack  ','Exit  ',
     &              'Exit ','FMFAC','Bldg','Sigma','Sigma',
     &              (specdoc(n,1),n=1,nspec)
      write(iout,51)'Source','  ','Type    ','X    ','Y    ',
     &              'Height','Elev ','Diameter','Velocity',
     &              'Temperatu','re     ','DW ','Y  ','Z  ',
     &              (specdoc(n,2),n=1,nspec)
      write(iout,51)' ',' ',' ','km   ','km   ',
     &              'm   ','m(MSL)','m   ','m/s  ',
     &              'K  ',' ','Flag','m  ','m  ',
     &              (specdoc(n,3),n=1,nspec)
      write(iout,*)
51    format(a6,a17,a8,2a11,5a10,2a7,2a10,3x,500(2x,a12))
c ---CSV---
      write(icsv,'(///a22/)')'CALPUFF Source Details'
      write(icsv,*)cm,cm,cm,'Mean',cm,'Mean',cm,
     &              'Release',cm,'Base',cm,'Stack',cm,'Exit',cm,
     &              'Exit',cm,'FMFAC',cm,'Bldg',cm,'Sigma',cm,'Sigma',
     &              (cm,specdoc(n,1),n=1,nspec)
      write(icsv,*)'Source',cm,cm,'Type',cm,'X',cm,'Y',cm,
     &              'Height',cm,'Elev',cm,'Diameter',cm,'Velocity',cm,
     &              'Temperature',cm,cm,'DW',cm,'Y',cm,'Z',
     &              (cm,specdoc(n,2),n=1,nspec)
      write(icsv,*)cm,cm,cm,'km',cm,'km',cm,
     &              'm',cm,'m(MSL)',cm,'m',cm,'m/s',cm,
     &              'K',cm,cm,'Flag',cm,'m',cm,'m',
     &              (cm,specdoc(n,3),n=1,nspec)
      write(icsv,*)
      
c --- Loop over sources
      do nk=1,ksrc

c ---    Use sort-order
c         n=norder(nk)
         n=nk

c ---    Set intitial part of list file format string and
         fmt=blnk180
         fmta=blnk180
         if(type(n).EQ.'POINT   ' .AND. mpoint.EQ.1) then
            nfmt=nfmt1
            fmt(1:nfmt)=fmt1(1:nfmt)
            nfmta=nfmta1
            fmta(1:nfmta)=fmta1(1:nfmta)
            lkeep=.TRUE.
         elseif(type(n).EQ.'AREA    ' .AND. marea.EQ.1) then
            nfmt=nfmt2
            fmt(1:nfmt)=fmt2(1:nfmt)
            nfmta=nfmta2
            fmta(1:nfmta)=fmta2(1:nfmta)
            lkeep=.TRUE.
         elseif(type(n).EQ.'VOLUME  ' .AND. mvolume.EQ.1) then
            nfmt=nfmt3
            fmt(1:nfmt)=fmt3(1:nfmt)
            nfmta=nfmta3
            fmta(1:nfmta)=fmta3(1:nfmta)
            lkeep=.TRUE.
         else
            lkeep=.FALSE.
         endif

         if(LKEEP) then
c ---       Set format for each emission rate
            do i=1,nspec
               fmta=fmta(1:nfmta)//'a14,'
               nfmta=nfmta+4
            enddo
            fmta=fmta(1:nfmta)//'a)'
            nfmta=nfmta+2

c ---TXT---
            if(type(n).EQ.'POINT   ' .AND. mpoint.EQ.1) then
               write(iout,fmta) name(2,n),type(n),
     &                  ax(n),ay(n),azht(n),azbase(n),adiam(n),avel(n),
     &                  atempk(n),afmf(n),abdw(n),asigy(n),asigz(n),
     &                  (aqrate(i,n),i=1,nspec)
            elseif(type(n).EQ.'AREA    ' .AND. marea.EQ.1) then
               write(iout,fmta) name(2,n),type(n),
     &                     x(n),y(n),azht(n),azbase(n),area6,
     &                     asigz(n),(aqrate(i,n),i=1,nspec)
            elseif(type(n).EQ.'VOLUME  ' .AND. mvolume.EQ.1) then
               write(iout,fmta) name(2,n),type(n),
     &                  ax(n),ay(n),azht(n),azbase(n),vols5,asigy(n),
     &                  asigz(n),(aqrate(i,n),i=1,nspec)
            endif
c ---CSV---
            if(type(n).EQ.'POINT   ' .AND. mpoint.EQ.1) then
               write(icsv,*) name(2,n),cm,cm,type(n),cm,
     &                  ax(n),cm,ay(n),cm,azht(n),cm,azbase(n),cm,
     &                  adiam(n),cm,avel(n),cm,atempk(n),cm,
     &                  afmf(n),cm,abdw(n),cm,asigy(n),cm,asigz(n),
     &                  (cm,aqrate(i,n),i=1,nspec)
            elseif(type(n).EQ.'AREA    ' .AND. marea.EQ.1) then
               write(icsv,*) name(2,n),cm,cm,type(n),cm,
     &                  x(n),cm,y(n),cm,azht(n),cm,azbase(n),cm,
     &                  cm,cm,cm,cm,cm,cm,asigz(n),
     &                  (cm,aqrate(i,n),i=1,nspec)
            elseif(type(n).EQ.'VOLUME  ' .AND. mvolume.EQ.1) then
               write(icsv,*) name(2,n),cm,cm,type(n),cm,
     &                  ax(n),cm,ay(n),cm,azht(n),cm,azbase(n),cm,
     &                  cm,cm,cm,cm,cm,asigy(n),cm,asigz(n),
     &                  (cm,aqrate(i,n),i=1,nspec)
            endif

         endif
      enddo


      return
      end

c----------------------------------------------------------------------
      subroutine rdmetcf
c----------------------------------------------------------------------
c --- INFOVIEW  Version: 1.6            Level: 090413          RDMETCF
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113                  RDMET
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads one file, and collects
c               selected CALMET information
c
c --- UPDATES:
c --- V1.4 (L090410) to V1.41 (L090413) DGS
c            - Allow repetition factor in height arrays
c
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Fill any missing entries in array variables with zero
c
c --- INPUTS:
c
c --- OUTPUT:
c
c     Common block /MAP/
c        feast,fnorth,iutmzn,
c        pmap,datum,utmhem,
c        clat0,clon0,clat1,clat2
c     Common block /OPTION/
c        mmetopt
c     Common block /TIME/
c        mettim
c     Common/DOCMET/
c        noobs,npsta,iextrp
c     Common/ZDATA/
c        nz,nsmth,nintr2,zface,zmid,bias,fextrp2
c        asmth,aintr2,aface,amid,abias,aextrp2
c
c --- RDMETCF called by: COMP_METCF
c --- RDMETCF calls:     RDFIELD,
c                        RHS320, OMIT320, COLUMN320, IRANGE, RGET320
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'err.cal'
      include 'cfdoc.ivu'

c --- Set dimension for the maximum number of variables that can be
c --- assigned in one statement
      parameter(mxvar=36)

c --- Local variables
      integer icommas(0:mxvar)
      logical lerror
      character*1 cm,xp,eq
      character*320 char320,next320

      data cm/','/
      data xp/'!'/
      data eq/'='/

c --- Initial values
      lerror=.FALSE.
      ifile=1

20    call RDFIELD(icfx,char320,klast,ieof)
      if(iret.NE.0) return
      if(ieof.GT.0) goto 21

c --- Model options
      do i=1,nmetopt
         i2=LEN_TRIM(cmetopt(i))
         if(INDEX(char320,xp//cmetopt(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') mmetopt(i,ifile)
            mmetopt(i,ifile)=ADJUSTL(mmetopt(i,ifile))
         endif
      enddo

c --- Model parameter values
      do i=1,nmetpvl
         i2=LEN_TRIM(cmetpvl(i))
         if(INDEX(char320,xp//cmetpvl(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') mmetpvl(i,ifile)
            mmetpvl(i,ifile)=ADJUSTL(mmetpvl(i,ifile))
         endif
      enddo

c --- Special cases
c --------------------------------------------------------

c --- 1. Convert ISTEPPG to seconds if found
      if(INDEX(char320,'!ISTEPPG=').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') isteppg
         isteppgs=isteppg*3600
c ---    Place into array
         do i=1,nmetpvl
            if(cmetpvl(i).EQ.'ISTEPPGS') then
               write(mmetpvl(i,ifile),'(i)') isteppgs
               mmetpvl(i,ifile)=ADJUSTL(mmetpvl(i,ifile))
            endif
         enddo
      endif

c --- 2. Get integer expression of selected controls
      if(INDEX(char320,'!NOOBS=').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') noobs
      endif
      if(INDEX(char320,'!NPSTA=').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') npsta
      endif
      if(INDEX(char320,'!IEXTRP=').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') iextrp
      endif

c --- 3. Extract array variables (layers)
      if(INDEX(char320,'!NZ=').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') nz
      endif
      if(INDEX(char320,'!NSMTH=').NE.0) then
         nvars=nz
         call RHS320(eq,char320)
         call OMIT320(xp,char320)
         klast=LEN_TRIM(char320)
         call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
         if(iret.NE.0) return
         num=MIN(nvars,nz)
         k=0
         do j=1,num
            call IRANGE(j,icommas,ncommas,i1,i2)
            irep=INDEX(char320(i1:i2),'*')
            if(irep.EQ.0) then
               k=k+1
               read(char320(i1:i2),'(i)') nsmth(k)
            else
c ---          Repetition factor found
               ileft=i1+irep-2
               iright=ileft+2
               read(char320(i1:ileft),'(i)') nrep
               do n=1,nrep
                  k=k+1
                  read(char320(iright:i2),'(i)') nsmth(k)
               enddo
            endif
         enddo
         if(nz.GT.k) then
            do j=k+1,nz
               nsmth(j)=0
            enddo
         endif
      endif
      if(INDEX(char320,'!NINTR2=').NE.0) then
         nvars=nz
         call RHS320(eq,char320)
         call OMIT320(xp,char320)
         klast=LEN_TRIM(char320)
         call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
         if(iret.NE.0) return
         num=MIN(nvars,nz)
         k=0
         do j=1,num
            call IRANGE(j,icommas,ncommas,i1,i2)
            irep=INDEX(char320(i1:i2),'*')
            if(irep.EQ.0) then
               k=k+1
               read(char320(i1:i2),'(i)') nintr2(k)
            else
c ---          Repetition factor found
               ileft=i1+irep-2
               iright=ileft+2
               read(char320(i1:ileft),'(i)') nrep
               do n=1,nrep
                  k=k+1
                  read(char320(iright:i2),'(i)') nintr2(k)
               enddo
            endif
         enddo
         if(nz.GT.k) then
            do j=k+1,nz
               nintr2(j)=0
            enddo
         endif
      endif
      if(INDEX(char320,'!BIAS=').NE.0) then
         nvars=nz
         call RHS320(eq,char320)
         call OMIT320(xp,char320)
         klast=LEN_TRIM(char320)
         call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
         if(iret.NE.0) return
         num=MIN(nvars,nz)
         k=0
         do j=1,num
            call IRANGE(j,icommas,ncommas,i1,i2)
            irep=INDEX(char320(i1:i2),'*')
            if(irep.EQ.0) then
               k=k+1
               call RGET320(char320,i1,i2,bias(k))
            else
c ---          Repetition factor found
               ileft=i1+irep-2
               iright=ileft+2
               read(char320(i1:ileft),'(i)') nrep
               do n=1,nrep
                  k=k+1
                  call RGET320(char320,iright,i2,bias(k))
               enddo
            endif
         enddo
         if(nz.GT.k) then
            do j=k+1,nz
               bias(j)=0.0
            enddo
         endif
      endif
      if(INDEX(char320,'!ZFACE=').NE.0) then
         nvars=nz+1
         call RHS320(eq,char320)
         call OMIT320(xp,char320)
         klast=LEN_TRIM(char320)
         call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
         if(iret.NE.0) return
         num=MIN(nvars,nz+1)
         do j=1,num
            call IRANGE(j,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,zface(j))
         enddo
         if(nz+1.GT.num) then
            do j=num+1,nz+1
               zface(j)=0.0
            enddo
         endif
      endif
      if(INDEX(char320,'!FEXTRP2=').NE.0) then
         nvars=nz
         call RHS320(eq,char320)
         call OMIT320(xp,char320)
         klast=LEN_TRIM(char320)
         call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
         if(iret.NE.0) return
         num=MIN(nvars,nz)
         k=0
         do j=1,num
            call IRANGE(j,icommas,ncommas,i1,i2)
            irep=INDEX(char320(i1:i2),'*')
            if(irep.EQ.0) then
               k=k+1
               call RGET320(char320,i1,i2,fextrp2(k))
            else
c ---          Repetition factor found
               ileft=i1+irep-2
               iright=ileft+2
               read(char320(i1:ileft),'(i)') nrep
               do n=1,nrep
                  k=k+1
                  call RGET320(char320,iright,i2,fextrp2(k))
               enddo
            endif
         enddo
         if(nz.GT.k) then
            do j=k+1,nz
               fextrp2(j)=0.0
            enddo
         endif
      endif

c --- Model runtimes
      do i=1,nmettim
         i2=LEN_TRIM(cmettim(i))
         if(INDEX(char320,xp//cmettim(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') mettim(i,ifile)
            mettim(i,ifile)=ADJUSTL(mettim(i,ifile))
         endif
      enddo

c --- Model Map Projection
      do i=1,nmetmap
         i2=LEN_TRIM(cmetmap(i))
         if(INDEX(char320,xp//cmetmap(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') metmap(i,ifile)
            metmap(i,ifile)=ADJUSTL(metmap(i,ifile))
         endif
      enddo

      goto 20

21    continue

c --- Process content read from file
c ----------------------------------


c --- Remove map projection information that does not apply to PMAP
      if(metmap(kpmap,ifile).EQ.'NONE   ') then
         metmap(kdatum,ifile)='  -     '
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
         metmap(krlon0,ifile)='  -             '
         metmap(krlat0,ifile)='  -             '
         metmap(kxlat1,ifile)='  -             '
         metmap(kxlat2,ifile)='  -             '
      elseif(metmap(kpmap,ifile).EQ.'LL     ') then
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
         metmap(krlon0,ifile)='  -             '
         metmap(krlat0,ifile)='  -             '
         metmap(kxlat1,ifile)='  -             '
         metmap(kxlat2,ifile)='  -             '
      elseif(metmap(kpmap,ifile).EQ.'UTM    ') then
         metmap(krlon0,ifile)='  -             '
         metmap(krlat0,ifile)='  -             '
         metmap(kxlat1,ifile)='  -             '
         metmap(kxlat2,ifile)='  -             '
      elseif(metmap(kpmap,ifile).EQ.'LCC    ') then
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
      elseif(metmap(kpmap,ifile).EQ.'PS     ') then
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
         metmap(kxlat2,ifile)='  -             '
      elseif(metmap(kpmap,ifile).EQ.'EM     ') then
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
         metmap(krlat0,ifile)='  -             '
         metmap(kxlat1,ifile)='  -             '
         metmap(kxlat2,ifile)='  -             '
      elseif(metmap(kpmap,ifile).EQ.'LAZA   ') then
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
         metmap(kxlat1,ifile)='  -             '
         metmap(kxlat2,ifile)='  -             '
      elseif(metmap(kpmap,ifile).EQ.'TTM    ') then
         metmap(kutmzn,ifile)='-   '
         metmap(kutmhem,ifile)='-   '
         metmap(kxlat1,ifile)='  -             '
         metmap(kxlat2,ifile)='  -             '
      endif

c --- Compute mid-pt heights of eack layer
      do j=1,nz
         zmid(j)=0.5*(zface(j)+zface(j+1))
      enddo

c --- Set NA flags for vertical variables
      asmth='    '
      aintr2='    '
      aface='    '
      amid='    '
      abias='    '
      aextrp2='    '
      if(noobs.EQ.2) abias='<NA>'
      if(iextrp.NE.3) aextrp2='<NA>'

      if(LERROR) then
         write(iout,*)'ERROR in RDMETCF'
         iret=1
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine rdpuffcf
c----------------------------------------------------------------------
c --- INFOVIEW  Version: 1.6            Level: 090410         RDPUFFCF
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113                 RDPUFF
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads one file, and collects
c               selected CALPUFF information
c
c --- UPDATES:
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Missing entries in array variables are not allowed, but
c              COLUMN320 now issues a warning rather than an error ---
c              Halt here if data are missing
c
c --- INPUTS:
c            FNAME - char*132     - File name (with path)
c            IFILE - integer      - Index for current file
c
c --- OUTPUT:
c
c     Common block /SOURCE/
c        ksrc,x,y,zht,zbase,diam,vel,tempk,sigy,sigz,
c        qrate,bdw,fmf,zplt,
c        ax,ay,azht,azbase,adiam,avel,atempk,asigy,asigz,
c        aqrate,abdw,afmf,azplt,
c        name,type,cspec
c     Common block /OPTION/
c        mpuffopt
c     Common block /TIME/
c        pufftim
c     Common block /MAPC/
c        puffmap
c     Common block /RECEPTOR/
c        puffrec
c
c --- RDPUFFCF called by: COMP_PUFCF
c --- RDPUFFCF calls:     RDFIELD,
c                         RHS320, OMIT320, COLUMN320, IRANGE, RGET320
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'err.cal'
      include 'ctldat.ivu'
      include 'cfdoc.ivu'

c --- Set dimension for the maximum number of variables that can be
c --- assigned in one statement
      parameter(mxvar=36)

c --- Local variables
      real qufac(7),qin(mxspec_cf)
      integer icommas(0:mxvar)
      integer ispref(mxspec_cf)
      logical lgroup13,lgroup14b,lgroup14c,lgroup15,lgroup16,lerror
      character*1 cm,xp,eq
      character*12 cspecin(mxspec_cf)
      character*14 aqin(mxspec_cf)
      character*80 srcname
      character*320 char320,next320

      data cm/','/
      data xp/'!'/
      data eq/'='/

c --- Compute conversion factors to g/s 
c --- (also applies to OU m3/s, g/m2/s, and OU m/s)
      qufac(1)=1.0                     !g/s
      qufac(2)=1000.0                  !kg/s
      qufac(3)=1.0/3.6                 !kg/hr
      qufac(4)=(2.*0.4536)/(8.76*3.6)  !ton/yr
      qufac(5)=1.0                     !OU m3/s
      qufac(6)=1.0/60.0                !OU m3/min
      qufac(7)=1.0/(8.76*3.6)          !MTon/yr

c --- Remember number of sources before more are read
      ksrc0=ksrc

      ifile=1

c --- Initial values
      ispec=0
      fptu=1.0
      faru=1.0
      flnu=1.0
      fvlu=1.0
      ipt1=0
      iar1=0
      iln1=0
      ivl1=0
      lerror=.FALSE.
      lgroup13=.FALSE.
      lgroup14b=.FALSE.
      lgroup14c=.FALSE.
      lgroup15=.FALSE.
      lgroup16=.FALSE.

20    call RDFIELD(icfx,char320,klast,ieof)
      if(iret.NE.0) return
      if(ieof.GT.0) goto 21

c --- Species
      if(INDEX(char320,'NSPEC').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') nspecin
      endif
      if(INDEX(char320,'CSPEC').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         ispec=ispec+1
         if(ispec.GT.MXSPEC_cf) then
            write(iout,*)
            write(iout,*)'ERROR in RDPUFFCF:'
            write(iout,*)'Number of species exceeds MXSPEC'
            write(iout,*)'MXSPEC,ispec,nspec = ',MXSPEC_cf,ispec,nspecin
            iret=1
            return
         endif
         read(char320(i1:i2),'(a)') cspecin(ispec)
         cspecin(ispec)=ADJUSTL(cspecin(ispec))
      endif


c --- Model Map Projection
      do i=1,npuffmap
         i2=LEN_TRIM(cpuffmap(i))
         if(INDEX(char320,xp//cpuffmap(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') puffmap(i,ifile)
            puffmap(i,ifile)=ADJUSTL(puffmap(i,ifile))
         endif
      enddo

c --- Model options
      do i=1,npuffopt
         i2=LEN_TRIM(cpuffopt(i))
         if(INDEX(char320,xp//cpuffopt(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') mpuffopt(i,ifile)
            mpuffopt(i,ifile)=ADJUSTL(mpuffopt(i,ifile))
         endif
      enddo

c --- Model runtimes
      do i=1,npufftim
         i2=LEN_TRIM(cpufftim(i))
         if(INDEX(char320,xp//cpufftim(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') pufftim(i,ifile)
            pufftim(i,ifile)=ADJUSTL(pufftim(i,ifile))
         endif
      enddo

c --- Model receptors
      do i=1,npuffrec
         i2=LEN_TRIM(cpuffrec(i))
         if(INDEX(char320,xp//cpuffrec(i)(1:i2)//'=').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            read(char320(i1:i2),'(a)') puffrec(i,ifile)
            puffrec(i,ifile)=ADJUSTL(puffrec(i,ifile))
         endif
      enddo

c --- Update current source group and store source number
      if(INDEX(char320,'NPT1').NE.0) then
         LGROUP13=.TRUE.
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') npt1
         if(npt1.EQ.0) LGROUP13=.FALSE.
      elseif(INDEX(char320,'NAR1').NE.0) then
         LGROUP13=.FALSE.
         LGROUP14b=.TRUE.
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') nar1
         if(nar1.EQ.0) LGROUP14b=.FALSE.
      elseif(INDEX(char320,'NLINES').NE.0) then
         LGROUP14b=.FALSE.
         LGROUP14c=.FALSE.
         LGROUP15=.TRUE.
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') nln1
         if(nln1.EQ.0) LGROUP15=.FALSE.
      elseif(INDEX(char320,'NVL1').NE.0) then
         LGROUP15=.FALSE.
         LGROUP16=.TRUE.
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') nvl1
         if(nvl1.EQ.0) LGROUP16=.FALSE.
      elseif(INDEX(char320,'NREC').NE.0) then
         LGROUP16=.FALSE.
      endif

c --- Emission rate conversion factors
      if(INDEX(char320,'IPTU').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') iptu

         if(iptu.EQ.0) then
            write(iout,*)
            write(iout,*)'ERROR in RDPUFFCF:'
            write(iout,*)'IPTU = ',iptu
            write(iout,*)char320(1:klast)
            write(iout,*)'i1,i2 = ',i1,i2
            iret=1
            return
         endif

         fptu=qufac(iptu)
      endif
      if(INDEX(char320,'IARU').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') iaru
         faru=qufac(iaru)
      endif
      if(INDEX(char320,'ILNU').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') ilnu
         flnu=qufac(ilnu)
      endif
      if(INDEX(char320,'IVLU').NE.0) then
         i1=INDEX(char320,eq,back=.true.)+1
         i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
         read(char320(i1:i2),'(i)') ivlu
         fvlu=qufac(ivlu)
      endif

c --- Group 13b
      if(LGROUP13) then
c ---    Name
         if(INDEX(char320,'SRCNAM').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            ipt1=ipt1+1
            if(ipt1.GT.npt1) then
c ---          Beyond 13b
               LGROUP13=.FALSE.
            else
               ksrc=ksrc+1
               type(ksrc)='POINT   '
               name(1,ksrc)=fileinp
               read(char320(i1:i2),'(a)') name(2,ksrc)
               name(2,ksrc)=ADJUSTL(name(2,ksrc))
            endif
         endif
c ---    Primary source parameters
         if(INDEX(char320,'X=').NE.0) then
            nvars=8+nspecin
            call RHS320(eq,char320)
            call OMIT320(xp,char320)
            klast=LEN_TRIM(char320)
            call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
            if(nvars.LT.8+nspecin) iret=1
            if(iret.NE.0) return
c ---       Assign values to source arrays
            call IRANGE(1,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,x(ksrc))
              read(char320(i1:i2),'(a)') ax(ksrc)
              ax(ksrc)=ADJUSTR(ax(ksrc))
            call IRANGE(2,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,y(ksrc))
              read(char320(i1:i2),'(a)') ay(ksrc)
              ay(ksrc)=ADJUSTR(ay(ksrc))
            call IRANGE(3,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,zht(ksrc))
              read(char320(i1:i2),'(a)') azht(ksrc)
              azht(ksrc)=ADJUSTR(azht(ksrc))
            call IRANGE(4,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,zbase(ksrc))
              read(char320(i1:i2),'(a)') azbase(ksrc)
              azbase(ksrc)=ADJUSTR(azbase(ksrc))
            call IRANGE(5,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,diam(ksrc))
              read(char320(i1:i2),'(a)') adiam(ksrc)
              adiam(ksrc)=ADJUSTR(adiam(ksrc))
            call IRANGE(6,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,vel(ksrc))
              read(char320(i1:i2),'(a)') avel(ksrc)
              avel(ksrc)=ADJUSTR(avel(ksrc))
            call IRANGE(7,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,tempk(ksrc))
              read(char320(i1:i2),'(a)') atempk(ksrc)
              atempk(ksrc)=ADJUSTR(atempk(ksrc))
            call IRANGE(8,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,bdw(ksrc))
              read(char320(i1:i2),'(a)') abdw(ksrc)
              abdw(ksrc)=ADJUSTR(abdw(ksrc))
            do is=1,nspecin
               k=8+is
               call IRANGE(k,icommas,ncommas,i1,i2)
                 call RGET320(char320,i1,i2,qrate(is,ksrc))
                 qrate(is,ksrc)=fptu*qrate(is,ksrc)
                 read(char320(i1:i2),'(a)') aqrate(is,ksrc)
                 aqrate(is,ksrc)=ADJUSTR(aqrate(is,ksrc))
            enddo
         endif
c ---    Initial sigmas
         if(INDEX(char320,'SIGYZI').NE.0) then
            nvars=2
            call RHS320(eq,char320)
            call OMIT320(xp,char320)
            klast=LEN_TRIM(char320)
            call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
            if(nvars.LT.2) iret=1
            if(iret.NE.0) return
c ---       Assign values to source arrays
            call IRANGE(1,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,sigy(ksrc))
              read(char320(i1:i2),'(a)') asigy(ksrc)
              asigy(ksrc)=ADJUSTR(asigy(ksrc))
            call IRANGE(2,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,sigz(ksrc))
              read(char320(i1:i2),'(a)') asigz(ksrc)
              asigz(ksrc)=ADJUSTR(asigz(ksrc))
         endif
c ---    Vertical momentum flux factor (0. to 1.)
         if(INDEX(char320,'FMFAC').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            call RGET320(char320,i1,i2,fmf(ksrc))
            read(char320(i1:i2),'(a)') afmf(ksrc)
            afmf(ksrc)=ADJUSTR(afmf(ksrc))
         endif
c ---    Platform height (downwash)
         if(INDEX(char320,'ZPLTFM').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            call RGET320(char320,i1,i2,zplt(ksrc))
            read(char320(i1:i2),'(a)') azplt(ksrc)
            azplt(ksrc)=ADJUSTR(azplt(ksrc))
         endif
      endif

c --- Group 14b (place code before Group 14c code)
      if(LGROUP14b) then
c ---    Name
         if(INDEX(char320,'SRCNAM').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            iar1=iar1+1
            if(iar1.GT.nar1) then
c ---          Beyond 14b
               LGROUP14b=.FALSE.
               LGROUP14c=.TRUE.
               iar1=0
            else
               ksrc=ksrc+1
               type(ksrc)='AREA    '
               name(1,ksrc)=fileinp
               read(char320(i1:i2),'(a)') name(2,ksrc)
               name(2,ksrc)=ADJUSTL(name(2,ksrc))
            endif
         endif
c ---    Source parameters
         if(INDEX(char320,'X=').NE.0) then
            nvars=3+nspecin
            call RHS320(eq,char320)
            call OMIT320(xp,char320)
            klast=LEN_TRIM(char320)
            call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
            if(nvars.LT.3+nspecin) iret=1
            if(iret.NE.0) return
c ---       Assign values to source arrays
            call IRANGE(1,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,zht(ksrc))
              read(char320(i1:i2),'(a)') azht(ksrc)
              azht(ksrc)=ADJUSTR(azht(ksrc))
            call IRANGE(2,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,zbase(ksrc))
              read(char320(i1:i2),'(a)') azbase(ksrc)
              azbase(ksrc)=ADJUSTR(azbase(ksrc))
            call IRANGE(3,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,sigz(ksrc))
              read(char320(i1:i2),'(a)') asigz(ksrc)
              asigz(ksrc)=ADJUSTR(asigz(ksrc))
            do is=1,nspecin
               k=3+is
               call IRANGE(k,icommas,ncommas,i1,i2)
                 call RGET320(char320,i1,i2,qrate(is,ksrc))
                 qrate(is,ksrc)=faru*qrate(is,ksrc)
                 read(char320(i1:i2),'(a)') aqrate(is,ksrc)
                 aqrate(is,ksrc)=ADJUSTR(aqrate(is,ksrc))
            enddo
         endif
      endif

c --- Group 14c (place code after Group 14b code)
      if(LGROUP14c) then
c ---    Name
         if(INDEX(char320,'SRCNAM').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            iar1=iar1+1
            if(iar1.GT.nar1) then
c ---          Beyond 14c
               LGROUP14c=.FALSE.
            else
c ---          Match SRCNAM to find ksrc index
               read(char320(i1:i2),'(a)') srcname
               srcname=ADJUSTL(srcname)
               jsrc=0
               do j=1,ksrc
                  if(type(j).EQ.'AREA    ' .AND.
     &                name(2,j).EQ.srcname) jsrc=j
               enddo
               if(jsrc.EQ.0) then
                  write(iout,*)
                  write(iout,*)'ERROR in RDPUFFCF:'
                  write(iout,*)'ERROR:  Source name in 14c not found'
                  write(iout,*)srcname
                  iret=1
                  return
               endif
            endif
         endif
c ---    Mean source location
         if(INDEX(char320,'XVERT').NE.0) then
            nvars=4
            call RHS320(eq,char320)
            call OMIT320(xp,char320)
            klast=LEN_TRIM(char320)
            call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
            if(nvars.LT.4) iret=1
            if(iret.NE.0) return
c ---       Assign value to source array
            call IRANGE(1,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,x1)
            call IRANGE(2,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,x2)
            call IRANGE(3,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,x3)
            call IRANGE(4,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,x4)
            x(jsrc)=0.25*(x1+x2+x3+x4)
         endif
         if(INDEX(char320,'YVERT').NE.0) then
            nvars=4
            call RHS320(eq,char320)
            call OMIT320(xp,char320)
            klast=LEN_TRIM(char320)
            call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
            if(nvars.LT.4) iret=1
            if(iret.NE.0) return
c ---       Assign value to source array
            call IRANGE(1,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,y1)
            call IRANGE(2,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,y2)
            call IRANGE(3,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,y3)
            call IRANGE(4,icommas,ncommas,i1,i2)
            call RGET320(char320,i1,i2,y4)
            y(jsrc)=0.25*(y1+y2+y3+y4)
         endif
      endif

c --- Group 16b
      if(LGROUP16) then
c ---    Name
         if(INDEX(char320,'SRCNAM').NE.0) then
            i1=INDEX(char320,eq,back=.true.)+1
            i2=(i1-1)+INDEX(char320(i1:klast),xp)-1
            ivl1=ivl1+1
            if(ivl1.GT.nvl1) then
c ---          Beyond 16b
               LGROUP16=.FALSE.
            else
               ksrc=ksrc+1
               type(ksrc)='VOLUME  '
               name(1,ksrc)=fileinp
               read(char320(i1:i2),'(a)') name(2,ksrc)
               name(2,ksrc)=ADJUSTL(name(2,ksrc))
            endif
         endif
c ---    Source parameters
         if(INDEX(char320,'X=').NE.0) then
            nvars=6+nspecin
            call RHS320(eq,char320)
            call OMIT320(xp,char320)
            klast=LEN_TRIM(char320)
            call COLUMN320(char320,klast,cm,nvars,ncommas,icommas)
            if(nvars.LT.6+nspecin) iret=1
            if(iret.NE.0) return
c ---       Assign values to source arrays
            call IRANGE(1,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,x(ksrc))
              read(char320(i1:i2),'(a)') ax(ksrc)
              ax(ksrc)=ADJUSTR(ax(ksrc))
            call IRANGE(2,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,y(ksrc))
              read(char320(i1:i2),'(a)') ay(ksrc)
              ay(ksrc)=ADJUSTR(ay(ksrc))
            call IRANGE(3,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,zht(ksrc))
              read(char320(i1:i2),'(a)') azht(ksrc)
              azht(ksrc)=ADJUSTR(azht(ksrc))
            call IRANGE(4,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,zbase(ksrc))
              read(char320(i1:i2),'(a)') azbase(ksrc)
              azbase(ksrc)=ADJUSTR(azbase(ksrc))
            call IRANGE(5,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,sigy(ksrc))
              read(char320(i1:i2),'(a)') asigy(ksrc)
              asigy(ksrc)=ADJUSTR(asigy(ksrc))
            call IRANGE(6,icommas,ncommas,i1,i2)
              call RGET320(char320,i1,i2,sigz(ksrc))
              read(char320(i1:i2),'(a)') asigz(ksrc)
              asigz(ksrc)=ADJUSTR(asigz(ksrc))
            do is=1,nspecin
               k=6+is
               call IRANGE(k,icommas,ncommas,i1,i2)
                 call RGET320(char320,i1,i2,qrate(is,ksrc))
                 qrate(is,ksrc)=fvlu*qrate(is,ksrc)
                 read(char320(i1:i2),'(a)') aqrate(is,ksrc)
                 aqrate(is,ksrc)=ADJUSTR(aqrate(is,ksrc))
            enddo
         endif
      endif


      goto 20

21    continue

c --- Process content read from file
c ----------------------------------

c --- Update output species array and set indicies
      do is=1,ispec
         k=0
c ---    Test for match in output list
         do n=1,nspec
            if(cspecin(is).EQ.cspec(n)) k=n
         enddo
         if(k.EQ.0) then
c ---       No match, add species to output list
            nspec=nspec+1
            cspec(nspec)=cspecin(is)
            ispref(is)=nspec
         else
c ---       Match
            ispref(is)=k
         endif
      enddo

c --- Transfer emission rates for sources just read to
c --- the output species order
      k1=ksrc0+1
      do k=k1,ksrc
c ---    Place rates just read into temp array
         do is=1,ispec
            qin(is)=qrate(is,k)
            qrate(is,k)=0.0
            aqin(is)=aqrate(is,k)
            aqrate(is,k)='         -    '
         enddo
c ---    Reorder in output array
         do is=1,ispec
            qrate(ispref(is),k)=qin(is)
            aqrate(ispref(is),k)=aqin(is)
         enddo
      enddo

c --- Remove map projection information that does not apply to PMAP
      if(puffmap(kpmap,ifile).EQ.'NONE   ') then
         puffmap(kdatum,ifile)='  -     '
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
         puffmap(krlon0,ifile)='  -             '
         puffmap(krlat0,ifile)='  -             '
         puffmap(kxlat1,ifile)='  -             '
         puffmap(kxlat2,ifile)='  -             '
      elseif(puffmap(kpmap,ifile).EQ.'LL     ') then
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
         puffmap(krlon0,ifile)='  -             '
         puffmap(krlat0,ifile)='  -             '
         puffmap(kxlat1,ifile)='  -             '
         puffmap(kxlat2,ifile)='  -             '
      elseif(puffmap(kpmap,ifile).EQ.'UTM    ') then
         puffmap(krlon0,ifile)='  -             '
         puffmap(krlat0,ifile)='  -             '
         puffmap(kxlat1,ifile)='  -             '
         puffmap(kxlat2,ifile)='  -             '
      elseif(puffmap(kpmap,ifile).EQ.'LCC    ') then
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
      elseif(puffmap(kpmap,ifile).EQ.'PS     ') then
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
         puffmap(kxlat2,ifile)='  -             '
      elseif(puffmap(kpmap,ifile).EQ.'EM     ') then
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
         puffmap(krlat0,ifile)='  -             '
         puffmap(kxlat1,ifile)='  -             '
         puffmap(kxlat2,ifile)='  -             '
      elseif(puffmap(kpmap,ifile).EQ.'LAZA   ') then
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
         puffmap(kxlat1,ifile)='  -             '
         puffmap(kxlat2,ifile)='  -             '
      elseif(puffmap(kpmap,ifile).EQ.'TTM    ') then
         puffmap(kutmzn,ifile)='-   '
         puffmap(kutmhem,ifile)='-   '
         puffmap(kxlat1,ifile)='  -             '
         puffmap(kxlat2,ifile)='  -             '
      endif

      if(LERROR) then
         write(iout,*)'ERROR in RDPUFFCF'
         iret=1
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine docfill
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          DOCFILL
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113                DOCFILL
c ---           D. Strimaitis
c
c --- PURPOSE:  Fills documentation arrays
c
c --- INPUTS:
c
c     Common/DOCMET/
c        noobs,npsta,iextrp
c
c --- OUTPUT:
c
c     Common/DOCMET/
c        nmetdoc,kmetdoc,cmetdoc,ametdoc,dmetdoc,fmetdoc
c
c --- DOCFILL called by:  COMP_METCF
c --- DOCFILL calls:      none
c----------------------------------------------------------------------
      include 'cfdoc.ivu'

c --- Fill in documentation information for CALMET
      nmetdoc=95
      k=0

c --- Set all to null values
      do i=1,nmetdoc
      kmetdoc(i)=0
      fmetdoc(i)='    '
      cmetdoc(i)='        '
      dmetdoc(i)='                                                  '
      ametdoc(i)='                '
      enddo

c --- Fill values where applicable

c -----------------------------
c --- Model OPTIONS
c -----------------------------

      k=k+1
      cmetdoc(k)='NOOBS   '
      ametdoc(k)='0,1,2           '
      dmetdoc(k)='0=w/Obs; 1=Partial Obs/No-Obs; 2=No-Obs mode         '

c      if(noobs.EQ.0) then
c      dmetdoc(k)='Read surface, upper, & overwater obs.               '
c      ametdoc(k)='0               '
c      elseif(noobs.EQ.1) then
c      dmetdoc(k)='Read surface & overwater obs.; prognostic upper     '
c      ametdoc(k)='1               '
c      elseif(noobs.EQ.2) then
c      dmetdoc(k)='No obs. met; prognostic for all                     '
c      ametdoc(k)='2               '
c      endif

      k=k+1
      cmetdoc(k)='ICLOUD  '
      dmetdoc(k)='0=Obs.Clouds; 3,4=Clouds from Prog. RH              '
      ametdoc(k)='0,3,4   '

      k=k+1
      cmetdoc(k)='ITPROG  '
      dmetdoc(k)='0=Obs.; 1=Obs.Sfc/Prog.Upr; 2=Prog. temperatures    '
      ametdoc(k)='0,1,2           '

      k=k+1
      cmetdoc(k)='ITWPROG '
      dmetdoc(k)='0=Obs.; 1=Obs.T_Diff/Prog.Lapse; 2=Prog. Overwater T'
      ametdoc(k)='0,1,2           '

      k=k+1
      cmetdoc(k)='IRHPROG '
      dmetdoc(k)='0=Obs.; 1=Prog. relative humidity data              '
      ametdoc(k)='0,1             '

      k=k+1
      cmetdoc(k)='IPROG   '
      dmetdoc(k)='Use gridded prognostic winds as Initial Guess Field '
      ametdoc(k)='4,14            '

      k=k+1
      cmetdoc(k)='IWFCOD  '
      dmetdoc(k)='Use Diagnostic Wind Model                           '
      ametdoc(k)='1               '

      k=k+1
      cmetdoc(k)='IFRADJ  '
      dmetdoc(k)='Compute Froude number adjustment                    '
      ametdoc(k)='1               '

      k=k+1
      cmetdoc(k)='IKINE   '
      dmetdoc(k)='Do Not compute kinematic effects                    '
      ametdoc(k)='0               '

      k=k+1
      cmetdoc(k)='IOBR    '
      dmetdoc(k)='Do Not use OBrien adjustment to w-velocities        '
      ametdoc(k)='0               '

      k=k+1
      cmetdoc(k)='ISLOPE  '
      dmetdoc(k)='Compute slope flow effects                          '
      ametdoc(k)='1               '

      k=k+1
      cmetdoc(k)='IEXTRP  '
      dmetdoc(k)='Similarity extrap. of wind (-4 ignore upper stn sfc)'
      ametdoc(k)='4,-4            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='ICALM   '
      dmetdoc(k)='Do Not extrapolate calm winds                       '
      ametdoc(k)='0               '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='IGFMET  '
      dmetdoc(k)='Do Not use coarse CALMET as Initial Guess           '
      ametdoc(k)='0               '

      k=k+1
      cmetdoc(k)='LVARY   '
      dmetdoc(k)='Do Not use varying radius of influence (winds)      '
      ametdoc(k)='F               '

      k=k+1
      cmetdoc(k)='NBAR    '
      dmetdoc(k)='Do Not use interpolation barriers (in general)      '
      ametdoc(k)='0               '

      k=k+1
      cmetdoc(k)='LLBREZE '
      dmetdoc(k)='Do Not use lake breeze model (in general)           '
      ametdoc(k)='F               '

      k=k+1
      cmetdoc(k)='IMIXH   '
      dmetdoc(k)='M-C or BG Conv. Mixing Ht. for land & water         '
      ametdoc(k)='1,2             '

      k=k+1
      cmetdoc(k)='ICOARE  '
      dmetdoc(k)='COARE overwater fluxes without wave adjustment      '
      ametdoc(k)='10              '

      k=k+1
      cmetdoc(k)='IWARM   '
      dmetdoc(k)='COARE warm layer SST adjustment Not computed        '
      ametdoc(k)='0               '

      k=k+1
      cmetdoc(k)='ICOOL   '
      dmetdoc(k)='COARE cool skin SST adjustment Not computed         '
      ametdoc(k)='0               '

      k=k+1
      cmetdoc(k)='IAVEZI  '
      dmetdoc(k)='Spatial averaging of mixing heights ON              '
      ametdoc(k)='1               '

      k=k+1
      cmetdoc(k)='IAVET   '
      dmetdoc(k)='Spatial averaging of temperatures ON                '
      ametdoc(k)='1               '

      k=k+1
      cmetdoc(k)='NFLAGP  '
      dmetdoc(k)='1/R**2 spatial precipitation interpolation          '
      ametdoc(k)='2               '

      k=k+1
      cmetdoc(k)='IDIOPT1 '
      dmetdoc(k)='Surface temperature Not from DIAG.DAT file          '
      ametdoc(k)='0               '
      kmetdoc(k)=1

      k=k+1
      cmetdoc(k)='ISURFT  '
      dmetdoc(k)='Surface temperature is 2D interpolated field        '
      ametdoc(k)='-1              '

      k=k+1
      cmetdoc(k)='IDIOPT2 '
      dmetdoc(k)='Temperature lapse rate Not from DIAG.DAT file       '
      ametdoc(k)='0               '
      kmetdoc(k)=1

      k=k+1
      cmetdoc(k)='IUPT    '
      dmetdoc(k)='Lapse rate is 2D interpolated field                 '
      ametdoc(k)='-1              '

      k=k+1
      cmetdoc(k)='IDIOPT3 '
      dmetdoc(k)='Initial guess windfield Not from DIAG.DAT file      '
      ametdoc(k)='0               '
      kmetdoc(k)=1

      k=k+1
      cmetdoc(k)='IUPWND  '
      dmetdoc(k)='Initial guess wind from 3D interpolated field       '
      ametdoc(k)='-1              '

      k=k+1
      cmetdoc(k)='IDIOPT4 '
      dmetdoc(k)='Observed surface winds Not from DIAG.DAT file       '
      ametdoc(k)='0               '
      kmetdoc(k)=1

      k=k+1
      cmetdoc(k)='IDIOPT5 '
      dmetdoc(k)='Observed upper winds Not from DIAG.DAT file         '
      ametdoc(k)='0               '
      kmetdoc(k)=1

      k=k+1
      cmetdoc(k)='IRTYPE  '
      dmetdoc(k)='Computes wind fields AND micromet. variables        '
      ametdoc(k)='1               '

      k=k+1
      cmetdoc(k)='LCALGRD '
      dmetdoc(k)='Create ALL fields needed for CALPUFF & CALGRID      '
      ametdoc(k)='T               '

c -----------------------------
c --- Model PARAMETERS
c -----------------------------

      k=k+1
      cmetdoc(k)='NZ      '
      dmetdoc(k)='Typically 10 to 12 vertical layers are used         '
      ametdoc(k)='None            '

      k=k+1
      cmetdoc(k)='ZFACE   '
      dmetdoc(k)='Interface hts (m) of layers; layer 1 starts at 0.0  '
      ametdoc(k)='None            '

      k=k+1
      cmetdoc(k)='BIAS    '
      dmetdoc(k)='0 all layers for standard weight of sfc and upr stn'
      ametdoc(k)='NZ*0            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='RMIN2   '
      dmetdoc(k)='Z-Extrapolate at all surface stations (no min)      '
      ametdoc(k)='-1.             '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='RMAX1   '
      dmetdoc(k)='MAX radius of influence over land in layer 1 (km)   '
      ametdoc(k)='None            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='RMAX2   '
      dmetdoc(k)='MAX radius of influence over land aloft (km)        '
      ametdoc(k)='None            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='RMAX3   '
      dmetdoc(k)='MAX radius of influence over water (km)             '
      ametdoc(k)='None            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='RMIN    '
      dmetdoc(k)='MIN radius of influence for wind interpolation (km) '
      ametdoc(k)='0.1             '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='TERRAD  '
      dmetdoc(k)='Terrain scale (km) for terrain effects              '
      ametdoc(k)='None            '

      k=k+1
      cmetdoc(k)='R1      '
      dmetdoc(k)='Distance (km) where OBS wt = IGF wt in layer 1      '
      ametdoc(k)='None            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='R2      '
      dmetdoc(k)='Distance (km) where OBS wt = IGF wt aloft           '
      ametdoc(k)='None            '
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='RPROG   '
      dmetdoc(k)='Distance (km) where OBS wt = PROGNOSTIC wt          '
      ametdoc(k)='0.              '
      kmetdoc(k)=1

      k=k+1
      cmetdoc(k)='DIVLIM  '
      dmetdoc(k)='MAX acceptable divergence in minimization           '
      ametdoc(k)='5.0E-06         '

      k=k+1
      cmetdoc(k)='NITER   '
      dmetdoc(k)='MAX # iterations in divergence minimization         '
      ametdoc(k)='50              '

      k=k+1
      cmetdoc(k)='NSMTH   '
      dmetdoc(k)='2 smoothing passes in layer 1; 4 in layers aloft    '
      ametdoc(k)='2,(MXNZ-1)*4    '     

      k=k+1
      cmetdoc(k)='NINTR2  '
      dmetdoc(k)='MAX # interpolation stations used = 99 in each layer'
      ametdoc(k)='NZ*99           '     

      k=k+1
      cmetdoc(k)='CRITFN  '
      dmetdoc(k)='Critical Froude Number for terrain channeling       '
      ametdoc(k)='1.              '     

      k=k+1
      cmetdoc(k)='ALPHA   '
      dmetdoc(k)='Empirical parameter for kinematic effects           '
      ametdoc(k)='0.1             '     

      k=k+1
      cmetdoc(k)='ZUPT    '
      dmetdoc(k)='Depth for computing domain-scale lapse rate (m)     '
      ametdoc(k)='200.            '     

      k=k+1
      cmetdoc(k)='ZUPWND  '
      dmetdoc(k)='Layer for computing domain-scale wind (m)           '
      ametdoc(k)='1.,1000.        '     

      k=k+1
      cmetdoc(k)='CONSTB  '
      dmetdoc(k)='Neutral, mechanical mixing ht. constant             '
      ametdoc(k)='1.41            '     

      k=k+1
      cmetdoc(k)='CONSTE  '
      dmetdoc(k)='Convective mixing ht. constant                      '
      ametdoc(k)='0.15            '     

      k=k+1
      cmetdoc(k)='CONSTN  '
      dmetdoc(k)='Stable mixing ht. constant                          '
      ametdoc(k)='2400.           '     

      k=k+1
      cmetdoc(k)='CONSTW  '
      dmetdoc(k)='Overwater mixing ht. constant                       '
      ametdoc(k)='0.16            '     

      k=k+1
      cmetdoc(k)='FCORIOL '
      dmetdoc(k)='Absolute value of Coriolis parameter (1/s)          '
      ametdoc(k)='1.0E-04         '     

      k=k+1
      cmetdoc(k)='MNMDAV  '
      dmetdoc(k)='Search radius for mixing ht averaging (cells)       '
      ametdoc(k)='1               '     

      k=k+1
      cmetdoc(k)='HAFANG  '
      dmetdoc(k)='Half-angle of upwind-looking cone (deg)             '
      ametdoc(k)='30              '     

      k=k+1
      cmetdoc(k)='ILEVZI  '
      dmetdoc(k)='Layer for wind in upwind averaging                  '
      ametdoc(k)='1               '     

      k=k+1
      cmetdoc(k)='THRESHL '
      dmetdoc(k)='Buoyancy flux for mixing ht growth (land) (W/m3)    '
      ametdoc(k)='0.              '     

      k=k+1
      cmetdoc(k)='THRESHW '
      dmetdoc(k)='Buoyancy flux for mixing ht growth (water) (W/m3)   '
      ametdoc(k)='.05             '     

      k=k+1
      cmetdoc(k)='ILUOC3D '
      dmetdoc(k)='Land use category for Ocean in 3D.DAT file          '
      ametdoc(k)='16,7            '     

      k=k+1
      cmetdoc(k)='DPTMIN  '
      dmetdoc(k)='MIN potential temp lapse rate above lid (K/m)       '
      ametdoc(k)='.001            '     

      k=k+1
      cmetdoc(k)='DZZI    '
      dmetdoc(k)='Layer above lid for computing lapse rate (m)        '
      ametdoc(k)='200.            '     

      k=k+1
      cmetdoc(k)='ZIMIN   '
      dmetdoc(k)='MIN over-land mixing height (m)                     '
      ametdoc(k)='50.             '     

      k=k+1
      cmetdoc(k)='ZIMAX   '
      dmetdoc(k)='MAX over-land mixing height (m)                     '
      ametdoc(k)='3000.           '     

      k=k+1
      cmetdoc(k)='ZIMINW  '
      dmetdoc(k)='MIN overwater mixing height (m)                     '
      ametdoc(k)='50.             '     

      k=k+1
      cmetdoc(k)='ZIMAXW  '
      dmetdoc(k)='MAX overwater mixing height (m)                     '
      ametdoc(k)='3000.           '     

      k=k+1
      cmetdoc(k)='DSHELF  '
      dmetdoc(k)='Coastal/shallow-water length scale for COARE (km)   '
      ametdoc(k)='0.              '     

      k=k+1
      cmetdoc(k)='IRAD    '
      dmetdoc(k)='1/R type of temperature interpolation               '
      ametdoc(k)='1               '     

      k=k+1
      cmetdoc(k)='TRADKM  '
      dmetdoc(k)='Radius of influence for temperature interp. (km)    '
      ametdoc(k)='500.            '     
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='NUMTS   '
      dmetdoc(k)='MAX # stations for temperature interpolation        '
      ametdoc(k)='5               '     
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='TGDEFB  '
      dmetdoc(k)='Default lapse rate below lid over water (K/m)     '
      ametdoc(k)='-.0098          '     

      k=k+1
      cmetdoc(k)='TGDEFA  '
      dmetdoc(k)='Default lapse rate above lid over water (K/m)     '
      ametdoc(k)='-.0045          '     

      k=k+1
      cmetdoc(k)='JWAT1   '
      dmetdoc(k)='Beginning LU for temperature interp over water      '
      ametdoc(k)='55              '     

      k=k+1
      cmetdoc(k)='JWAT2   '
      dmetdoc(k)='Ending LU for temperature interp over water         '
      ametdoc(k)='55              '     

      k=k+1
      cmetdoc(k)='SIGMAP  '
      dmetdoc(k)='Radius of influence for precipitation interp. (km)  '
      ametdoc(k)='100.            '     

      k=k+1
      cmetdoc(k)='CUTP    '
      dmetdoc(k)='MIN precipitation rate cut-off (mm/hr)              '
      ametdoc(k)='.01             '     

      k=k+1
      cmetdoc(k)='NSSTA   '
      dmetdoc(k)='Number of surface met stations                      '
      ametdoc(k)='None            '     
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='NUSTA   '
      dmetdoc(k)='Number of upper-air met stations                    '
      ametdoc(k)='None            '     
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='NOWSTA  '
      dmetdoc(k)='Number of overwater met stations                    '
      ametdoc(k)='None            '     
      if(noobs.EQ.2) fmetdoc(k)='<NA>'

      k=k+1
      cmetdoc(k)='NPSTA   '
      dmetdoc(k)='Number of precipitation stations                    '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='NM3D    '
      dmetdoc(k)='Number of prognostic data files                     '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='NIGF    '
      dmetdoc(k)='Number of CALMET data files for initial guess       '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='ISTEPPGS'
      dmetdoc(k)='Timestep in prognostic data files (s)              '
      ametdoc(k)='3600            '     

      k=k+1
      cmetdoc(k)='NX      '
      dmetdoc(k)='Number of EASTING grid cells                        '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='NY      '
      dmetdoc(k)='Number of NORTHING grid cells                       '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='DGRIDKM '
      dmetdoc(k)='Grid cell size (km)                                 '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='XORIGKM '
      dmetdoc(k)='X-coordinate of lower-left corner of grid (km)      '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='YORIGKM '
      dmetdoc(k)='Y-coordinate of lower-left corner of grid (km)      '
      ametdoc(k)='None            '     

      k=k+1
      cmetdoc(k)='KBAR    '
      dmetdoc(k)='Level to which barriers apply (typically NZ)        '
      ametdoc(k)='NZ              '     

      k=k+1
      cmetdoc(k)='FEXTR2  '
      dmetdoc(k)='NZ Factors to scale surface wind (seldom used)      '
      ametdoc(k)='NZ*0.0          '     
      if(iextrp.NE.3) fmetdoc(k)='<NA>'

      return
      end

c----------------------------------------------------------------------
      subroutine docmap
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317           DOCMAP
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113                 DOCMAP
c ---           D. Strimaitis
c
c --- PURPOSE:  Maps documentation array entries to model options
c
c --- INPUTS:
c     Common/DOCMET/
c        nmetdoc,cmetdoc
c     Common/OPTION/
c        nmetopt,cmetopt
c     Common/PARVAL/
c        nmetpvl,cmetpvl
c
c
c --- OUTPUT:
c
c     Common/OPTION/
c        moptdoc
c
c --- DOCMAP called by:  COMP_METCF
c --- DOCMAP calls:      none
c----------------------------------------------------------------------
      include 'cfdoc.ivu'

c --- Loop over model options
      do i=1,nmetopt
         moptdoc(i)=0
c ---    Search for match on current variable name
         do j=1,nmetdoc
            if(cmetopt(i).EQ.cmetdoc(j)) moptdoc(i)=j
         enddo
      enddo

c --- Loop over model parameters
      do i=1,nmetpvl
         mpvldoc(i)=0
c ---    Search for match on current variable name
         do j=1,nmetdoc
            if(cmetpvl(i).EQ.cmetdoc(j)) mpvldoc(i)=j
         enddo
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine rdfield(io,char320,klast,ieof)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          RDFIELD
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113                RDFIELD
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads one assignment field (!var = values!) into
c               string
c
c --- INPUTS:
c               IO - integer      - Input file unit number
c
c --- OUTPUT:
c          CHAR320 - char*132     - Assignment string
c            KLAST - integer      - Last position processed in string
c             IEOF - integer      - End-of-file flag
c                                   0: not at end     1: at end
c
c --- RDFIELD called by: RDPUFFCF, RDMETCF
c --- RDFIELD calls:     NOBLNK320, CASE320, MLINES320
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'err.cal'

      character*1 cm,xp,eq
      character*180 char180,blnk180
      character*320 char320,blnk320

      data cm/','/
      data xp/'!'/
      data eq/'='/

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

20    char180=blnk180
c --- Read record as text
      read(io,'(a180)',end=21) char180
      klast=LEN_TRIM(char180)

c --- Record must contain the start of an assignment
c --- with a delimiter ! and an = sign
      idelim1=INDEX(char180,xp)
      k=MAX(1,idelim1)
      iequals=INDEX(char180(k:klast),eq)
      if(idelim1.EQ.0 .OR. iequals.EQ.0) goto 20

c --- Create upper-case version of line without blanks
      char320=blnk320
      num=klast-idelim1+1
      char320(1:num)=char180(idelim1:klast)
      call NOBLNK320(char320)
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)
      if(iret.NE.0) return
c --- Set increments to delimiters and equals sign
      idelim1=INDEX(char320,xp)
      i1=idelim1+1
      iequals=INDEX(char320(i1:klast),eq)
      i2=idelim1+iequals+1
      idelim2=INDEX(char320(i2:klast),xp)
      i3=idelim1+iequals+idelim2

c --- Second delimiter on line?
      if(idelim2.EQ.0) then
c ---    Treat cases where assignment uses multiple lines
         call MLINES320(io,char320)
         if(iret.NE.0) return
c ---    Make sure that assignment string is complete now
         klast=LEN_TRIM(char320)
         idelim1=INDEX(char320,xp)
         i1=idelim1+1
         iequals=INDEX(char320(i1:klast),eq)
         i2=idelim1+iequals+1
         idelim2=INDEX(char320(i2:klast),xp)
         i3=idelim1+iequals+idelim2
         if(idelim1.EQ.0 .OR. iequals.EQ.0 .OR. idelim2.EQ.0) then
            write(iout,*)
            write(iout,*)'ERROR in RDFIELD:'
            write(iout,*)'Current assignment string is not correct'
            write(iout,*)char320(1:klast)
            iret=1
            return
         endif
      endif
c --- Last character of string that is needed
      klast=i3

      ieof=0
      return

21    ieof=1
      return

      end

c----------------------------------------------------------------------
      subroutine case320(mode,c320,nlim)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          CASE320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 1.3      Level: 060922                CASE320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert all letters within a character string to 
c               either upper or lower case
c
c --- INPUTS:
c
c             MODE - character*5       - Output directive for either 
c                                        upper or lower case
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c                                        uniform case
c
c --- CASE320 called by: (utility)
c --- CASE320 calls:      IACHAR, CHAR
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'err.cal'

      character*320 c320
      character*5 mode

      if(nlim.GT.320) then
         write(iout,*)'ERROR in Subr. CASE320 - string is too long'
         write(iout,*)'Maximum length  : 320'
         write(iout,*)'Requested length: ',nlim
         iret=1
         return
      endif

      if(mode.EQ.'UPPER' .OR. mode.EQ.'upper') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.97 .AND. ia.LE.122) then
               ia=ia-32
               c320(k:k)=CHAR(ia)
            endif
         enddo

      elseif(mode.EQ.'LOWER' .OR. mode.EQ.'lower') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.65 .AND. ia.LE.90) then
               ia=ia+32
               c320(k:k)=CHAR(ia)
            endif
         enddo
      else
         write(iout,*)'ERROR in Subr. CASE320 - invalid output mode'
         write(iout,*)'Expected UPPER or LOWER'
         write(iout,*)'Found: ',mode
         iret=1
         return
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine noblnk320(c320)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317        NOBLNK320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 1.3      Level: 060922              NOBLNK320
c ---           D. Strimaitis
c
c --- PURPOSE:  Remove all blanks from string (left-justify)
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c
c --- NOBLNK320 called by: (utility)
c --- NOBLNK320 calls:      ADJUSTL, LEN_TRIM
c----------------------------------------------------------------------

      character*320 c320,d320

c --- Blank out work string
      do i=1,320
         d320(i:i)=' '
      enddo

c --- Remove leading blanks
      c320=ADJUSTL(c320)
      last=LEN_TRIM(c320)

c --- Transfer non-blanks into work string
      n=0
      do i=1,last
         if(c320(i:i).NE.' ') then
            n=n+1
            d320(n:n)=c320(i:i)
         endif
      enddo

c --- Pass work string
      c320=d320

      return
      end

c----------------------------------------------------------------------
      subroutine COLUMN320(c320,nlim,cd,ncol,nc,ic)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090410        COLUMN320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 1.3      Level: 060922              COLUMN320
c ---           D. Strimaitis
c
c --- PURPOSE:  Identifies location of all column delimiters in a
c               string and checks for expected number
c
c --- UPDATES:
c --- V1.3 (L090317) to V1.4 (L090410) DGS
c            - Change error-check on expected number of columns to
c              warning, and pass back the number found
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c               CD - character*1       - Column delimiter character
c             NCOL - integer           - Number of columns expected
c
c --- OUTPUT:
c
c               NC - integer           - Number of delimiters
c       IC(0:NLIM) - integer           - Location of delimiters
c             NCOL - integer           - Number of columns found
c
c --- COLUMN320 called by: (utility)
c --- COLUMN320 calls:      none
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'err.cal'

      character*1 cd
      character*320 c320
      integer ic(0:ncol)

      if(nlim.GT.320) then
         write(iout,*)'ERROR in Subr. COLUMN320 - string is too long'
         write(iout,*)'Maximum length  : 320'
         write(iout,*)'Requested length: ',nlim
         iret=1
         return
      endif

      nc=0
      ic(0)=0
      do k=1,nlim
         if(c320(k:k).EQ.cd) then
            if(nc.LT.ncol) then
               nc=nc+1
               ic(nc)=k
            endif
         endif
      enddo
c --- Last delimiter in line is usually missing
      if(nc.LT.ncol) then
         nc=nc+1
         ic(nc)=nlim+1
      endif
c --- Check number of columns
      if(nc.LT.ncol) then
         write(iout,*)
         write(iout,*)'WARNING in Subr. COLUMN320 - '
         write(iout,*)'Too few columns in the line'
         write(iout,*)'Expected number of columns: ',ncol
         write(iout,*)'                     Found: ',nc
         write(iout,'(a)')c320(1:nlim)
         write(iout,*)'Zero is used for missing columns'
         write(iout,*)
         ncol=nc
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine IRANGE(n,ic,nc,i1,i2)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317           IRANGE
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 1.3      Level: 060922                 IRANGE
c ---           D. Strimaitis
c
c --- PURPOSE:  Extracts starting and ending position of field for a
c               particular column in a CSV line
c
c --- INPUTS:
c
c                N - integer           - Column number for field
c         IC(0:NC) - integer           - Location of commas
c               NC - integer           - Number of commas
c
c --- OUTPUT:
c
c               I1 - integer           - Starting position of field
c               I2 - integer           - Ending position of field
c
c --- IRANGE called by: (utility)
c --- IRANGE calls:      none
c----------------------------------------------------------------------

      integer ic(0:nc)

      i1=ic(n-1)+1
      i2=ic(n)-1

      return
      end

c----------------------------------------------------------------------
      subroutine rget320(c320,i1,i2,x)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          RGET320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 1.3      Level: 060922                RGET320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert character string to a real.  This sub is needed
c               when there is no decimal point.  The LF95 compiler
c               does not do the internal read conversion without the '.'
c               Also, determine when exponential notation is present.
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c               i1 - integer           - First character of substring
c               i2 - integer           - Last character of substring
c
c --- OUTPUT:
c
c                X - real              - Output real
c
c --- RGET320 called by: (utility)
c --- RGET320 calls:      none
c----------------------------------------------------------------------
      character*320 c320
      character*20 c20, blnk20
      data blnk20/'                    '/

      if(i1.GT.i2) return

      c20=blnk20
      c20=c320(i1:i2)

c --- Left justify field
      c20=ADJUSTL(c20)

c --- Test for exponential notation
      ie=MAX(INDEX(c20,'e'),INDEX(c20,'E'))

      if(ie.EQ.0) then
c ---    Simple real number
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            ipt=LEN_TRIM(c20)+1
            c20(ipt:ipt)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(f)') x

      else
c ---    Exponential notation
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            do k=20,ie+1,-1
               km1=k-1
               c20(k:k)=c20(km1:km1)
            enddo
            c20(ie:ie)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(e)') x

      endif

      return
      end

c----------------------------------------------------------------------
      subroutine mlines320(io,char320)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317        MLINES320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 081113              MLINES320
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads additional records from control file format
c               file when an input assignment extends to more than
c               one line
c
c --- INPUTS:
c               IO - integer      - File unit number
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- MLINES320 called by: RDFIELD
c --- MLINES320 calls:     NOBLNK320, CASE320
c----------------------------------------------------------------------
      include 'params.ivu'
      include 'err.cal'

      character*1 cm,xp
      character*180 char180,blnk180
      character*320 char320,blnk320,work320

      data cm/','/
      data xp/'!'/

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

c --- Remove all characters that precede the first '!'
      work320=blnk320
      work320=char320(INDEX(char320,xp):320)
      char320=ADJUSTL(work320)

10    klast=LEN_TRIM(char320)

c --- Check current assignment string to see if it is complete
      if(char320(klast:klast).EQ.xp) then
c ---    Assignment is complete
         return
c      elseif(char320(klast:klast).EQ.cm) then
      else
c ---    Append next line
         char180=blnk180
         read(io,'(a180)',end=999) char180
         work320=blnk320
         work320(1:180)=char180
         call NOBLNK320(work320)
         klast2=LEN_TRIM(work320)
         call CASE320('UPPER',work320,klast2)
         if(iret.NE.0) return
         i1=klast+1
         i2=klast+klast2
         if(i2.GT.320) then
            write(iout,*)'ERROR -- assignment string exceeds 320 ',
     &                'characters.  Total number of characters ',
     &                'reached: ',i2
            write(iout,*)char320
            write(iout,*)work320
            iret=1
            return
         endif
         char320(i1:i2)=work320(1:klast2)
         goto 10
c      else
c         write(*,*)'ERROR -- completed assignment string must end ',
c     &             'with ! and a continued assignment string must ',
c     &             'end with a comma'
c         write(*,*)char320
c         stop 'HALTED in subroutine MLINES320'
      endif

999   write(iout,*)'ERROR -- End-of-File encountered when completing ',
     &          'assignment string'
      write(iout,*)char320
      iret=1
      return

      end

c----------------------------------------------------------------------
      subroutine rhs320(char1,char320)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317           RHS320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 060922                 RHS320
c ---           D. Strimaitis
c
c --- PURPOSE:  Removes characters from the left-hand side of a
c               string up through "char1", and returns the RHS
c               that remains
c
c --- INPUTS:
c            CHAR1 - char*1       - LHS character limit
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- RHS320 called by: (main)
c --- RHS320 calls:     none
c----------------------------------------------------------------------
      character*1 char1
      character*320 char320,work320

      do i=1,320
         work320(i:i)=' '
      enddo

      i1=INDEX(char320,char1)+1
      i2=LEN_TRIM(char320)
      work320=char320(i1:i2)
      char320=ADJUSTL(work320)

      return
      end

c----------------------------------------------------------------------
      subroutine omit320(char1,char320)
c----------------------------------------------------------------------
c
c --- INFOVIEW  Version: 1.6            Level: 090317          OMIT320
c ---           D. Strimaitis
c
c --- Adapted From:
c --- CALEXAM   Version: 2.0      Level: 060922                OMIT320
c ---           D. Strimaitis
c
c --- PURPOSE:  Removes all instances of a character from a string
c
c --- INPUTS:
c            CHAR1 - char*1       - Character removed
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- OMIT320 called by: (main)
c --- OMIT320 calls:     none
c----------------------------------------------------------------------
      character*1 char1
      character*320 char320,work320

      do i=1,320
         work320(i:i)=' '
      enddo

      kk=0
      klast=LEN_TRIM(char320)
      do k=1,klast
         if(char320(k:k).NE.char1) then
            kk=kk+1
            work320(kk:kk)=char320(k:k)
         endif
      enddo
      char320=work320

      return
      end
